/**
 * @name DMS
 * @description Material Design 3 theme generated by DankMaterialShell
 * @author DankMaterialShell
 * @version 2.0.0
 * @website https://github.com/bbedward/DankMaterialShell
*/

* {
    font-family: 'Adwaita Sans', 'Inter Variable', -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji';

    --app-bg: {{colors.background.default.hex}};
    --background-primary: {{colors.background.default.hex}};
    --background-secondary: {{colors.background.default.hex}};
    --background-secondary-alt: {{colors.background.default.hex}};
    --background-tertiary: {{colors.background.default.hex}};

    --primary-500: {{colors.background.default.hex}};
    --primary-600: {{colors.background.default.hex}};
    --primary-630: {{colors.background.default.hex}};
    --primary-645: {{colors.background.default.hex}};
    --primary-660: {{colors.background.default.hex}};
    --primary-700: {{colors.background.default.hex}};

    --channeltextarea-background: {{colors.background.default.hex}};
    --custom-channel-members-bg: {{colors.background.default.hex}};

    --sidebar-background: {{colors.background.default.hex}};
    --sidebar-color: {{colors.background.default.hex}};
    --server-container: {{colors.background.default.hex}};
    --main-color: {{colors.background.default.hex}};
    --main-content-color: {{colors.background.default.hex}};

    --profile-gradient-primary-color: {{colors.background.default.hex}};
    --profile-gradient-secondary-color: {{colors.background.default.hex}};
    --profile-body-background-color: {{colors.background.default.hex}};
    --bg-overlay-1: {{colors.background.default.hex}};
    --bg-overlay-2: {{colors.background.default.hex}};
    --bg-overlay-3: {{colors.background.default.hex}};
    --home-background: {{colors.background.default.hex}};

    --__header-bar-background: {{colors.background.default.hex}} !important;
    --header-secondary: {{colors.background.default.hex}};
    --channel-header-bg: {{colors.background.default.hex}};
    --__channel-header-background: {{colors.background.default.hex}} !important;

    --bg-base-tertiary: {{colors.background.default.hex}};

    --card-primary-bg: {{colors.surface_container.default.hex}};
    --input-background: {{colors.surface_container.default.hex}};
    --autocomplete-bg: {{colors.surface_container.default.hex}};
    --background-nested-floating: {{colors.surface_container.default.hex}};
    --background-floating: {{colors.surface_container.default.hex}};
    --scrollbar-auto-track: {{colors.surface_container.default.hex}};
    --scrollbar-thin-track: {{colors.surface_container.default.hex}};

    --border-subtle: {{colors.background.default.hex}};
    --background-base-lowest: {{colors.background.default.hex}};
    --background-surface-high: {{colors.background.default.hex}};

    --button-secondary-background: {{colors.surface_container_high.default.hex}};
    --background-surface-higher: {{colors.surface_container_high.default.hex}};
    --background-base-lower: {{colors.surface_container_high.default.hex}};

    --background-message-hover: color-mix(in srgb, {{colors.on_surface.default.hex}} 4%, transparent);
    --button-secondary-background-hover: {{colors.surface_container_highest.default.hex}};
    --background-base-low: {{colors.surface_container_highest.default.hex}};
    --background-surface-highest: {{colors.surface_container_highest.default.hex}};
    --chat-background-default: {{colors.surface_container_highest.default.hex}};

    --button-secondary-background-active: color-mix(in srgb, {{colors.surface_container_highest.default.hex}}, {{colors.primary.default.hex}} 20%);

    --scrollbar-auto-thumb: {{colors.outline.default.hex}};
    --scrollbar-thin-thumb: {{colors.outline.default.hex}};
    --interactive-muted: {{colors.outline.default.hex}};
    --text-muted: {{colors.outline.default.hex}};
    --background-modifier-hover: {{colors.surface_container_high.default.hex}};
    --background-modifier-active: {{colors.surface_container_highest.default.hex}};
    --background-modifier-accent: color-mix(in srgb, {{colors.primary.default.hex}} 12%, transparent);
    --background-accent: color-mix(in srgb, {{colors.primary.default.hex}} 12%, transparent);
    --background-mentioned: color-mix(in srgb, {{colors.primary.default.hex}} 8%, transparent);
    --background-mentioned-hover: color-mix(in srgb, {{colors.primary.default.hex}} 12%, transparent);
    --mention-foreground: {{colors.primary.default.hex}};
    --info-warning-foreground: {{colors.primary.default.hex}};

    --input-border: {{colors.outline_variant.default.hex}};
    --border-normal: {{colors.outline_variant.default.hex}};
    --icon-secondary: {{colors.on_surface_variant.default.hex}};
    --icon-tertiary: {{colors.on_surface_variant.default.hex}};
    --channel-icon: {{colors.on_surface_variant.default.hex}};
    --channels-default: {{colors.on_surface_variant.default.hex}};
    --header-primary: {{colors.on_surface.default.hex}};
    --__lottieIconColor: {{colors.on_surface_variant.default.hex}};
    --interactive-normal: {{colors.on_surface_variant.default.hex}};

    --red-400: {{colors.error.default.hex}};
    --background-modifier-selected: color-mix(in srgb, {{colors.primary.default.hex}} 15%, transparent);

    --notice-background-positive: {{colors.primary_container.default.hex}};
    --notice-text-positive: {{colors.on_primary_container.default.hex}};

    --status-danger: {{colors.error.default.hex}};
    --button-outline-danger-border: {{colors.error.default.hex}};
    --button-outline-danger-text: {{colors.error.default.hex}};
    --button-danger-background: {{colors.error.default.hex}};

    --yellow-300: {{colors.tertiary.default.hex}};

    --brand-experiment: {{colors.primary.default.hex}};
    --brand-experiment-360: {{colors.primary.default.hex}};
    --brand-experiment-500: {{colors.primary.default.hex}};
    --profile-gradient-button-color: {{colors.primary.default.hex}};

    --green-360: {{colors.primary.default.hex}};

    --text-normal: {{colors.on_surface.default.hex}};
    --text-link: {{colors.primary.default.hex}};
    --interactive-active: {{colors.on_surface.default.hex}};
}

rect[fill="#d83a41"] {
    fill: {{colors.error.default.hex}} !important;
}

rect[fill="#cc954c"] {
    fill: {{colors.tertiary.default.hex}} !important;
}

rect[fill="#40a258"] {
    fill: {{colors.primary.default.hex}} !important;
}

[class*="repliedTextContent"],
[class*="repliedMessage"] [class*="username"],
[class*="repliedMessage"] [class*="repliedTextPreview"] {
    color: {{colors.on_surface_variant.default.hex}} !important;
    opacity: 0.9 !important;
}

.guilds__5e434,
.scroller_ef3116 {
    background: {{colors.surface.default.hex}} !important;
}

.sidebarList__5e434,
.container__2637a,
.scroller__629e4 {
    background: {{colors.surface.default.hex}} !important;
}

.container_c8ffbb,
.members_c8ffbb,
.member_c8ffbb,
.membersWrap_c8ffbb {
    background: {{colors.surface.default.hex}} !important;
    background-image: none !important;
}

.chatContent_f75fb0,
.messagesWrapper__36d07,
.chatGradient__36d07 {
    background: {{colors.surface.default.hex}} !important;
    background-image: none !important;
}

.subtitleContainer_f75fb0,
.title_f75fb0,
.container__9293f,
.headerBar__8a7fc,
.title_c38106 {
    background: {{colors.surface.default.hex}} !important;
}

.panels__5e434 {
    background: {{colors.surface_container.default.hex}} !important;
}

.channelTextArea_f75fb0,
.form_f75fb0,
.wrapper__44df5,
.channelBottomBarArea_f75fb0,
.attachWrapper__0923f,
.themedBackground__74017,
.stackedBars__74017 {
    background: {{colors.surface_container.default.hex}} !important;
    background-image: none !important;
}

.slateTextArea_ec4baf {
    background: {{colors.surface_container.default.hex}} !important;
    color: {{colors.on_surface.default.hex}} !important;
}

.slateTextArea_ec4baf [data-slate-placeholder],
.slateTextArea_ec4baf [data-slate-node],
.slateTextArea_ec4baf span[data-slate-zero-width],
[data-slate-placeholder],
[class*="placeholder"],
[class*="slateTextArea"] [class*="placeholder"] {
    color: {{colors.on_surface_variant.default.hex}} !important;
    opacity: 0.7 !important;
}

.message__5126c::before,
.mentioned__5126c::before,
.replying__5126c::before,
.ephemeral__5126c::before {
    background: none !important;
    background-image: none !important;
}

[class*="mention"][class*="channel"],
a[href*="/channels/"][class*="mention"] {
    color: {{colors.primary.default.hex}} !important;
    background: {{colors.surface_container_high.default.hex}} !important;
}

[class*="numberBadge"],
[class*="mentionsBadge"] {
    background-color: {{colors.primary.default.hex}} !important;
    color: {{colors.on_primary.default.hex}} !important;
}

.newMessagesBar__0f481 {
    background-color: {{colors.primary.default.hex}} !important;
    color: {{colors.on_primary.default.hex}} !important;
}

.divider__908e2 {
    border-color: {{colors.primary.default.hex}} !important;
}

[class*="channel"]:not([class*="muted"]) [class*="name"],
[class*="channel"]:not([class*="muted"]) [class*="link"] {
    color: {{colors.on_surface.default.hex}} !important;
}

[class*="channel"][class*="muted"] [class*="name"],
[class*="channel"][class*="muted"] [class*="link"] {
    color: {{colors.on_surface_variant.default.hex}} !important;
    opacity: 0.6 !important;
}
