/*
* GTK Colors
* Generated with Matugen
*/

/* Upstream adw-gtk3, I couldnt get light mode colors to work without it */

@define-color blue_1 #99c1f1;
@define-color blue_2 #62a0ea;
@define-color blue_3 #3584e4;
@define-color blue_4 #1c71d8;
@define-color blue_5 #1a5fb4;
@define-color green_1 #8ff0a4;
@define-color green_2 #57e389;
@define-color green_3 #33d17a;
@define-color green_4 #2ec27e;
@define-color green_5 #26a269;
@define-color yellow_1 #f9f06b;
@define-color yellow_2 #f8e45c;
@define-color yellow_3 #f6d32d;
@define-color yellow_4 #f5c211;
@define-color yellow_5 #e5a50a;
@define-color orange_1 #ffbe6f;
@define-color orange_2 #ffa348;
@define-color orange_3 #ff7800;
@define-color orange_4 #e66100;
@define-color orange_5 #c64600;
@define-color red_1 #f66151;
@define-color red_2 #ed333b;
@define-color red_3 #e01b24;
@define-color red_4 #c01c28;
@define-color red_5 #a51d2d;
@define-color purple_1 #dc8add;
@define-color purple_2 #c061cb;
@define-color purple_3 #9141ac;
@define-color purple_4 #813d9c;
@define-color purple_5 #613583;
@define-color brown_1 #cdab8f;
@define-color brown_2 #b5835a;
@define-color brown_3 #986a44;
@define-color brown_4 #865e3c;
@define-color brown_5 #63452c;
@define-color light_1 #ffffff;
@define-color light_2 #f6f5f4;
@define-color light_3 #deddda;
@define-color light_4 #c0bfbc;
@define-color light_5 #9a9996;
@define-color dark_1 #77767b;
@define-color dark_2 #5e5c64;
@define-color dark_3 #3d3846;
@define-color dark_4 #241f31;
@define-color dark_5 #000000;
@define-color accent_bg_color @blue_3;
@define-color accent_fg_color white;
@define-color destructive_bg_color @red_3;
@define-color destructive_fg_color white;
@define-color success_bg_color @green_4;
@define-color success_fg_color white;
@define-color warning_bg_color @yellow_5;
@define-color warning_fg_color rgba(0,0,0,0.8);
@define-color error_bg_color @red_3;
@define-color error_fg_color white;
@define-color accent_color mix(@accent_bg_color,black,0.17);
@define-color destructive_color mix(@destructive_bg_color,black,0.17);
@define-color success_color mix(@success_bg_color,black,0.17);
@define-color warning_color mix(@warning_bg_color,black,0.17);
@define-color error_color mix(@error_bg_color,black,0.17);
@define-color window_bg_color #fafafb;
@define-color window_fg_color #323233;
@define-color view_bg_color #ffffff;
@define-color view_fg_color #333334;
@define-color headerbar_bg_color #ebebed;
@define-color headerbar_fg_color #2f2f30;
@define-color headerbar_border_color rgba(0, 0, 6, 0.8);
@define-color headerbar_backdrop_color @window_bg_color;
@define-color headerbar_shade_color rgba(0, 0, 6, 0.12);
@define-color headerbar_darker_shade_color rgba(0, 0, 6, 0.12);
@define-color sidebar_bg_color #ebebed;
@define-color sidebar_fg_color #2f2f30;
@define-color sidebar_backdrop_color #f2f2f4;
@define-color sidebar_shade_color rgba(0, 0, 6, 0.07);
@define-color sidebar_border_color rgba(0, 0, 6, 0.07);
@define-color card_bg_color #ffffff;
@define-color card_fg_color #333334;
@define-color card_shade_color rgba(0, 0, 6, 0.07);
@define-color dialog_bg_color #fafafb;
@define-color dialog_fg_color #323233;
@define-color popover_bg_color #ffffff;
@define-color popover_fg_color #333334;
@define-color popover_shade_color rgba(0, 0, 6, 0.07);
@define-color thumbnail_bg_color #ffffff;
@define-color thumbnail_fg_color #333334;
@define-color shade_color rgba(0, 0, 6, 0.07);
@define-color scrollbar_outline_color white;
@define-color panel_bg_color black;
@define-color panel_fg_color white;
@define-color theme_fg_color @window_fg_color;
@define-color theme_text_color @view_fg_color;
@define-color theme_bg_color @window_bg_color;
@define-color theme_base_color @view_bg_color;
@define-color theme_selected_bg_color @accent_bg_color;
@define-color theme_selected_fg_color @accent_fg_color;
@define-color insensitive_bg_color mix(@window_bg_color,@view_bg_color,0.4);
@define-color insensitive_fg_color alpha(@window_fg_color,0.5);
@define-color insensitive_base_color @view_bg_color;
@define-color theme_unfocused_fg_color mix(@window_fg_color,@window_bg_color,0.5);
@define-color theme_unfocused_text_color @view_fg_color;
@define-color theme_unfocused_bg_color @window_bg_color;
@define-color theme_unfocused_base_color @window_bg_color;
@define-color theme_unfocused_selected_bg_color @accent_bg_color;
@define-color theme_unfocused_selected_fg_color @accent_fg_color;
@define-color unfocused_insensitive_color mix(mix(@window_fg_color,@window_bg_color,0.5),@window_bg_color,0.5);
@define-color borders mix(currentColor,@window_bg_color,0.85);
@define-color unfocused_borders mix(currentColor,@window_bg_color,0.81);
@define-color warning_color @warning_color;
@define-color error_color @error_color;
@define-color success_color @success_color;
@define-color wm_title shade(@window_fg_color, 1.8);
@define-color wm_unfocused_title mix(@window_fg_color,@window_bg_color,0.5);
@define-color wm_highlight alpha(white,0.5);
@define-color wm_borders_edge alpha(white,0.5);
@define-color wm_bg_a shade(@window_bg_color, 1.2);
@define-color wm_bg_b @window_bg_color;
@define-color wm_shadow alpha(black, 0.35);
@define-color wm_border alpha(black, 0.18);
@define-color wm_button_hover_color_a shade(@window_bg_color, 1.3);
@define-color wm_button_hover_color_b @window_bg_color;
@define-color wm_button_active_color_a shade(@window_bg_color, 0.85);
@define-color wm_button_active_color_b shade(@window_bg_color, 0.89);
@define-color wm_button_active_color_c shade(@window_bg_color, 0.9);
@define-color content_view_bg @view_bg_color;
@define-color text_view_bg @view_bg_color;
* {
  padding: 0;
  -GtkToolButton-icon-spacing: 4;
  -GtkTextView-error-underline-color: @error_bg_color;
  -GtkScrolledWindow-scrollbar-spacing: 0;
  -GtkToolItemGroup-expander-size: 11;
  -GtkWidget-text-handle-width: 20;
  -GtkWidget-text-handle-height: 24;
  -GtkDialog-button-spacing: 4;
  -GtkDialog-action-area-border: 0;
  outline-color: alpha(currentColor,0.3);
  outline-style: dashed;
  outline-offset: -3px;
  outline-width: 1px;
  -gtk-outline-radius: 7px;
  -gtk-secondary-caret-color: @accent_bg_color;
}

.background {
  color: @window_fg_color;
  background-color: @window_bg_color;
}
.background:backdrop {
  text-shadow: none;
  -gtk-icon-shadow: none;
}

*:disabled {
  -gtk-icon-effect: dim;
}

.gtkstyle-fallback {
  color: @window_fg_color;
  background-color: @window_bg_color;
}
.gtkstyle-fallback:hover {
  color: @window_fg_color;
  background-color: shade(@window_bg_color,1.1);
}
.gtkstyle-fallback:active {
  color: @window_fg_color;
  background-color: shade(@window_bg_color,0.9);
}
.gtkstyle-fallback:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_bg_color,@view_bg_color,0.4);
}
.gtkstyle-fallback:selected {
  color: @accent_fg_color;
  background-color: @accent_bg_color;
}

.osd .scale-popup, .app-notification,
.app-notification.frame, .csd popover.background.touch-selection, .csd popover.background.magnifier, popover.background.touch-selection, popover.background.magnifier, .csd popover.background.osd, popover.background.osd,
.osd {
  color: rgba(255, 255, 255, 0.9);
  border: none;
  background-color: rgba(0, 0, 0, 0.7);
  background-clip: padding-box;
}

treeview.view:selected:focus, treeview.view:selected, headerbar entry selection, .titlebar entry selection,
entry selection, spinbutton:not(.vertical) selection, flowbox flowboxchild:selected, .view text selection:focus, iconview text selection:focus, .view text selection, iconview text selection,
textview text selection:focus,
textview text selection, .view:selected, iconview:selected,
.view text:selected,
iconview text:selected,
textview text:selected {
  background-color: alpha(@accent_bg_color,0.25);
}
.selection-mode button.titlebutton, treeview.view:selected:focus, treeview.view:selected, headerbar entry selection, .titlebar entry selection,
entry selection, spinbutton:not(.vertical) selection, flowbox flowboxchild:selected, .view text selection:focus, iconview text selection:focus, .view text selection, iconview text selection,
textview text selection:focus,
textview text selection, .view:selected, iconview:selected,
.view text:selected,
iconview text:selected,
textview text:selected {
  color: @window_fg_color;
}
.view, iconview,
.view text,
iconview text,
textview text {
  color: @view_fg_color;
  background-color: @view_bg_color;
  caret-color: shade(@view_fg_color,1.05);
}
.view:disabled, iconview:disabled,
.view text:disabled,
iconview text:disabled,
textview text:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_bg_color,@view_bg_color,0.4);
}
.view:selected:focus, iconview:selected:focus, .view:selected, iconview:selected,
textview text:selected:focus,
.view text:selected,
iconview text:selected,
textview text:selected {
  border-radius: 9px;
}

textview border {
  background-color: mix(@window_bg_color,@view_bg_color,0.5);
}

.rubberband,
rubberband,
XfdesktopIconView.view .rubberband,
.content-view rubberband,
.content-view .rubberband,
treeview.view rubberband,
flowbox rubberband {
  border: 1px solid @accent_color;
  background-color: alpha(@accent_color,0.2);
}

flowbox flowboxchild {
  padding: 3px;
}
flowbox flowboxchild:selected {
  outline-offset: -2px;
}

.content-view .tile {
  margin: 2px;
  background-color: transparent;
  border-radius: 0;
  padding: 0;
}
.content-view .tile:backdrop {
  background-color: transparent;
}
.content-view .tile:active, .content-view .tile:selected {
  background-color: transparent;
}
.content-view .tile:disabled {
  background-color: transparent;
}

headerbar entry selection, .titlebar entry selection {
  background-color: alpha(@accent_bg_color,0.25);
}
.selection-mode button.titlebutton, label:selected, headerbar entry selection, .titlebar entry selection {
  color: @window_fg_color;
}
label {
  caret-color: currentColor;
}
label selection {
  background-color: @accent_bg_color;
  color: @accent_fg_color;
}
label:disabled {
  color: alpha(@window_fg_color,0.5);
}
button label:disabled {
  color: inherit;
}
label:disabled:backdrop {
  color: alpha(@window_fg_color,0.5);
}
button label:disabled:backdrop {
  color: inherit;
}
label.error {
  color: @error_bg_color;
}
label.error:disabled {
  color: alpha(@error_bg_color,0.5);
}
label.error:disabled:backdrop {
  color: alpha(@error_bg_color,0.4);
}

assistant.csd .sidebar {
  border-top-style: none;
}
assistant .sidebar label {
  padding: 6px 12px;
}
assistant .sidebar label.highlight {
  background-color: alpha(currentColor,0.1);
}

.dim-label, .welcome .dim-label,
headerbar .subtitle, .titlebar:not(headerbar) .subtitle, label.separator {
  opacity: 0.55;
  text-shadow: none;
}

.large-title {
  font-weight: 300;
  font-size: 24pt;
}

.title-1 {
  font-weight: 800;
  font-size: 20pt;
}

.title-2 {
  font-weight: 800;
  font-size: 15pt;
}

.title-3 {
  font-weight: 700;
  font-size: 15pt;
}

.title-4 {
  font-weight: 700;
  font-size: 13pt;
}

.heading {
  font-weight: 700;
  font-size: 11pt;
}

.body {
  font-weight: 400;
  font-size: 11pt;
}

.caption-heading {
  font-weight: 700;
  font-size: 9pt;
}

.caption {
  font-weight: 400;
  font-size: 9pt;
}

@keyframes spin {
  to {
    -gtk-icon-transform: rotate(1turn);
  }
}
spinner {
  background: none;
}
spinner:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
}
spinner {
  opacity: 0;
  -gtk-icon-source: -gtk-icontheme("process-working-symbolic");
}
spinner:checked {
  opacity: 1;
  animation: spin 1s linear infinite;
}
spinner:checked:disabled {
  opacity: 0.5;
}

@keyframes needs_attention {
  from {
    background-image: -gtk-gradient(radial, center center, 0, center center, 0.01, to(@accent_bg_color), to(transparent));
  }
  to {
    background-image: -gtk-gradient(radial, center center, 0, center center, 0.5, to(@accent_bg_color), to(transparent));
  }
}
notebook > header > tabs > arrow,
button {
  min-height: 24px;
  min-width: 16px;
  padding: 4px 10px;
  border: 1px solid;
  border-color: transparent;
  border-radius: 9px;
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  font-weight: bold;
  -gtk-outline-radius: 7px;
  color: @window_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.9);
}
notebook > header > tabs > arrow,
button {
  outline: 0 solid transparent;
  outline-offset: 4px;
  transition: outline-color 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), outline-width 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), outline-offset 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}
notebook > header > tabs > arrow:focus,
button:focus {
  outline-color: alpha(@accent_color,0.5);
  outline-width: 2px;
  outline-offset: -2px;
}
button.sidebar-button, notebook > header > tabs > arrow, notebook > header > tabs > arrow.flat,
button.flat {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  transition: none;
}
button.sidebar-button:hover, notebook > header > tabs > arrow:hover,
button.flat:hover {
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}
button.sidebar-button:hover:active, notebook > header > tabs > arrow:hover:active,
button.flat:hover:active {
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}

notebook > header > tabs > arrow:hover,
button:hover {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.85);
  box-shadow: none;
  -gtk-icon-effect: highlight;
}
notebook > header > tabs > arrow:active, notebook > header > tabs > arrow:checked,
button:active,
button:checked {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
notebook > header > tabs > arrow:active:hover, notebook > header > tabs > arrow:checked:hover,
button:active:hover,
button:checked:hover {
  background-color: mix(@window_fg_color,@window_bg_color,0.65);
}
notebook > header > tabs > arrow:active, notebook > header > tabs > arrow:checked,
button:active,
button:checked {
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}
notebook > header > tabs > arrow:active, notebook > header > tabs > arrow:checked,
button:active,
button:checked {
  outline: 0 solid transparent;
  outline-offset: 4px;
  transition: outline-color 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), outline-width 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), outline-offset 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}
notebook > header > tabs > arrow:active:focus, notebook > header > tabs > arrow:checked:focus,
button:active:focus,
button:checked:focus {
  outline-color: alpha(@accent_color,0.5);
  outline-width: 2px;
  outline-offset: -2px;
}
notebook > header > tabs > arrow:backdrop,
button:backdrop.flat,
button:backdrop {
  transition: 200ms ease-out;
  -gtk-icon-effect: none;
}
button.sidebar-button:backdrop, notebook > header > tabs > arrow:backdrop, button.sidebar-button:disabled, notebook > header > tabs > arrow:disabled,
button.flat:backdrop,
button.flat:disabled,
button.flat:backdrop:disabled {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}

notebook > header > tabs > arrow:disabled,
button:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
notebook > header > tabs > arrow:disabled:active, notebook > header > tabs > arrow:disabled:checked,
button:disabled:active,
button:disabled:checked {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
  box-shadow: none;
}
notebook > header > tabs > arrow.image-button,
button.image-button {
  min-width: 24px;
  padding-left: 4px;
  padding-right: 4px;
}
notebook > header > tabs > arrow.text-button,
button.text-button {
  padding-left: 10px;
  padding-right: 10px;
}
notebook > header > tabs > arrow.text-button.image-button,
button.text-button.image-button {
  padding-left: 4px;
  padding-right: 4px;
}
notebook > header > tabs > arrow.text-button.image-button label,
button.text-button.image-button label {
  padding-left: 4px;
  padding-right: 4px;
}
combobox:drop(active) button.combo, notebook > header > tabs > arrow:drop(active),
button:drop(active) {
  color: @accent_bg_color;
  border-color: @accent_bg_color;
  box-shadow: inset 0 0 0 1px @accent_bg_color;
}

row:selected button {
  border-color: mix(black,@accent_bg_color,0.85);
}
row:selected button.sidebar-button:not(:active):not(:checked):not(:hover):not(disabled),
row:selected button.flat:not(:active):not(:checked):not(:hover):not(disabled) {
  color: @window_fg_color;
  border-color: transparent;
}

button.osd {
  min-width: 32px;
  min-height: 32px;
}
button.osd.image-button {
  min-width: 34px;
}
button.osd {
  color: rgba(255, 255, 255, 0.9);
  border-radius: 5px;
  color: white;
  border-color: transparent;
  background-color: rgba(0, 0, 0, 0.65);
  background-clip: padding-box;
  border: none;
  box-shadow: none;
}
button.osd:hover {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.25);
  background-clip: padding-box;
  border: none;
  box-shadow: none;
}
button.osd:active,
button.osd:checked {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.2);
  background-clip: padding-box;
  border: none;
  box-shadow: none;
}
button.osd:disabled:backdrop,
button.osd:disabled {
  color: rgba(255, 255, 255, 0.5);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.5));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
  border: none;
}
button.osd:backdrop {
  color: rgba(255, 255, 255, 0.9);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.7));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
  border: none;
}
.app-notification button,
.app-notification.frame button, .csd popover.background.touch-selection button, .csd popover.background.magnifier button, popover.background.touch-selection button, popover.background.magnifier button,
.osd button {
  color: white;
  border-color: transparent;
  background-color: rgba(0, 0, 0, 0.65);
  background-clip: padding-box;
}
.app-notification button:hover, popover.background.touch-selection button:hover, popover.background.magnifier button:hover,
.osd button:hover {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.25);
  background-clip: padding-box;
}
.app-notification button:active, popover.background.touch-selection button:active, popover.background.magnifier button:active, .app-notification button:checked, popover.background.touch-selection button:checked, popover.background.magnifier button:checked,
.osd button:active:backdrop,
.osd button:active,
.osd button:checked:backdrop,
.osd button:checked {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.2);
  background-clip: padding-box;
}
.app-notification button:disabled, popover.background.touch-selection button:disabled, popover.background.magnifier button:disabled,
.osd button:disabled:backdrop,
.osd button:disabled {
  color: rgba(255, 255, 255, 0.5);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.5));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.app-notification button:backdrop, popover.background.touch-selection button:backdrop, popover.background.magnifier button:backdrop,
.osd button:backdrop {
  color: rgba(255, 255, 255, 0.9);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.7));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.app-notification button.flat, popover.background.touch-selection button.flat, popover.background.magnifier button.flat,
.osd button.flat {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
  text-shadow: 0 1px black;
  -gtk-icon-shadow: 0 1px black;
}
.app-notification button.flat:hover, popover.background.touch-selection button.flat:hover, popover.background.magnifier button.flat:hover,
.osd button.flat:hover {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.25);
  background-clip: padding-box;
}
.app-notification button.flat:disabled, popover.background.touch-selection button.flat:disabled, popover.background.magnifier button.flat:disabled,
.osd button.flat:disabled {
  color: rgba(255, 255, 255, 0.5);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.5));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
  background-image: none;
  border-color: transparent;
  box-shadow: none;
}
.app-notification button.flat:backdrop, popover.background.touch-selection button.flat:backdrop, popover.background.magnifier button.flat:backdrop,
.osd button.flat:backdrop {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.app-notification button.flat:active, popover.background.touch-selection button.flat:active, popover.background.magnifier button.flat:active, .app-notification button.flat:checked, popover.background.touch-selection button.flat:checked, popover.background.magnifier button.flat:checked,
.osd button.flat:active,
.osd button.flat:checked {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.2);
  background-clip: padding-box;
}

button.suggested-action {
  color: white;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(white,@accent_bg_color,0.9);
}
button.suggested-action.flat {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: @accent_bg_color;
}
button.suggested-action:hover {
  color: white;
  background-color: mix(white,@accent_bg_color,0.85);
  box-shadow: none;
}
button.suggested-action:active,
button.suggested-action:checked {
  color: white;
  background-color: mix(white,@accent_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
button.suggested-action:active:hover,
button.suggested-action:checked:hover {
  background-color: mix(white,@accent_bg_color,0.65);
}
button.suggested-action.flat:disabled {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: alpha(@accent_bg_color,0.8);
}
button.suggested-action:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
button.suggested-action:disabled:active,
button.suggested-action:disabled:checked {
  color: alpha(white,0.5);
  background-color: mix(white,@accent_bg_color,0.7);
  box-shadow: none;
}
.osd button.suggested-action {
  color: white;
  border-color: transparent;
  background-color: rgba(0, 0, 0, 0.65);
  background-clip: padding-box;
}
.osd button.suggested-action:hover {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.25);
  background-clip: padding-box;
}
.osd button.suggested-action:active:backdrop,
.osd button.suggested-action:active,
.osd button.suggested-action:checked:backdrop,
.osd button.suggested-action:checked {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.2);
  background-clip: padding-box;
}
.osd button.suggested-action:disabled:backdrop,
.osd button.suggested-action:disabled {
  color: rgba(255, 255, 255, 0.5);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.5));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.osd button.suggested-action:backdrop {
  color: rgba(255, 255, 255, 0.9);
  border-color: transparent;
  background-color: transparent;
  background-image: image(alpha(@accent_bg_color,0.5));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
button.destructive-action {
  color: @destructive_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@destructive_color,alpha(currentColor,0.15),0.9);
}
button.destructive-action.flat {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: @destructive_color;
}
button.destructive-action:hover {
  color: @destructive_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@destructive_color,alpha(currentColor,0.2),0.9);
}
button.destructive-action:focus {
  outline-color: @destructive_color;
}
button.destructive-action:active,
button.destructive-action:checked {
  color: @destructive_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@destructive_color,alpha(currentColor,0.35),0.9);
}
button.destructive-action:active:hover,
button.destructive-action:checked:hover {
  color: @destructive_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@destructive_color,alpha(currentColor,0.4),0.9);
}
button.destructive-action.flat:disabled {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: alpha(@destructive_bg_color,0.8);
}
button.destructive-action:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
button.destructive-action:disabled:active,
button.destructive-action:disabled:checked {
  color: alpha(@destructive_color,0.5);
  background-color: mix(@destructive_color,alpha(currentColor,0.15),0.7);
  box-shadow: none;
}
.osd button.destructive-action {
  color: white;
  border-color: transparent;
  background-color: rgba(0, 0, 0, 0.65);
  background-clip: padding-box;
}
.osd button.destructive-action:hover {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.25);
  background-clip: padding-box;
}
.osd button.destructive-action:active:backdrop,
.osd button.destructive-action:active,
.osd button.destructive-action:checked:backdrop,
.osd button.destructive-action:checked {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.2);
  background-clip: padding-box;
}
.osd button.destructive-action:disabled:backdrop,
.osd button.destructive-action:disabled {
  color: rgba(255, 255, 255, 0.5);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.5));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.osd button.destructive-action:backdrop {
  color: rgba(255, 255, 255, 0.9);
  border-color: transparent;
  background-color: transparent;
  background-image: image(alpha(alpha(currentColor,0.15),0.5));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.stack-switcher > button {
  outline-offset: -3px;
}
.stack-switcher > button > label {
  padding-left: 6px;
  padding-right: 6px;
}
.stack-switcher > button > image {
  padding-left: 6px;
  padding-right: 6px;
  padding-top: 3px;
  padding-bottom: 3px;
}
.stack-switcher > button.text-button {
  padding-left: 10px;
  padding-right: 10px;
}
.stack-switcher > button.image-button {
  padding-left: 2px;
  padding-right: 2px;
}
.stack-switcher > button.needs-attention:active > label,
.stack-switcher > button.needs-attention:active > image,
.stack-switcher > button.needs-attention:checked > label,
.stack-switcher > button.needs-attention:checked > image {
  animation: none;
  background-image: none;
}
button.font separator,
button.file separator {
  background-color: transparent;
}
button.font > box > box > label {
  font-weight: bold;
}
.primary-toolbar button {
  -gtk-icon-shadow: none;
}
button.circular {
  border-radius: 9999px;
  -gtk-outline-radius: 9999px;
  padding: 4px;
}
button.circular label {
  padding: 0;
}
button.circular:not(.flat):not(.osd):not(:checked):not(:active):not(:disabled):not(:backdrop) {
  color: @window_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.9);
  border-color: transparent;
}
button.circular:hover:not(.osd):not(:checked):not(:active):not(:disabled):not(:backdrop) {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.85);
  box-shadow: none;
  border-color: transparent;
}
button.circular {
  background-origin: padding-box, border-box;
  background-clip: padding-box, border-box;
}

stacksidebar row.needs-attention > label,
.stack-switcher > button.needs-attention > label,
.stack-switcher > button.needs-attention > image {
  animation: needs_attention 150ms ease-in;
  background-image: radial-gradient(farthest-side, @accent_color 96%, transparent);
  background-size: 6px 6px;
  background-repeat: no-repeat;
  background-position: right 3px;
}
stacksidebar row.needs-attention > label:dir(rtl),
.stack-switcher > button.needs-attention > label:dir(rtl),
.stack-switcher > button.needs-attention > image:dir(rtl) {
  background-position: left 3px;
}

.inline-toolbar toolbutton > button {
  color: @window_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.9);
}
.inline-toolbar toolbutton > button:hover {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.85);
  box-shadow: none;
}
.inline-toolbar toolbutton > button:active, .inline-toolbar toolbutton > button:checked {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.inline-toolbar toolbutton > button:active:hover, .inline-toolbar toolbutton > button:checked:hover {
  background-color: mix(@window_fg_color,@window_bg_color,0.65);
}
.inline-toolbar toolbutton > button:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
.inline-toolbar toolbutton > button:disabled:active, .inline-toolbar toolbutton > button:disabled:checked {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
  box-shadow: none;
}
.inline-toolbar toolbutton > button:backdrop {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.inline-toolbar toolbutton > button:backdrop:active, .inline-toolbar toolbutton > button:backdrop:checked {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
}
.inline-toolbar toolbutton > button:backdrop:active:hover, .inline-toolbar toolbutton > button:backdrop:checked:hover {
  background-color: mix(@window_fg_color,@window_bg_color,0.6);
}
.inline-toolbar toolbutton > button:backdrop:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.inline-toolbar toolbutton > button:backdrop:disabled:active, .inline-toolbar toolbutton > button:backdrop:disabled:checked {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
}

modelbutton.flat,
.menuitem.button.flat {
  min-height: 32px;
  padding-left: 5px;
  padding-right: 5px;
  border-radius: 9px;
  outline-offset: -2px;
}
modelbutton.flat:hover,
.menuitem.button.flat:hover {
  background-color: alpha(currentColor,0.1);
}
modelbutton.flat:selected,
.menuitem.button.flat:selected {
  background-color: alpha(currentColor,0.1);
}

modelbutton.flat arrow {
  background: none;
}
modelbutton.flat arrow:hover {
  background: none;
}
modelbutton.flat arrow.left {
  -gtk-icon-source: -gtk-icontheme("pan-start-symbolic");
}
modelbutton.flat arrow.right {
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
}

button.color {
  padding: 4px;
}
button.color colorswatch:only-child, button.color colorswatch:only-child overlay {
  border-radius: 0;
}
.osd button.color colorswatch:only-child {
  box-shadow: none;
}
.osd button.color:disabled colorswatch:only-child, .osd button.color:backdrop colorswatch:only-child, .osd button.color:active colorswatch:only-child, .osd button.color:checked colorswatch:only-child, button.color:disabled colorswatch:only-child, button.color:backdrop colorswatch:only-child, button.color:active colorswatch:only-child, button.color:checked colorswatch:only-child {
  box-shadow: none;
}

list row button.image-button:not(.flat):hover {
  background-color: alpha(currentColor,0.04);
}
list row button.image-button:not(.flat):active {
  background-color: alpha(currentColor,0.08);
}
list row button.image-button:not(.flat):checked {
  background-color: alpha(@accent_bg_color,0.25);
}
list row button.image-button:not(.flat):checked:hover {
  background-color: alpha(@accent_bg_color,0.32);
}
list row button.image-button:not(.flat):checked:active {
  background-color: alpha(@accent_bg_color,0.39);
}

button.toggle:not(.image-button) {
  padding-left: 8px;
  padding-right: 8px;
}

.osd .linked > button {
  border-color: transparent;
}

.linked:not(.vertical) > combobox > box > button.combo, filechooser .path-bar.linked > button, .xfce4-panel.background button menu .linked button, headerbar .linked.raised button:not(:only-child),
headerbar .linked button:not(:only-child), headerbar > .stack-switcher.linked > button:not(:only-child), .titlebar .linked.raised button:not(:only-child),
.titlebar .linked button:not(:only-child), .titlebar > .stack-switcher.linked > button:not(:only-child),
.linked:not(.vertical) > entry, .linked:not(.vertical) > spinbutton:not(.vertical),
.inline-toolbar button,
.linked > button, toolbar.inline-toolbar toolbutton > button.flat {
  border-left-color: alpha(currentColor,0.15);
  border-right-color: alpha(currentColor,0.15);
  border-right-style: none;
  border-radius: 0;
  -gtk-outline-radius: 0;
}

combobox.linked button:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box > button.combo, filechooser .path-bar.linked > button:dir(rtl):last-child, filechooser .path-bar.linked > button:dir(ltr):first-child, .xfce4-panel.background button menu .linked button:first-child,
headerbar .linked button:first-child:not(:only-child),
.titlebar .linked button:first-child:not(:only-child),
.linked:not(.vertical) > entry:first-child, .linked:not(.vertical) > spinbutton:first-child:not(.vertical),
.inline-toolbar button:first-child,
.linked > button:first-child, toolbar.inline-toolbar toolbutton:first-child > button.flat {
  border-left-color: transparent;
  border-top-left-radius: 9px;
  border-bottom-left-radius: 9px;
  -gtk-outline-top-left-radius: 9px;
  -gtk-outline-bottom-left-radius: 9px;
}
combobox.linked button:not(button):focus:nth-child(2):dir(rtl), .linked:not(.vertical) > combobox:first-child > box > button.combo:not(button):focus, filechooser .path-bar.linked > button:not(button):focus:dir(rtl):last-child, .xfce4-panel.background button menu .linked button:not(button):focus:first-child,
headerbar .linked button:not(button):focus:first-child:not(:only-child),
.titlebar .linked button:not(button):focus:first-child:not(:only-child),
.linked:not(.vertical) > entry:not(button):focus:first-child, .linked:not(.vertical) > spinbutton:not(button):focus:first-child:not(.vertical),
.inline-toolbar button:not(button):focus:first-child,
.linked > button:not(button):focus:first-child, toolbar.inline-toolbar toolbutton:first-child > button.flat:not(button):focus {
  border-left-color: alpha(@accent_color,0.5);
}

combobox.linked button:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box > button.combo, filechooser .path-bar.linked > button:dir(rtl):first-child, filechooser .path-bar.linked > button:dir(ltr):last-child, .xfce4-panel.background button menu .linked button:last-child,
headerbar .linked button:last-child:not(:only-child),
.titlebar .linked button:last-child:not(:only-child),
.linked:not(.vertical) > entry:last-child, .linked:not(.vertical) > spinbutton:last-child:not(.vertical),
.inline-toolbar button:last-child,
.linked > button:last-child, toolbar.inline-toolbar toolbutton:last-child > button.flat {
  border-right-color: transparent;
  border-right-style: solid;
  border-top-right-radius: 9px;
  border-bottom-right-radius: 9px;
  -gtk-outline-top-right-radius: 9px;
  -gtk-outline-bottom-right-radius: 9px;
}
combobox.linked button:not(button):focus:nth-child(2):dir(ltr), .linked:not(.vertical) > combobox:last-child > box > button.combo:not(button):focus, filechooser .path-bar.linked > button:not(button):focus:dir(rtl):first-child, .xfce4-panel.background button menu .linked button:not(button):focus:last-child,
headerbar .linked button:not(button):focus:last-child:not(:only-child),
.titlebar .linked button:not(button):focus:last-child:not(:only-child),
.linked:not(.vertical) > entry:not(button):focus:last-child, .linked:not(.vertical) > spinbutton:not(button):focus:last-child:not(.vertical),
.inline-toolbar button:not(button):focus:last-child,
.linked > button:not(button):focus:last-child, toolbar.inline-toolbar toolbutton:last-child > button.flat:not(button):focus {
  border-right-color: alpha(@accent_color,0.5);
}

.linked:not(.vertical) > combobox:only-child > box > button.combo, filechooser .path-bar.linked > button:only-child, .xfce4-panel.background button menu .linked button:only-child,
headerbar .linked button:only-child:not(:only-child),
.titlebar .linked button:only-child:not(:only-child),
.linked:not(.vertical) > entry:only-child, .linked:not(.vertical) > spinbutton:only-child:not(.vertical),
.inline-toolbar button:only-child,
.linked > button:only-child, toolbar.inline-toolbar toolbutton:only-child > button.flat {
  border-style: solid;
  border-radius: 9px;
  -gtk-outline-radius: 9px;
}

.xfce4-panel.background button menu .linked button, headerbar .linked.raised button:not(:only-child),
headerbar .linked button:not(:only-child), headerbar > .stack-switcher.linked > button:not(:only-child), .titlebar .linked.raised button:not(:only-child),
.titlebar .linked button:not(:only-child), .titlebar > .stack-switcher.linked > button:not(:only-child),
.linked:not(.vertical) > entry, .linked:not(.vertical) > spinbutton:not(.vertical),
.inline-toolbar button,
.linked > button {
  border-left-color: alpha(currentColor,0.15);
  border-right-color: alpha(currentColor,0.15);
}
.linked.vertical > combobox > box > button.combo,
.linked.vertical > entry, .linked.vertical > spinbutton:not(.vertical),
.linked.vertical > button {
  border-top-color: alpha(currentColor,0.15);
  border-bottom-color: alpha(currentColor,0.15);
  border-style: solid none none none;
  border-radius: 0;
  -gtk-outline-radius: 0;
}

.linked.vertical > combobox:first-child > box > button.combo,
.linked.vertical > entry:first-child, .linked.vertical > spinbutton:first-child:not(.vertical),
.linked.vertical > button:first-child {
  border-top-color: transparent;
  border-top-left-radius: 9px;
  border-top-right-radius: 9px;
  -gtk-outline-top-left-radius: 9px;
  -gtk-outline-top-right-radius: 9px;
}

.linked.vertical > combobox:last-child > box > button.combo,
.linked.vertical > entry:last-child, .linked.vertical > spinbutton:last-child:not(.vertical),
.linked.vertical > button:last-child {
  border-bottom-color: transparent;
  border-bottom-style: solid;
  border-bottom-left-radius: 9px;
  border-bottom-right-radius: 9px;
  -gtk-outline-bottom-left-radius: 9px;
  -gtk-outline-bottom-right-radius: 9px;
}

.linked.vertical > combobox:only-child > box > button.combo,
.linked.vertical > entry:only-child, .linked.vertical > spinbutton:only-child:not(.vertical),
.linked.vertical > button:only-child {
  border-style: solid;
  border-radius: 9px;
  -gtk-outline-radius: 9px;
}

.linked.vertical > entry, .linked.vertical > spinbutton:not(.vertical),
.linked.vertical > button {
  border-top-color: alpha(currentColor,0.15);
  border-bottom-color: alpha(currentColor,0.15);
}
.scale-popup button:hover, calendar.button, button:link,
button:visited, button:link:hover, button:link:active, button:link:checked,
button:visited:hover,
button:visited:active,
button:visited:checked, modelbutton.flat,
.menuitem.button.flat, list row button.image-button:not(.flat) {
  background-color: transparent;
  background-image: none;
  border-color: transparent;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0), 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}

messagedialog.csd.background button.text-button.suggested-action, .content list button.suggested-action, .content list > row:not(:selected):hover button.suggested-action,
list.content button.suggested-action,
list.content > row:not(:selected):hover button.suggested-action,
list.view.frame button.suggested-action,
list.view.frame > row:not(:selected):hover button.suggested-action, popover.background button.suggested-action, headerbar.selection-mode button, headerbar.selection-mode button.text-button:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar.selection-mode button, .titlebar.selection-mode button.text-button:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar button.suggested-action.text-button:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.suggested-action.text-button:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.suggested-action,
headerbar .linked button.suggested-action:not(:only-child), headerbar > .stack-switcher.linked > button.suggested-action, .titlebar .linked.raised button.suggested-action,
.titlebar .linked button.suggested-action:not(:only-child), .titlebar > .stack-switcher.linked > button.suggested-action, headerbar .linked entry ~ .suggested-action.image-button, .titlebar .linked entry ~ .suggested-action.image-button, headerbar button.suggested-action:not(.titlebutton), .titlebar button.suggested-action:not(.titlebutton), headerbar viewswitcher button.suggested-action.radio:checked, .titlebar viewswitcher button.suggested-action.radio:checked {
  color: @accent_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@accent_fg_color,@accent_bg_color,0.9);
}
messagedialog.csd.background button.text-button.suggested-action:hover, .content list button.suggested-action:hover, .content list > row:not(:selected):hover button.suggested-action:hover,
list.content button.suggested-action:hover,
list.content > row:not(:selected):hover button.suggested-action:hover,
list.view.frame button.suggested-action:hover,
list.view.frame > row:not(:selected):hover button.suggested-action:hover, popover.background button.suggested-action:hover, headerbar.selection-mode button:hover, headerbar.selection-mode button.text-button:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar.selection-mode button:hover, .titlebar.selection-mode button.text-button:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar button.suggested-action.text-button:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.suggested-action.text-button:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.suggested-action:hover,
headerbar .linked button.suggested-action:hover:not(:only-child), headerbar > .stack-switcher.linked > button.suggested-action:hover, .titlebar .linked.raised button.suggested-action:hover,
.titlebar .linked button.suggested-action:hover:not(:only-child), .titlebar > .stack-switcher.linked > button.suggested-action:hover, headerbar .linked entry ~ .suggested-action.image-button:hover, .titlebar .linked entry ~ .suggested-action.image-button:hover, headerbar button.suggested-action:hover:not(.titlebutton), .titlebar button.suggested-action:hover:not(.titlebutton), headerbar viewswitcher button.suggested-action.radio:hover:checked, .titlebar viewswitcher button.suggested-action.radio:hover:checked {
  color: @accent_fg_color;
  background-color: mix(@accent_fg_color,@accent_bg_color,0.85);
  box-shadow: none;
}
messagedialog.csd.background button.text-button.suggested-action:active, .content list button.suggested-action:active, .content list > row:not(:selected):hover button.suggested-action:active,
list.content button.suggested-action:active,
list.content > row:not(:selected):hover button.suggested-action:active,
list.view.frame button.suggested-action:active,
list.view.frame > row:not(:selected):hover button.suggested-action:active, popover.background button.suggested-action:active, headerbar.selection-mode button:active, headerbar.selection-mode button.text-button:active:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar.selection-mode button:active, .titlebar.selection-mode button.text-button:active:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar button.suggested-action.text-button:active:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.suggested-action.text-button:active:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.suggested-action:active,
headerbar .linked button.suggested-action:active:not(:only-child), headerbar > .stack-switcher.linked > button.suggested-action:active, .titlebar .linked.raised button.suggested-action:active,
.titlebar .linked button.suggested-action:active:not(:only-child), .titlebar > .stack-switcher.linked > button.suggested-action:active, headerbar .linked entry ~ .suggested-action.image-button:active, .titlebar .linked entry ~ .suggested-action.image-button:active, headerbar button.suggested-action:active:not(.titlebutton), .titlebar button.suggested-action:active:not(.titlebutton), messagedialog.csd.background button.text-button.suggested-action:checked, .content list button.suggested-action:checked, .content list > row:not(:selected):hover button.suggested-action:checked,
list.content button.suggested-action:checked,
list.content > row:not(:selected):hover button.suggested-action:checked,
list.view.frame button.suggested-action:checked,
list.view.frame > row:not(:selected):hover button.suggested-action:checked, popover.background button.suggested-action:checked, headerbar.selection-mode button:checked, headerbar.selection-mode button.text-button:checked:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar.selection-mode button:checked, .titlebar.selection-mode button.text-button:checked:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar button.suggested-action.text-button:checked:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.suggested-action.text-button:checked:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.suggested-action:checked,
headerbar .linked button.suggested-action:checked:not(:only-child), headerbar > .stack-switcher.linked > button.suggested-action:checked, .titlebar .linked.raised button.suggested-action:checked,
.titlebar .linked button.suggested-action:checked:not(:only-child), .titlebar > .stack-switcher.linked > button.suggested-action:checked, headerbar .linked entry ~ .suggested-action.image-button:checked, .titlebar .linked entry ~ .suggested-action.image-button:checked, headerbar button.suggested-action:checked:not(.titlebutton), .titlebar button.suggested-action:checked:not(.titlebutton), headerbar viewswitcher button.suggested-action.radio:checked, .titlebar viewswitcher button.suggested-action.radio:checked {
  color: @accent_fg_color;
  background-color: mix(@accent_fg_color,alpha(@accent_bg_color,0.5),0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
messagedialog.csd.background button.text-button.suggested-action:active:hover, .content list button.suggested-action:active:hover, .content list > row:not(:selected):hover button.suggested-action:active:hover,
list.content button.suggested-action:active:hover,
list.content > row:not(:selected):hover button.suggested-action:active:hover,
list.view.frame button.suggested-action:active:hover, popover.background button.suggested-action:active:hover, headerbar.selection-mode button:active:hover, headerbar.selection-mode button.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar.selection-mode button:active:hover, .titlebar.selection-mode button.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar button.suggested-action.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.suggested-action.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.suggested-action:active:hover,
headerbar .linked button.suggested-action:active:hover:not(:only-child), headerbar > .stack-switcher.linked > button.suggested-action:active:hover, .titlebar .linked.raised button.suggested-action:active:hover,
.titlebar .linked button.suggested-action:active:hover:not(:only-child), .titlebar > .stack-switcher.linked > button.suggested-action:active:hover, headerbar .linked entry ~ .suggested-action.image-button:active:hover, .titlebar .linked entry ~ .suggested-action.image-button:active:hover, headerbar button.suggested-action:active:hover:not(.titlebutton), .titlebar button.suggested-action:active:hover:not(.titlebutton), messagedialog.csd.background button.text-button.suggested-action:checked:hover, .content list button.suggested-action:checked:hover, .content list > row:not(:selected):hover button.suggested-action:checked:hover,
list.content button.suggested-action:checked:hover,
list.content > row:not(:selected):hover button.suggested-action:checked:hover,
list.view.frame button.suggested-action:checked:hover, popover.background button.suggested-action:checked:hover, headerbar.selection-mode button:checked:hover, headerbar.selection-mode button.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar.selection-mode button:checked:hover, .titlebar.selection-mode button.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar button.suggested-action.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.suggested-action.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.suggested-action:checked:hover,
headerbar .linked button.suggested-action:checked:hover:not(:only-child), headerbar > .stack-switcher.linked > button.suggested-action:checked:hover, .titlebar .linked.raised button.suggested-action:checked:hover,
.titlebar .linked button.suggested-action:checked:hover:not(:only-child), .titlebar > .stack-switcher.linked > button.suggested-action:checked:hover, headerbar .linked entry ~ .suggested-action.image-button:checked:hover, .titlebar .linked entry ~ .suggested-action.image-button:checked:hover, headerbar button.suggested-action:checked:hover:not(.titlebutton), .titlebar button.suggested-action:checked:hover:not(.titlebutton), headerbar viewswitcher button.suggested-action.radio:checked:hover, .titlebar viewswitcher button.suggested-action.radio:checked:hover {
  background-color: mix(@accent_fg_color,alpha(@accent_bg_color,0.5),0.65);
}
messagedialog.csd.background button.text-button.suggested-action:backdrop, .content list button.suggested-action:backdrop, .content list > row:not(:selected):hover button.suggested-action:backdrop,
list.content button.suggested-action:backdrop,
list.content > row:not(:selected):hover button.suggested-action:backdrop,
list.view.frame button.suggested-action:backdrop,
list.view.frame > row:not(:selected):hover button.suggested-action:backdrop, popover.background button.suggested-action:backdrop, headerbar.selection-mode button:backdrop, headerbar.selection-mode button.text-button:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar.selection-mode button:backdrop, .titlebar.selection-mode button.text-button:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar button.suggested-action.text-button:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.suggested-action.text-button:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.suggested-action:backdrop,
headerbar .linked button.suggested-action:backdrop:not(:only-child), headerbar > .stack-switcher.linked > button.suggested-action:backdrop, .titlebar .linked.raised button.suggested-action:backdrop,
.titlebar .linked button.suggested-action:backdrop:not(:only-child), .titlebar > .stack-switcher.linked > button.suggested-action:backdrop, headerbar .linked entry ~ .suggested-action.image-button:backdrop, .titlebar .linked entry ~ .suggested-action.image-button:backdrop, headerbar button.suggested-action:backdrop:not(.titlebutton), .titlebar button.suggested-action:backdrop:not(.titlebutton), headerbar viewswitcher button.suggested-action.radio:backdrop:checked, .titlebar viewswitcher button.suggested-action.radio:backdrop:checked {
  color: alpha(@accent_fg_color,0.5);
  background-color: mix(@accent_fg_color,alpha(@accent_bg_color,0.5),0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
messagedialog.csd.background button.text-button.suggested-action:disabled, .content list button.suggested-action:disabled, .content list > row:not(:selected):hover button.suggested-action:disabled,
list.content button.suggested-action:disabled,
list.content > row:not(:selected):hover button.suggested-action:disabled,
list.view.frame button.suggested-action:disabled,
list.view.frame > row:not(:selected):hover button.suggested-action:disabled, popover.background button.suggested-action:disabled, headerbar.selection-mode button:disabled, headerbar.selection-mode button.text-button:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar.selection-mode button:disabled, .titlebar.selection-mode button.text-button:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar button.suggested-action.text-button:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.suggested-action.text-button:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.suggested-action:disabled,
headerbar .linked button.suggested-action:disabled:not(:only-child), headerbar > .stack-switcher.linked > button.suggested-action:disabled, .titlebar .linked.raised button.suggested-action:disabled,
.titlebar .linked button.suggested-action:disabled:not(:only-child), .titlebar > .stack-switcher.linked > button.suggested-action:disabled, headerbar .linked entry ~ .suggested-action.image-button:disabled, .titlebar .linked entry ~ .suggested-action.image-button:disabled, headerbar button.suggested-action:disabled:not(.titlebutton), .titlebar button.suggested-action:disabled:not(.titlebutton), headerbar viewswitcher button.suggested-action.radio:disabled:checked, .titlebar viewswitcher button.suggested-action.radio:disabled:checked {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,alpha(@accent_bg_color,0.5),0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
messagedialog.csd.background button.text-button.suggested-action:disabled:backdrop, .content list button.suggested-action:disabled:backdrop, .content list > row:not(:selected):hover button.suggested-action:disabled:backdrop,
list.content button.suggested-action:disabled:backdrop,
list.content > row:not(:selected):hover button.suggested-action:disabled:backdrop,
list.view.frame button.suggested-action:disabled:backdrop, popover.background button.suggested-action:disabled:backdrop, headerbar.selection-mode button:disabled:backdrop, headerbar.selection-mode button.text-button:disabled:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar.selection-mode button:disabled:backdrop, .titlebar.selection-mode button.text-button:disabled:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar button.suggested-action.text-button:disabled:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.suggested-action.text-button:disabled:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.suggested-action:disabled:backdrop,
headerbar .linked button.suggested-action:disabled:backdrop:not(:only-child), headerbar > .stack-switcher.linked > button.suggested-action:disabled:backdrop, .titlebar .linked.raised button.suggested-action:disabled:backdrop,
.titlebar .linked button.suggested-action:disabled:backdrop:not(:only-child), .titlebar > .stack-switcher.linked > button.suggested-action:disabled:backdrop, headerbar .linked entry ~ .suggested-action.image-button:disabled:backdrop, .titlebar .linked entry ~ .suggested-action.image-button:disabled:backdrop, headerbar button.suggested-action:disabled:backdrop:not(.titlebutton), .titlebar button.suggested-action:disabled:backdrop:not(.titlebutton), headerbar viewswitcher button.suggested-action.radio:disabled:backdrop:checked, .titlebar viewswitcher button.suggested-action.radio:disabled:backdrop:checked {
  color: alpha(mix(mix(@window_fg_color,@window_bg_color,0.5),@window_bg_color,0.5),0.5);
  background-color: mix(mix(mix(@window_fg_color,@window_bg_color,0.5),@window_bg_color,0.5),alpha(@accent_bg_color,0.5),0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}

messagedialog.csd.background button.text-button.destructive-action, .content list button.destructive-action, .content list > row:not(:selected):hover button.destructive-action,
list.content button.destructive-action,
list.content > row:not(:selected):hover button.destructive-action,
list.view.frame button.destructive-action,
list.view.frame > row:not(:selected):hover button.destructive-action, popover.background button.destructive-action, headerbar button.destructive-action.text-button:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action,
headerbar .linked button.destructive-action:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action, .titlebar .linked.raised button.destructive-action,
.titlebar .linked button.destructive-action:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action, headerbar .linked entry ~ .destructive-action.image-button, .titlebar .linked entry ~ .destructive-action.image-button, headerbar button.destructive-action:not(.titlebutton), .titlebar button.destructive-action:not(.titlebutton), headerbar viewswitcher button.destructive-action.radio:checked, .titlebar viewswitcher button.destructive-action.radio:checked {
  color: @destructive_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@destructive_color,alpha(currentColor,0.15),0.9);
}
messagedialog.csd.background button.text-button.destructive-action:hover, .content list button.destructive-action:hover, .content list > row:not(:selected):hover button.destructive-action:hover,
list.content button.destructive-action:hover,
list.content > row:not(:selected):hover button.destructive-action:hover,
list.view.frame button.destructive-action:hover,
list.view.frame > row:not(:selected):hover button.destructive-action:hover, popover.background button.destructive-action:hover, headerbar button.destructive-action.text-button:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action:hover,
headerbar .linked button.destructive-action:hover:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action:hover, .titlebar .linked.raised button.destructive-action:hover,
.titlebar .linked button.destructive-action:hover:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action:hover, headerbar .linked entry ~ .destructive-action.image-button:hover, .titlebar .linked entry ~ .destructive-action.image-button:hover, headerbar button.destructive-action:hover:not(.titlebutton), .titlebar button.destructive-action:hover:not(.titlebutton), headerbar viewswitcher button.destructive-action.radio:hover:checked, .titlebar viewswitcher button.destructive-action.radio:hover:checked {
  color: @destructive_color;
  background-color: mix(@destructive_color,alpha(currentColor,0.2),0.85);
  box-shadow: none;
}
messagedialog.csd.background button.text-button.destructive-action:active, .content list button.destructive-action:active, .content list > row:not(:selected):hover button.destructive-action:active,
list.content button.destructive-action:active,
list.content > row:not(:selected):hover button.destructive-action:active,
list.view.frame button.destructive-action:active,
list.view.frame > row:not(:selected):hover button.destructive-action:active, popover.background button.destructive-action:active, headerbar button.destructive-action.text-button:active:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:active:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action:active,
headerbar .linked button.destructive-action:active:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action:active, .titlebar .linked.raised button.destructive-action:active,
.titlebar .linked button.destructive-action:active:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action:active, headerbar .linked entry ~ .destructive-action.image-button:active, .titlebar .linked entry ~ .destructive-action.image-button:active, headerbar button.destructive-action:active:not(.titlebutton), .titlebar button.destructive-action:active:not(.titlebutton), messagedialog.csd.background button.text-button.destructive-action:checked, .content list button.destructive-action:checked, .content list > row:not(:selected):hover button.destructive-action:checked,
list.content button.destructive-action:checked,
list.content > row:not(:selected):hover button.destructive-action:checked,
list.view.frame button.destructive-action:checked,
list.view.frame > row:not(:selected):hover button.destructive-action:checked, popover.background button.destructive-action:checked, headerbar button.destructive-action.text-button:checked:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:checked:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action:checked,
headerbar .linked button.destructive-action:checked:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action:checked, .titlebar .linked.raised button.destructive-action:checked,
.titlebar .linked button.destructive-action:checked:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action:checked, headerbar .linked entry ~ .destructive-action.image-button:checked, .titlebar .linked entry ~ .destructive-action.image-button:checked, headerbar button.destructive-action:checked:not(.titlebutton), .titlebar button.destructive-action:checked:not(.titlebutton), headerbar viewswitcher button.destructive-action.radio:checked, .titlebar viewswitcher button.destructive-action.radio:checked {
  color: @destructive_color;
  background-color: mix(@destructive_color,alpha(currentColor,0.35),0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
messagedialog.csd.background button.text-button.destructive-action:active:hover, .content list button.destructive-action:active:hover, .content list > row:not(:selected):hover button.destructive-action:active:hover,
list.content button.destructive-action:active:hover,
list.content > row:not(:selected):hover button.destructive-action:active:hover,
list.view.frame button.destructive-action:active:hover, popover.background button.destructive-action:active:hover, headerbar button.destructive-action.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action:active:hover,
headerbar .linked button.destructive-action:active:hover:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action:active:hover, .titlebar .linked.raised button.destructive-action:active:hover,
.titlebar .linked button.destructive-action:active:hover:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action:active:hover, headerbar .linked entry ~ .destructive-action.image-button:active:hover, .titlebar .linked entry ~ .destructive-action.image-button:active:hover, headerbar button.destructive-action:active:hover:not(.titlebutton), .titlebar button.destructive-action:active:hover:not(.titlebutton), messagedialog.csd.background button.text-button.destructive-action:checked:hover, .content list button.destructive-action:checked:hover, .content list > row:not(:selected):hover button.destructive-action:checked:hover,
list.content button.destructive-action:checked:hover,
list.content > row:not(:selected):hover button.destructive-action:checked:hover,
list.view.frame button.destructive-action:checked:hover, popover.background button.destructive-action:checked:hover, headerbar button.destructive-action.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action:checked:hover,
headerbar .linked button.destructive-action:checked:hover:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action:checked:hover, .titlebar .linked.raised button.destructive-action:checked:hover,
.titlebar .linked button.destructive-action:checked:hover:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action:checked:hover, headerbar .linked entry ~ .destructive-action.image-button:checked:hover, .titlebar .linked entry ~ .destructive-action.image-button:checked:hover, headerbar button.destructive-action:checked:hover:not(.titlebutton), .titlebar button.destructive-action:checked:hover:not(.titlebutton), headerbar viewswitcher button.destructive-action.radio:checked:hover, .titlebar viewswitcher button.destructive-action.radio:checked:hover {
  background-color: mix(@destructive_color,alpha(currentColor,0.35),0.65);
}
messagedialog.csd.background button.text-button.destructive-action:backdrop, .content list button.destructive-action:backdrop, .content list > row:not(:selected):hover button.destructive-action:backdrop,
list.content button.destructive-action:backdrop,
list.content > row:not(:selected):hover button.destructive-action:backdrop,
list.view.frame button.destructive-action:backdrop,
list.view.frame > row:not(:selected):hover button.destructive-action:backdrop, popover.background button.destructive-action:backdrop, headerbar button.destructive-action.text-button:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action:backdrop,
headerbar .linked button.destructive-action:backdrop:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action:backdrop, .titlebar .linked.raised button.destructive-action:backdrop,
.titlebar .linked button.destructive-action:backdrop:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action:backdrop, headerbar .linked entry ~ .destructive-action.image-button:backdrop, .titlebar .linked entry ~ .destructive-action.image-button:backdrop, headerbar button.destructive-action:backdrop:not(.titlebutton), .titlebar button.destructive-action:backdrop:not(.titlebutton), headerbar viewswitcher button.destructive-action.radio:backdrop:checked, .titlebar viewswitcher button.destructive-action.radio:backdrop:checked {
  color: alpha(@destructive_color,0.5);
  background-color: mix(@destructive_color,alpha(currentColor,0.15),0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
messagedialog.csd.background button.text-button.destructive-action:disabled, .content list button.destructive-action:disabled, .content list > row:not(:selected):hover button.destructive-action:disabled,
list.content button.destructive-action:disabled,
list.content > row:not(:selected):hover button.destructive-action:disabled,
list.view.frame button.destructive-action:disabled,
list.view.frame > row:not(:selected):hover button.destructive-action:disabled, popover.background button.destructive-action:disabled, headerbar button.destructive-action.text-button:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action:disabled,
headerbar .linked button.destructive-action:disabled:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action:disabled, .titlebar .linked.raised button.destructive-action:disabled,
.titlebar .linked button.destructive-action:disabled:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action:disabled, headerbar .linked entry ~ .destructive-action.image-button:disabled, .titlebar .linked entry ~ .destructive-action.image-button:disabled, headerbar button.destructive-action:disabled:not(.titlebutton), .titlebar button.destructive-action:disabled:not(.titlebutton), headerbar viewswitcher button.destructive-action.radio:disabled:checked, .titlebar viewswitcher button.destructive-action.radio:disabled:checked {
  color: alpha(@destructive_color,0.5);
  background-color: mix(@destructive_color,alpha(currentColor,0.15),0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
messagedialog.csd.background button.text-button.destructive-action:disabled:backdrop, .content list button.destructive-action:disabled:backdrop, .content list > row:not(:selected):hover button.destructive-action:disabled:backdrop,
list.content button.destructive-action:disabled:backdrop,
list.content > row:not(:selected):hover button.destructive-action:disabled:backdrop,
list.view.frame button.destructive-action:disabled:backdrop, popover.background button.destructive-action:disabled:backdrop, headerbar button.destructive-action.text-button:disabled:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:disabled:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action:disabled:backdrop,
headerbar .linked button.destructive-action:disabled:backdrop:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action:disabled:backdrop, .titlebar .linked.raised button.destructive-action:disabled:backdrop,
.titlebar .linked button.destructive-action:disabled:backdrop:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action:disabled:backdrop, headerbar .linked entry ~ .destructive-action.image-button:disabled:backdrop, .titlebar .linked entry ~ .destructive-action.image-button:disabled:backdrop, headerbar button.destructive-action:disabled:backdrop:not(.titlebutton), .titlebar button.destructive-action:disabled:backdrop:not(.titlebutton), headerbar viewswitcher button.destructive-action.radio:disabled:backdrop:checked, .titlebar viewswitcher button.destructive-action.radio:disabled:backdrop:checked {
  color: alpha(@destructive_color,0.5);
  background-color: mix(@destructive_color,alpha(currentColor,0.15),0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
messagedialog.csd.background button.text-button.destructive-action:focus, .content list button.destructive-action:focus, .content list > row:not(:selected):hover button.destructive-action:focus,
list.content button.destructive-action:focus,
list.content > row:not(:selected):hover button.destructive-action:focus,
list.view.frame button.destructive-action:focus,
list.view.frame > row:not(:selected):hover button.destructive-action:focus, popover.background button.destructive-action:focus, headerbar button.destructive-action.text-button:focus:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.destructive-action.text-button:focus:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.destructive-action:focus,
headerbar .linked button.destructive-action:focus:not(:only-child), headerbar > .stack-switcher.linked > button.destructive-action:focus, .titlebar .linked.raised button.destructive-action:focus,
.titlebar .linked button.destructive-action:focus:not(:only-child), .titlebar > .stack-switcher.linked > button.destructive-action:focus, headerbar .linked entry ~ .destructive-action.image-button:focus, .titlebar .linked entry ~ .destructive-action.image-button:focus, headerbar button.destructive-action:focus:not(.titlebutton), .titlebar button.destructive-action:focus:not(.titlebutton), headerbar viewswitcher button.destructive-action.radio:focus:checked, .titlebar viewswitcher button.destructive-action.radio:focus:checked {
  outline-color: @destructive_color;
}

spinbutton:not(.vertical),
entry {
  min-height: 34px;
  padding-left: 8px;
  padding-right: 8px;
  border: 1px solid;
  border-radius: 9px;
  transition: all 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  color: @view_fg_color;
  border-color: transparent;
  background-color: alpha(currentColor,0.1);
  box-shadow: inset 0 0 0 1px alpha(alpha(@accent_color,0.5),0);
}
spinbutton:not(.vertical) image.left,
entry image.left {
  margin-right: 6px;
}
spinbutton:not(.vertical) image.right,
entry image.right {
  margin-left: 6px;
}
spinbutton.flat:not(.vertical),
entry.flat:focus,
entry.flat:backdrop,
entry.flat:disabled,
entry.flat:backdrop:disabled,
entry.flat {
  min-height: 0;
  padding: 2px;
  background-color: transparent;
  border-color: transparent;
  border-radius: 0;
}
spinbutton:focus:not(.vertical),
entry:focus {
  box-shadow: inset 0 0 0 1px alpha(@accent_color,0.5);
  border-color: alpha(@accent_color,0.5);
  transition: 300ms ease-in-out;
  transition-property: border, box-shadow;
}
spinbutton:disabled:not(.vertical),
entry:disabled {
  color: alpha(@window_fg_color,0.5);
  border-color: transparent;
  background-color: mix(currentColor,@window_bg_color,0.95);
}
spinbutton.error:not(.vertical),
entry.error {
  color: @error_bg_color;
  border-color: @error_bg_color;
}
spinbutton.error:focus:not(.vertical),
entry.error:focus {
  box-shadow: inset 0 0 0 1px @error_bg_color;
  border-color: @error_bg_color;
  transition: 300ms ease-in-out;
  transition-property: border, box-shadow;
}
spinbutton.error:not(.vertical) selection,
entry.error selection {
  background-color: @error_bg_color;
}
spinbutton.warning:not(.vertical),
entry.warning {
  color: @warning_bg_color;
  border-color: @warning_bg_color;
}
spinbutton.warning:focus:not(.vertical),
entry.warning:focus {
  box-shadow: inset 0 0 0 1px @warning_bg_color;
  border-color: @warning_bg_color;
  transition: 300ms ease-in-out;
  transition-property: border, box-shadow;
}
spinbutton.warning:not(.vertical) selection,
entry.warning selection {
  background-color: @warning_bg_color;
}
spinbutton:not(.vertical) image,
entry image {
  color: mix(@window_fg_color,@view_bg_color,0.2);
}
spinbutton:not(.vertical) image:hover,
entry image:hover {
  color: @window_fg_color;
}
spinbutton:not(.vertical) image:active,
entry image:active {
  color: @accent_bg_color;
}
spinbutton:drop(active):not(.vertical),
entry:drop(active):focus,
entry:drop(active) {
  border-color: @accent_bg_color;
  box-shadow: inset 0 0 0 1px @accent_bg_color;
}
.osd spinbutton:not(.vertical),
.osd entry {
  color: white;
  border-color: rgba(0, 0, 0, 0.9);
  background-color: rgba(0, 0, 0, 0.5);
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: 0 1px black;
  -gtk-icon-shadow: 0 1px black;
}
.osd spinbutton:focus:not(.vertical),
.osd entry:focus {
  color: white;
  border-color: @accent_bg_color;
  background-color: rgba(0, 0, 0, 0.5);
  background-clip: padding-box;
  box-shadow: inset 0 0 0 1px alpha(@accent_color,0.5);
  text-shadow: 0 1px black;
  -gtk-icon-shadow: 0 1px black;
}
.osd spinbutton:backdrop:not(.vertical),
.osd entry:backdrop {
  color: white;
  border-color: rgba(0, 0, 0, 0.9);
  background-color: rgba(0, 0, 0, 0.5);
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.osd spinbutton:disabled:not(.vertical),
.osd entry:disabled {
  color: rgba(255, 255, 255, 0.5);
  border-color: rgba(0, 0, 0, 0.9);
  background-color: rgba(0, 0, 0, 0.5);
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
spinbutton:not(.vertical) progress,
entry progress {
  margin: 2px -6px;
  background-color: transparent;
  background-image: none;
  border-radius: 0;
  border-width: 0 0 2px;
  border-color: @accent_bg_color;
  border-style: solid;
  box-shadow: none;
}
.linked:not(.vertical) > spinbutton:focus:not(.vertical) + spinbutton:not(.vertical), .linked:not(.vertical) > spinbutton:focus:not(.vertical) + button, .linked:not(.vertical) > spinbutton:focus:not(.vertical) + combobox > box > button.combo,
.linked:not(.vertical) > spinbutton:focus:not(.vertical) + entry,
.linked:not(.vertical) > entry:focus + button,
.linked:not(.vertical) > entry:focus + combobox > box > button.combo,
.linked:not(.vertical) > entry:focus + spinbutton:not(.vertical),
.linked:not(.vertical) > entry:focus + entry {
  border-left-color: alpha(@accent_color,0.5);
}
.linked:not(.vertical) > spinbutton:focus.error:not(.vertical) + spinbutton:not(.vertical), .linked:not(.vertical) > spinbutton:focus.error:not(.vertical) + button, .linked:not(.vertical) > spinbutton:focus.error:not(.vertical) + combobox > box > button.combo,
.linked:not(.vertical) > spinbutton:focus.error:not(.vertical) + entry,
.linked:not(.vertical) > entry:focus.error + button,
.linked:not(.vertical) > entry:focus.error + combobox > box > button.combo,
.linked:not(.vertical) > entry:focus.error + spinbutton:not(.vertical),
.linked:not(.vertical) > entry:focus.error + entry {
  border-left-color: @error_bg_color;
}
.linked:not(.vertical) > spinbutton:drop(active):not(.vertical) + spinbutton:not(.vertical), .linked:not(.vertical) > spinbutton:drop(active):not(.vertical) + button, .linked:not(.vertical) > spinbutton:drop(active):not(.vertical) + combobox > box > button.combo,
.linked:not(.vertical) > spinbutton:drop(active):not(.vertical) + entry,
.linked:not(.vertical) > entry:drop(active) + button,
.linked:not(.vertical) > entry:drop(active) + combobox > box > button.combo,
.linked:not(.vertical) > entry:drop(active) + spinbutton:not(.vertical),
.linked:not(.vertical) > entry:drop(active) + entry {
  border-left-color: @accent_bg_color;
}
.linked.vertical > spinbutton:not(:disabled):not(.vertical) + entry:not(:disabled), .linked.vertical > spinbutton:not(:disabled):not(.vertical) + spinbutton:not(:disabled):not(.vertical),
.linked.vertical > entry:not(:disabled) + entry:not(:disabled),
.linked.vertical > entry:not(:disabled) + spinbutton:not(:disabled):not(.vertical) {
  border-top-color: mix(alpha(currentColor,0.15),@view_bg_color,0.7);
}
.linked.vertical > spinbutton:not(:disabled):not(.vertical) + entry:not(:disabled):backdrop, .linked.vertical > spinbutton:not(:disabled):not(.vertical) + spinbutton:not(:disabled):backdrop:not(.vertical),
.linked.vertical > entry:not(:disabled) + entry:not(:disabled):backdrop,
.linked.vertical > entry:not(:disabled) + spinbutton:not(:disabled):backdrop:not(.vertical) {
  border-top-color: mix(mix(currentColor,@window_bg_color,0.81),@window_bg_color,0.7);
}
.linked.vertical > spinbutton:disabled:not(.vertical) + spinbutton:disabled:not(.vertical), .linked.vertical > spinbutton:disabled:not(.vertical) + entry:disabled,
.linked.vertical > entry:disabled + spinbutton:disabled:not(.vertical),
.linked.vertical > entry:disabled + entry:disabled {
  border-top-color: mix(alpha(currentColor,0.15),@view_bg_color,0.7);
}
.linked.vertical > spinbutton:not(.vertical) + spinbutton:focus:not(:only-child):not(.vertical),
.linked.vertical > spinbutton:not(.vertical) + entry:focus:not(:only-child),
.linked.vertical > entry + spinbutton:focus:not(:only-child):not(.vertical),
.linked.vertical > entry + entry:focus:not(:only-child) {
  border-top-color: alpha(@accent_color,0.5);
}
.linked.vertical > spinbutton:not(.vertical) + spinbutton:focus.error:not(:only-child):not(.vertical),
.linked.vertical > spinbutton:not(.vertical) + entry:focus.error:not(:only-child),
.linked.vertical > entry + spinbutton:focus.error:not(:only-child):not(.vertical),
.linked.vertical > entry + entry:focus.error:not(:only-child) {
  border-top-color: @error_bg_color;
}
.linked.vertical > spinbutton:not(.vertical) + spinbutton:drop(active):not(:only-child):not(.vertical),
.linked.vertical > spinbutton:not(.vertical) + entry:drop(active):not(:only-child),
.linked.vertical > entry + spinbutton:drop(active):not(:only-child):not(.vertical),
.linked.vertical > entry + entry:drop(active):not(:only-child) {
  border-top-color: @accent_bg_color;
}
.linked.vertical > spinbutton:focus:not(:only-child):not(.vertical) + spinbutton:not(.vertical),
.linked.vertical > spinbutton:focus:not(:only-child):not(.vertical) + entry,
.linked.vertical > spinbutton:focus:not(:only-child):not(.vertical) + button,
.linked.vertical > spinbutton:focus:not(:only-child):not(.vertical) + combobox > box > button.combo,
.linked.vertical > entry:focus:not(:only-child) + spinbutton:not(.vertical),
.linked.vertical > entry:focus:not(:only-child) + entry,
.linked.vertical > entry:focus:not(:only-child) + button,
.linked.vertical > entry:focus:not(:only-child) + combobox > box > button.combo {
  border-top-color: alpha(@accent_color,0.5);
}
.linked.vertical > spinbutton:focus.error:not(:only-child):not(.vertical) + spinbutton:not(.vertical),
.linked.vertical > spinbutton:focus.error:not(:only-child):not(.vertical) + entry,
.linked.vertical > spinbutton:focus.error:not(:only-child):not(.vertical) + button,
.linked.vertical > spinbutton:focus.error:not(:only-child):not(.vertical) + combobox > box > button.combo,
.linked.vertical > entry:focus.error:not(:only-child) + spinbutton:not(.vertical),
.linked.vertical > entry:focus.error:not(:only-child) + entry,
.linked.vertical > entry:focus.error:not(:only-child) + button,
.linked.vertical > entry:focus.error:not(:only-child) + combobox > box > button.combo {
  border-top-color: @error_bg_color;
}
.linked.vertical > spinbutton:drop(active):not(:only-child):not(.vertical) + spinbutton:not(.vertical),
.linked.vertical > spinbutton:drop(active):not(:only-child):not(.vertical) + entry,
.linked.vertical > spinbutton:drop(active):not(:only-child):not(.vertical) + button,
.linked.vertical > spinbutton:drop(active):not(:only-child):not(.vertical) + combobox > box > button.combo,
.linked.vertical > entry:drop(active):not(:only-child) + spinbutton:not(.vertical),
.linked.vertical > entry:drop(active):not(:only-child) + entry,
.linked.vertical > entry:drop(active):not(:only-child) + button,
.linked.vertical > entry:drop(active):not(:only-child) + combobox > box > button.combo {
  border-top-color: @accent_bg_color;
}
spinbutton.error:not(.vertical),
entry.error {
  color: @error_bg_color;
}

treeview entry:focus:dir(rtl), treeview entry:focus:dir(ltr) {
  background-color: @view_bg_color;
  transition-property: color, background;
}
treeview entry.flat, treeview entry {
  border-radius: 0;
  background-image: none;
  background-color: @view_bg_color;
}
treeview entry.flat:focus, treeview entry:focus {
  border-color: @accent_bg_color;
}

.entry-tag {
  padding: 5px;
  margin-top: 2px;
  margin-bottom: 2px;
}
:dir(ltr) .entry-tag {
  margin-left: 8px;
  margin-right: -5px;
}
:dir(rtl) .entry-tag {
  margin-left: -5px;
  margin-right: 8px;
}
.entry-tag {
  border-style: none;
  color: @accent_fg_color;
  background-color: @accent_bg_color;
}
.entry-tag:hover {
  background-color: mix(white,@accent_bg_color,0.9);
}
:backdrop .entry-tag {
  color: @window_bg_color;
  background-color: @accent_bg_color;
}
.entry-tag.button {
  background-color: transparent;
  color: alpha(@accent_fg_color,0.7);
}
:not(:backdrop) .entry-tag.button:hover {
  border: 1px solid @accent_bg_color;
  color: @accent_fg_color;
}
:not(:backdrop) .entry-tag.button:active {
  background-color: @accent_bg_color;
  color: alpha(@accent_fg_color,0.7);
}

button:link > label,
button:visited > label, button:link,
button:visited,
*:link {
  color: @accent_color;
}
button:link > label:visited,
button:visited > label:visited,
button:visited,
*:link:visited {
  color: mix(@accent_color,@view_fg_color,0.2);
}
*:selected button:link > label:visited,
*:selected button:visited > label:visited,
*:selected button:visited,
*:selected *:link:visited {
  color: mix(@accent_fg_color,mix(@accent_color,@view_fg_color,0.2),0.4);
}
button:link > label:hover,
button:visited > label:hover,
button:hover:visited,
*:link:hover {
  color: mix(white,@accent_color,0.9);
}
*:selected button:link > label:hover,
*:selected button:visited > label:hover,
*:selected button:hover:visited,
*:selected *:link:hover {
  color: mix(@accent_fg_color,mix(white,@accent_color,0.9),0.1);
}
button:link > label:active,
button:visited > label:active,
button:active:visited,
*:link:active {
  color: @accent_color;
}
*:selected button:link > label:active,
*:selected button:visited > label:active,
*:selected button:active:visited,
*:selected *:link:active {
  color: mix(@accent_fg_color,@accent_color,0.2);
}
button:link > label:backdrop,
button:visited > label:backdrop,
button:backdrop:visited,
*:link:backdrop:backdrop:hover,
*:link:backdrop:backdrop:hover:selected,
*:link:backdrop {
  color: alpha(@accent_color,0.9);
}
button:link > label:disabled,
button:visited > label:disabled,
button:disabled:visited,
*:link:disabled,
*:link:disabled:backdrop {
  color: alpha(mix(white,black,0.5),0.8);
}
button:link > label:selected,
button:visited > label:selected,
button:selected:visited, *:selected button:link > label,
*:selected button:visited > label,
*:selected button:visited,
*:link:selected,
*:selected *:link {
  color: mix(@accent_fg_color,@accent_color,0.2);
}

button:link,
button:visited {
  text-shadow: none;
  font-weight: 400;
}
button:link:hover, button:link:active, button:link:checked,
button:visited:hover,
button:visited:active,
button:visited:checked {
  text-shadow: none;
}
button:link,
button:visited {
  outline: 0 solid transparent;
  outline-offset: 4px;
  transition: outline-color 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), outline-width 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), outline-offset 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}
button:link:focus,
button:visited:focus {
  outline-color: alpha(@accent_color,0.5);
  outline-width: 2px;
  outline-offset: -2px;
}
button:link > label,
button:visited > label {
  text-decoration-line: underline;
}

spinbutton:not(.vertical) {
  padding: 0;
}
.osd spinbutton:not(.vertical) entry, spinbutton:not(.vertical) entry {
  min-width: 28px;
  margin: 0;
  background: none;
  background-color: transparent;
  border: none;
  border-radius: 0;
  box-shadow: none;
}
spinbutton:not(.vertical) entry:backdrop:disabled {
  background-color: transparent;
}
spinbutton:not(.vertical) button {
  min-height: 16px;
  margin: 0;
  padding-bottom: 0;
  padding-top: 0;
  color: mix(@window_fg_color,@view_bg_color,0.1);
  background-color: transparent;
  border-style: none none none solid;
  border-color: alpha(currentColor,0.15);
  border-radius: 0;
  box-shadow: none;
}
spinbutton:not(.vertical) button:dir(rtl) {
  border-style: none solid none none;
}
spinbutton:not(.vertical) button:hover {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.85);
  box-shadow: none;
  background-color: alpha(currentColor,0.1);
}
spinbutton:not(.vertical) button:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
  background-color: transparent;
}
spinbutton:not(.vertical) button:active {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
spinbutton:not(.vertical) button:active:hover {
  background-color: mix(@window_fg_color,@window_bg_color,0.65);
}
spinbutton:not(.vertical) button:active {
  background-color: alpha(currentColor,0.15);
}
spinbutton:not(.vertical) button:dir(ltr):last-child {
  border-radius: 0 9px 9px 0;
}
spinbutton:not(.vertical) button:dir(rtl):first-child {
  border-radius: 9px 0 0 9px;
}
.osd spinbutton:not(.vertical) button {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: rgba(255, 255, 255, 0.9);
  border-style: none none none solid;
  border-color: alpha(rgba(0, 0, 0, 0.9),0.7);
  border-radius: 0;
  box-shadow: none;
  -gtk-icon-shadow: 0 1px black;
}
.osd spinbutton:not(.vertical) button:dir(rtl) {
  border-style: none solid none none;
}
.osd spinbutton:not(.vertical) button:hover {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: rgba(255, 255, 255, 0.9);
  border-color: alpha(black,0.5);
  background-color: alpha(rgba(255, 255, 255, 0.9),0.1);
  -gtk-icon-shadow: 0 1px black;
  box-shadow: none;
}
.osd spinbutton:not(.vertical) button:backdrop {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: rgba(255, 255, 255, 0.9);
  border-color: alpha(black,0.5);
  -gtk-icon-shadow: none;
  box-shadow: none;
}
.osd spinbutton:not(.vertical) button:disabled {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: rgba(255, 255, 255, 0.5);
  border-color: alpha(black,0.5);
  -gtk-icon-shadow: none;
  box-shadow: none;
}
.osd spinbutton:not(.vertical) button:dir(ltr):last-child {
  border-radius: 0 9px 9px 0;
}
.osd spinbutton:not(.vertical) button:dir(rtl):first-child {
  border-radius: 9px 0 0 9px;
}
spinbutton.vertical:disabled {
  color: alpha(@window_fg_color,0.5);
}
spinbutton.vertical:drop(active) {
  border-color: transparent;
  box-shadow: none;
}
spinbutton.vertical entry {
  min-height: 32px;
  min-width: 32px;
  padding: 0;
  border-radius: 0;
}
spinbutton.vertical button {
  min-height: 32px;
  min-width: 32px;
  padding: 0;
}
spinbutton.vertical button.up {
  border-color: alpha(currentColor,0.15);
  border-radius: 9px 9px 0 0;
  border-style: none none solid none;
}
spinbutton.vertical button.down {
  border-color: alpha(currentColor,0.15);
  border-radius: 0 0 9px 9px;
  border-style: solid none none none;
}
.osd spinbutton.vertical button:first-child {
  color: white;
  border-color: transparent;
  background-color: rgba(0, 0, 0, 0.65);
  background-clip: padding-box;
}
.osd spinbutton.vertical button:first-child:hover {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.25);
  background-clip: padding-box;
}
.osd spinbutton.vertical button:first-child:active {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.2);
  background-clip: padding-box;
}
.osd spinbutton.vertical button:first-child:disabled {
  color: rgba(255, 255, 255, 0.5);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.5));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.osd spinbutton.vertical button:first-child:backdrop {
  color: rgba(255, 255, 255, 0.9);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.7));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
treeview spinbutton:not(.vertical) {
  min-height: 0;
  border-style: none;
  border-radius: 0;
}
treeview spinbutton:not(.vertical) entry {
  min-height: 0;
  padding: 1px 2px;
}
spinbutton {
  font-feature-settings: "tnum";
}

combobox arrow {
  -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
  min-height: 16px;
  min-width: 16px;
}
combobox:drop(active) {
  box-shadow: none;
}
searchbar > revealer > box,
.location-bar, .inline-toolbar, toolbar {
  -GtkWidget-window-dragging: true;
  padding: 4px;
  background-color: @window_bg_color;
}
searchbar > revealer > box:backdrop,
.location-bar:backdrop, .inline-toolbar:backdrop, toolbar:backdrop {
  background-color: @window_bg_color;
}

toolbar {
  padding: 4px 3px 3px 4px;
}
.osd toolbar {
  background-color: transparent;
}
toolbar.osd {
  padding: 13px;
  border: none;
  border-radius: 5px;
  background-color: rgba(0, 0, 0, 0.7);
}
toolbar.osd.left, toolbar.osd.right, toolbar.osd.top, toolbar.osd.bottom {
  border-radius: 0;
}
toolbar.horizontal separator {
  margin: 0 7px 1px 6px;
}
toolbar.vertical separator {
  margin: 6px 1px 7px 0;
}
toolbar:not(.inline-toolbar):not(.osd) > *:not(.toggle):not(.popup) > * {
  margin-right: 1px;
  margin-bottom: 1px;
}

.inline-toolbar {
  background-color: @window_bg_color;
  border-color: alpha(currentColor,0.15);
  border-style: solid;
  padding: 3px;
  border-width: 0 1px 1px;
  border-radius: 0 0 5px 5px;
}

searchbar > revealer > box,
.location-bar {
  border-width: 0 0 1px;
  border-style: solid;
  border-color: alpha(currentColor,0.15);
  padding: 3px;
}

searchbar > revealer > box {
  margin: -6px;
  padding: 6px;
}

.primary-toolbar, .toolbar {
  background-color: @window_bg_color;
}
.primary-toolbar:backdrop, .toolbar:backdrop {
  background-color: @window_bg_color;
}

.titlebar:not(headerbar),
headerbar {
  padding: 0 6px;
  min-height: 46px;
  border-width: 0 0 1px;
  border-style: solid;
  border-color: @headerbar_shade_color;
  border-radius: 0;
  background: @headerbar_bg_color linear-gradient(to top, @headerbar_bg_color, @headerbar_bg_color);
}
.titlebar:backdrop:not(headerbar),
headerbar:backdrop {
  background-color: @headerbar_backdrop_color;
  background-image: none;
  transition: 200ms ease-out;
}
.titlebar:not(headerbar) .title,
headerbar .title {
  padding-left: 12px;
  padding-right: 12px;
  font-weight: bold;
}
.titlebar:not(headerbar) .subtitle,
headerbar .subtitle {
  font-size: smaller;
  padding-left: 12px;
  padding-right: 12px;
}
.titlebar:not(headerbar) stackswitcher button:checked:backdrop,
.titlebar:not(headerbar) button.toggle:checked:backdrop,
headerbar stackswitcher button:checked:backdrop,
headerbar button.toggle:checked:backdrop {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
}
.titlebar:not(headerbar) stackswitcher button:checked:backdrop:hover,
.titlebar:not(headerbar) button.toggle:checked:backdrop:hover,
headerbar stackswitcher button:checked:backdrop:hover,
headerbar button.toggle:checked:backdrop:hover {
  background-color: mix(@window_fg_color,@window_bg_color,0.6);
}
.selection-mode .titlebar:not(headerbar), .selection-mode.titlebar:not(headerbar),
.selection-mode headerbar,
headerbar.selection-mode {
  color: @accent_fg_color;
  border-color: mix(black,@accent_bg_color,0.85);
  background: @accent_bg_color linear-gradient(to top, @accent_bg_color, @accent_bg_color);
}
.selection-mode .titlebar:backdrop:not(headerbar), .selection-mode.titlebar:backdrop:not(headerbar),
.selection-mode headerbar:backdrop,
headerbar.selection-mode:backdrop {
  background-color: @accent_bg_color;
  background-image: none;
  box-shadow: inset 0 1px mix(alpha(white,0.5),@accent_bg_color,0.4);
}
.selection-mode .titlebar:backdrop:not(headerbar) label, .selection-mode.titlebar:backdrop:not(headerbar) label,
.selection-mode headerbar:backdrop label,
headerbar.selection-mode:backdrop label {
  text-shadow: none;
  color: @accent_fg_color;
}
.selection-mode .titlebar:not(headerbar) .subtitle:link, .selection-mode.titlebar:not(headerbar) .subtitle:link,
.selection-mode headerbar .subtitle:link,
headerbar.selection-mode .subtitle:link {
  color: mix(@accent_fg_color,@accent_color,0.2);
}
.selection-mode .titlebar:not(headerbar) button, .selection-mode.titlebar:not(headerbar) button,
.selection-mode headerbar button,
headerbar.selection-mode button {
  color: @accent_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@accent_fg_color,@accent_bg_color,0.9);
}
.selection-mode .titlebar:not(headerbar) button.flat, .selection-mode.titlebar:not(headerbar) button.flat,
.selection-mode headerbar button.flat,
headerbar.selection-mode button.flat {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}

.selection-mode .titlebar:not(headerbar) button:hover, .selection-mode.titlebar:not(headerbar) button:hover,
.selection-mode headerbar button:hover,
headerbar.selection-mode button:hover {
  color: @accent_fg_color;
  background-color: mix(@accent_fg_color,@accent_bg_color,0.85);
  box-shadow: none;
}
.selection-mode .titlebar:not(headerbar) button:active, .selection-mode .titlebar:not(headerbar) button:checked, .selection-mode.titlebar:not(headerbar) button:active, .selection-mode.titlebar:not(headerbar) button:checked,
.selection-mode headerbar button:active,
.selection-mode headerbar button:checked,
.selection-mode headerbar button.toggle:checked,
.selection-mode headerbar button.toggle:active,
headerbar.selection-mode button:active,
headerbar.selection-mode button:checked,
headerbar.selection-mode button.toggle:checked,
headerbar.selection-mode button.toggle:active {
  color: @accent_fg_color;
  background-color: mix(@accent_fg_color,@accent_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.selection-mode .titlebar:not(headerbar) button:active:hover, .selection-mode .titlebar:not(headerbar) button:checked:hover, .selection-mode.titlebar:not(headerbar) button:active:hover, .selection-mode.titlebar:not(headerbar) button:checked:hover,
.selection-mode headerbar button:active:hover,
.selection-mode headerbar button:checked:hover,
.selection-mode headerbar button.toggle:checked:hover,
.selection-mode headerbar button.toggle:active:hover,
headerbar.selection-mode button:active:hover,
headerbar.selection-mode button:checked:hover,
headerbar.selection-mode button.toggle:checked:hover,
headerbar.selection-mode button.toggle:active:hover {
  background-color: mix(@accent_fg_color,@accent_bg_color,0.65);
}
.selection-mode .titlebar:not(headerbar) button:backdrop, .selection-mode.titlebar:not(headerbar) button:backdrop,
.selection-mode headerbar button:backdrop.flat,
.selection-mode headerbar button:backdrop,
headerbar.selection-mode button:backdrop.flat,
headerbar.selection-mode button:backdrop {
  color: alpha(@accent_fg_color,0.5);
  background-color: mix(@accent_fg_color,@accent_bg_color,0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
  -gtk-icon-effect: none;
  border-color: transparent;
}
.selection-mode .titlebar:not(headerbar) button:backdrop:active, .selection-mode .titlebar:not(headerbar) button:backdrop:checked, .selection-mode.titlebar:not(headerbar) button:backdrop:active, .selection-mode.titlebar:not(headerbar) button:backdrop:checked,
.selection-mode headerbar button:backdrop.flat:active,
.selection-mode headerbar button:backdrop.flat:checked,
.selection-mode headerbar button:backdrop:active,
.selection-mode headerbar button:backdrop:checked,
headerbar.selection-mode button:backdrop.flat:active,
headerbar.selection-mode button:backdrop.flat:checked,
headerbar.selection-mode button:backdrop:active,
headerbar.selection-mode button:backdrop:checked {
  color: alpha(@accent_fg_color,0.5);
  background-color: mix(@accent_fg_color,@accent_bg_color,0.7);
}
.selection-mode .titlebar:not(headerbar) button:backdrop:active:hover, .selection-mode .titlebar:not(headerbar) button:backdrop:checked:hover, .selection-mode.titlebar:not(headerbar) button:backdrop:active:hover, .selection-mode.titlebar:not(headerbar) button:backdrop:checked:hover,
.selection-mode headerbar button:backdrop.flat:active:hover,
.selection-mode headerbar button:backdrop.flat:checked:hover,
.selection-mode headerbar button:backdrop:active:hover,
.selection-mode headerbar button:backdrop:checked:hover,
headerbar.selection-mode button:backdrop.flat:active:hover,
headerbar.selection-mode button:backdrop.flat:checked:hover,
headerbar.selection-mode button:backdrop:active:hover,
headerbar.selection-mode button:backdrop:checked:hover {
  background-color: mix(@accent_fg_color,@accent_bg_color,0.6);
}
.selection-mode .titlebar:not(headerbar) button:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled,
.selection-mode headerbar button:backdrop.flat:disabled,
.selection-mode headerbar button:backdrop:disabled,
headerbar.selection-mode button:backdrop.flat:disabled,
headerbar.selection-mode button:backdrop:disabled {
  color: alpha(@accent_fg_color,0.5);
  background-color: mix(@accent_fg_color,@accent_bg_color,0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.selection-mode .titlebar:not(headerbar) button:backdrop:disabled:active, .selection-mode .titlebar:not(headerbar) button:backdrop:disabled:checked, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled:active, .selection-mode.titlebar:not(headerbar) button:backdrop:disabled:checked,
.selection-mode headerbar button:backdrop.flat:disabled:active,
.selection-mode headerbar button:backdrop.flat:disabled:checked,
.selection-mode headerbar button:backdrop:disabled:active,
.selection-mode headerbar button:backdrop:disabled:checked,
headerbar.selection-mode button:backdrop.flat:disabled:active,
headerbar.selection-mode button:backdrop.flat:disabled:checked,
headerbar.selection-mode button:backdrop:disabled:active,
headerbar.selection-mode button:backdrop:disabled:checked {
  color: alpha(@accent_fg_color,0.5);
  background-color: mix(@accent_fg_color,@accent_bg_color,0.7);
}
.selection-mode .titlebar:not(headerbar) button.flat:backdrop, .selection-mode .titlebar:not(headerbar) button.flat:disabled, .selection-mode.titlebar:not(headerbar) button.flat:backdrop, .selection-mode.titlebar:not(headerbar) button.flat:disabled,
.selection-mode headerbar button.flat:backdrop,
.selection-mode headerbar button.flat:disabled,
.selection-mode headerbar button.flat:backdrop:disabled,
headerbar.selection-mode button.flat:backdrop,
headerbar.selection-mode button.flat:disabled,
headerbar.selection-mode button.flat:backdrop:disabled {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}

.selection-mode .titlebar:not(headerbar) button:disabled, .selection-mode.titlebar:not(headerbar) button:disabled,
.selection-mode headerbar button:disabled,
headerbar.selection-mode button:disabled {
  color: alpha(@accent_fg_color,0.5);
  background-color: mix(@accent_fg_color,@accent_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
.selection-mode .titlebar:not(headerbar) button:disabled:active, .selection-mode .titlebar:not(headerbar) button:disabled:checked, .selection-mode.titlebar:not(headerbar) button:disabled:active, .selection-mode.titlebar:not(headerbar) button:disabled:checked,
.selection-mode headerbar button:disabled:active,
.selection-mode headerbar button:disabled:checked,
headerbar.selection-mode button:disabled:active,
headerbar.selection-mode button:disabled:checked {
  color: alpha(@accent_fg_color,0.5);
  background-color: mix(@accent_fg_color,@accent_bg_color,0.7);
  box-shadow: none;
}
.selection-mode .titlebar:not(headerbar) button.suggested-action, .selection-mode.titlebar:not(headerbar) button.suggested-action,
.selection-mode headerbar button.suggested-action,
headerbar.selection-mode button.suggested-action {
  color: @window_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.9);
  border-color: mix(black,@accent_bg_color,0.85);
}
.selection-mode .titlebar:not(headerbar) button.suggested-action:hover, .selection-mode.titlebar:not(headerbar) button.suggested-action:hover,
.selection-mode headerbar button.suggested-action:hover,
headerbar.selection-mode button.suggested-action:hover {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.85);
  box-shadow: none;
}
.selection-mode .titlebar:not(headerbar) button.suggested-action:active, .selection-mode.titlebar:not(headerbar) button.suggested-action:active,
.selection-mode headerbar button.suggested-action:active,
headerbar.selection-mode button.suggested-action:active {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.selection-mode .titlebar:not(headerbar) button.suggested-action:active:hover, .selection-mode.titlebar:not(headerbar) button.suggested-action:active:hover,
.selection-mode headerbar button.suggested-action:active:hover,
headerbar.selection-mode button.suggested-action:active:hover {
  background-color: mix(@window_fg_color,@window_bg_color,0.65);
}
.selection-mode .titlebar:not(headerbar) button.suggested-action:disabled, .selection-mode.titlebar:not(headerbar) button.suggested-action:disabled,
.selection-mode headerbar button.suggested-action:disabled,
headerbar.selection-mode button.suggested-action:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
.selection-mode .titlebar:not(headerbar) button.suggested-action:backdrop, .selection-mode.titlebar:not(headerbar) button.suggested-action:backdrop,
.selection-mode headerbar button.suggested-action:backdrop,
headerbar.selection-mode button.suggested-action:backdrop {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.selection-mode .titlebar:not(headerbar) button.suggested-action:backdrop:disabled, .selection-mode.titlebar:not(headerbar) button.suggested-action:backdrop:disabled,
.selection-mode headerbar button.suggested-action:backdrop:disabled,
headerbar.selection-mode button.suggested-action:backdrop:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.selection-mode .titlebar:not(headerbar) .selection-menu, .selection-mode.titlebar:not(headerbar) .selection-menu,
.selection-mode headerbar .selection-menu:backdrop,
.selection-mode headerbar .selection-menu,
headerbar.selection-mode .selection-menu:backdrop,
headerbar.selection-mode .selection-menu {
  border-color: alpha(@accent_bg_color,0);
  background-color: alpha(@accent_bg_color,0);
  background-image: none;
  box-shadow: none;
  min-height: 20px;
  padding: 6px 10px;
}
.selection-mode .titlebar:not(headerbar) .selection-menu arrow, .selection-mode.titlebar:not(headerbar) .selection-menu arrow,
.selection-mode headerbar .selection-menu:backdrop arrow,
.selection-mode headerbar .selection-menu arrow,
headerbar.selection-mode .selection-menu:backdrop arrow,
headerbar.selection-mode .selection-menu arrow {
  -GtkArrow-arrow-scaling: 1;
}
.selection-mode .titlebar:not(headerbar) .selection-menu .arrow, .selection-mode.titlebar:not(headerbar) .selection-menu .arrow,
.selection-mode headerbar .selection-menu:backdrop .arrow,
.selection-mode headerbar .selection-menu .arrow,
headerbar.selection-mode .selection-menu:backdrop .arrow,
headerbar.selection-mode .selection-menu .arrow {
  -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
  color: alpha(@accent_fg_color,0.5);
  -gtk-icon-shadow: none;
}
.tiled .titlebar:not(headerbar), .tiled-top .titlebar:not(headerbar), .tiled-right .titlebar:not(headerbar), .tiled-bottom .titlebar:not(headerbar), .tiled-left .titlebar:not(headerbar), .maximized .titlebar:not(headerbar), .fullscreen .titlebar:not(headerbar),
.tiled headerbar:backdrop,
.tiled headerbar,
.tiled-top headerbar:backdrop,
.tiled-top headerbar,
.tiled-right headerbar:backdrop,
.tiled-right headerbar,
.tiled-bottom headerbar:backdrop,
.tiled-bottom headerbar,
.tiled-left headerbar:backdrop,
.tiled-left headerbar,
.maximized headerbar:backdrop,
.maximized headerbar,
.fullscreen headerbar:backdrop,
.fullscreen headerbar {
  border-radius: 0;
}
.default-decoration.titlebar:not(headerbar),
headerbar.default-decoration {
  min-height: 36px;
  padding: 0 6px 0 6px;
}
.default-decoration.titlebar:not(headerbar) button.titlebutton,
headerbar.default-decoration button.titlebutton {
  border-radius: 100%;
}
.titlebar:not(headerbar) separator.titlebutton,
headerbar separator.titlebutton {
  opacity: 0;
}
.solid-csd .titlebar:dir(rtl):not(headerbar), .solid-csd .titlebar:dir(ltr):not(headerbar),
.solid-csd headerbar:backdrop:dir(rtl),
.solid-csd headerbar:backdrop:dir(ltr),
.solid-csd headerbar:dir(rtl),
.solid-csd headerbar:dir(ltr) {
  margin-left: -1px;
  margin-right: -1px;
  margin-top: -1px;
  border-radius: 0;
  box-shadow: none;
}

headerbar entry,
headerbar spinbutton,
headerbar separator:not(.sidebar),
headerbar button {
  margin-top: 6px;
  margin-bottom: 6px;
}
headerbar switch {
  margin-top: 9px;
  margin-bottom: 9px;
}
headerbar.titlebar headerbar:not(.titlebar) {
  background: none;
  box-shadow: none;
}

.background .titlebar:backdrop, .background .titlebar {
  border-top-left-radius: 15px;
  border-top-right-radius: 15px;
}

.background.tiled .titlebar:backdrop, .background.tiled .titlebar,
.background.tiled-top .titlebar:backdrop,
.background.tiled-top .titlebar,
.background.tiled-right .titlebar:backdrop,
.background.tiled-right .titlebar,
.background.tiled-bottom .titlebar:backdrop,
.background.tiled-bottom .titlebar,
.background.tiled-left .titlebar:backdrop,
.background.tiled-left .titlebar,
.background.maximized .titlebar:backdrop,
.background.maximized .titlebar,
.background.solid-csd .titlebar:backdrop,
.background.solid-csd .titlebar {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

window separator:first-child + headerbar:backdrop, window separator:first-child + headerbar, window headerbar:first-child:backdrop, window headerbar:first-child {
  border-top-left-radius: 15px;
}
window headerbar:last-child:backdrop, window headerbar:last-child {
  border-top-right-radius: 15px;
}
window stack headerbar:first-child:backdrop, window stack headerbar:first-child, window stack headerbar:last-child:backdrop, window stack headerbar:last-child {
  border-top-left-radius: 15px;
  border-top-right-radius: 15px;
}
window.tiled headerbar, window.tiled headerbar:first-child, window.tiled headerbar:last-child, window.tiled headerbar:only-child, window.tiled headerbar:backdrop, window.tiled headerbar:backdrop:first-child, window.tiled headerbar:backdrop:last-child, window.tiled headerbar:backdrop:only-child, window.tiled-top headerbar, window.tiled-top headerbar:first-child, window.tiled-top headerbar:last-child, window.tiled-top headerbar:only-child, window.tiled-top headerbar:backdrop, window.tiled-top headerbar:backdrop:first-child, window.tiled-top headerbar:backdrop:last-child, window.tiled-top headerbar:backdrop:only-child, window.tiled-right headerbar, window.tiled-right headerbar:first-child, window.tiled-right headerbar:last-child, window.tiled-right headerbar:only-child, window.tiled-right headerbar:backdrop, window.tiled-right headerbar:backdrop:first-child, window.tiled-right headerbar:backdrop:last-child, window.tiled-right headerbar:backdrop:only-child, window.tiled-bottom headerbar, window.tiled-bottom headerbar:first-child, window.tiled-bottom headerbar:last-child, window.tiled-bottom headerbar:only-child, window.tiled-bottom headerbar:backdrop, window.tiled-bottom headerbar:backdrop:first-child, window.tiled-bottom headerbar:backdrop:last-child, window.tiled-bottom headerbar:backdrop:only-child, window.tiled-left headerbar, window.tiled-left headerbar:first-child, window.tiled-left headerbar:last-child, window.tiled-left headerbar:only-child, window.tiled-left headerbar:backdrop, window.tiled-left headerbar:backdrop:first-child, window.tiled-left headerbar:backdrop:last-child, window.tiled-left headerbar:backdrop:only-child, window.maximized headerbar, window.maximized headerbar:first-child, window.maximized headerbar:last-child, window.maximized headerbar:only-child, window.maximized headerbar:backdrop, window.maximized headerbar:backdrop:first-child, window.maximized headerbar:backdrop:last-child, window.maximized headerbar:backdrop:only-child, window.fullscreen headerbar, window.fullscreen headerbar:first-child, window.fullscreen headerbar:last-child, window.fullscreen headerbar:only-child, window.fullscreen headerbar:backdrop, window.fullscreen headerbar:backdrop:first-child, window.fullscreen headerbar:backdrop:last-child, window.fullscreen headerbar:backdrop:only-child, window.solid-csd headerbar, window.solid-csd headerbar:first-child, window.solid-csd headerbar:last-child, window.solid-csd headerbar:only-child, window.solid-csd headerbar:backdrop, window.solid-csd headerbar:backdrop:first-child, window.solid-csd headerbar:backdrop:last-child, window.solid-csd headerbar:backdrop:only-child {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

window.csd > .titlebar:not(headerbar) {
  padding: 0;
  background-color: transparent;
  background-image: none;
  border-style: none;
  border-color: transparent;
  box-shadow: none;
}
.titlebar:not(headerbar) separator {
  background-color: alpha(currentColor,0.15);
}
.titlebar:not(headerbar) separator:backdrop {
  background-color: mix(currentColor,@window_bg_color,0.81);
}

window.devel headerbar.titlebar:not(.selection-mode) {
  background: @window_bg_color cross-fade(10% -gtk-icontheme("system-run-symbolic"), image(transparent)) 90% 0/256px 256px no-repeat, linear-gradient(to right, transparent 65%, alpha(@accent_bg_color,0.2)), linear-gradient(to top, mix(black,@headerbar_bg_color,0.97), @headerbar_bg_color 3px, mix(white,@headerbar_bg_color,0.94));
}
window.devel headerbar.titlebar:not(.selection-mode):backdrop {
  background: @headerbar_backdrop_color cross-fade(10% -gtk-icontheme("system-run-symbolic"), image(transparent)) 90% 0/256px 256px no-repeat, image(@headerbar_backdrop_color);
}

headerbar {
  box-shadow: 0 1px alpha(@headerbar_shade_color,0.3);
}

headerbar:not(.selection-mode), headerbar:not(.selection-mode) entry, .titlebar:not(.selection-mode), .titlebar:not(.selection-mode) entry {
  color: @headerbar_fg_color;
}
headerbar:not(.selection-mode):backdrop,
headerbar:not(.selection-mode) entry:backdrop, .titlebar:not(.selection-mode):backdrop,
.titlebar:not(.selection-mode) entry:backdrop {
  color: mix(@headerbar_fg_color,@headerbar_bg_color,0.5);
}
headerbar:not(.selection-mode) entry:disabled, .titlebar:not(.selection-mode) entry:disabled {
  color: mix(@headerbar_fg_color,@headerbar_bg_color,0.5);
}
headerbar:not(.selection-mode) button:not(.suggested-action):not(.destructive-action), .titlebar:not(.selection-mode) button:not(.suggested-action):not(.destructive-action) {
  color: @headerbar_fg_color;
}
headerbar:not(.selection-mode) button:not(.suggested-action):not(.destructive-action):backdrop, .titlebar:not(.selection-mode) button:not(.suggested-action):not(.destructive-action):backdrop {
  color: mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5);
}
headerbar:not(.selection-mode) button:not(.suggested-action):not(.destructive-action):disabled, .titlebar:not(.selection-mode) button:not(.suggested-action):not(.destructive-action):disabled {
  color: mix(@headerbar_fg_color,@headerbar_bg_color,0.5);
}
headerbar:not(.selection-mode) switch image, .titlebar:not(.selection-mode) switch image {
  color: transparent;
}
headerbar button:not(.titlebutton), headerbar button:not(.titlebutton):backdrop, .titlebar button:not(.titlebutton), .titlebar button:not(.titlebutton):backdrop {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar .linked.raised button:not(:only-child):not(:first-child),
headerbar .linked button:not(:only-child):not(:first-child), headerbar > .stack-switcher.linked > button:not(:only-child):not(:first-child), .titlebar .linked.raised button:not(:only-child):not(:first-child),
.titlebar .linked button:not(:only-child):not(:first-child), .titlebar > .stack-switcher.linked > button:not(:only-child):not(:first-child) {
  border-left-color: alpha(currentColor,0.15);
}
headerbar .linked.raised button:not(:only-child):not(:last-child),
headerbar .linked button:not(:only-child):not(:last-child), headerbar > .stack-switcher.linked > button:not(:only-child):not(:last-child), .titlebar .linked.raised button:not(:only-child):not(:last-child),
.titlebar .linked button:not(:only-child):not(:last-child), .titlebar > .stack-switcher.linked > button:not(:only-child):not(:last-child) {
  border-right-color: alpha(currentColor,0.15);
}
headerbar .linked.raised button:not(:only-child):first-child,
headerbar .linked button:not(:only-child):first-child, headerbar > .stack-switcher.linked > button:not(:only-child):first-child, .titlebar .linked.raised button:not(:only-child):first-child,
.titlebar .linked button:not(:only-child):first-child, .titlebar > .stack-switcher.linked > button:not(:only-child):first-child {
  border-left-color: transparent;
}
headerbar .linked.raised button:not(:only-child):last-child,
headerbar .linked button:not(:only-child):last-child, headerbar > .stack-switcher.linked > button:not(:only-child):last-child, .titlebar .linked.raised button:not(:only-child):last-child,
.titlebar .linked button:not(:only-child):last-child, .titlebar > .stack-switcher.linked > button:not(:only-child):last-child {
  border-right-color: transparent;
}
headerbar .linked.raised button:not(:only-child):first-child:not(:backdrop):checked, headerbar .linked.raised button:not(:only-child):first-child:not(:backdrop):active, headerbar .linked.raised button:not(:only-child):not(:last-child):not(:backdrop):checked, headerbar .linked.raised button:not(:only-child):not(:last-child):not(:backdrop):active,
headerbar .linked button:not(:only-child):first-child:not(:backdrop):checked,
headerbar .linked button:not(:only-child):first-child:not(:backdrop):active,
headerbar .linked button:not(:only-child):not(:last-child):not(:backdrop):checked,
headerbar .linked button:not(:only-child):not(:last-child):not(:backdrop):active, headerbar > .stack-switcher.linked > button:not(:only-child):first-child:not(:backdrop):checked, headerbar > .stack-switcher.linked > button:not(:only-child):first-child:not(:backdrop):active, headerbar > .stack-switcher.linked > button:not(:only-child):not(:last-child):not(:backdrop):checked, headerbar > .stack-switcher.linked > button:not(:only-child):not(:last-child):not(:backdrop):active, .titlebar .linked.raised button:not(:only-child):first-child:not(:backdrop):checked, .titlebar .linked.raised button:not(:only-child):first-child:not(:backdrop):active, .titlebar .linked.raised button:not(:only-child):not(:last-child):not(:backdrop):checked, .titlebar .linked.raised button:not(:only-child):not(:last-child):not(:backdrop):active,
.titlebar .linked button:not(:only-child):first-child:not(:backdrop):checked,
.titlebar .linked button:not(:only-child):first-child:not(:backdrop):active,
.titlebar .linked button:not(:only-child):not(:last-child):not(:backdrop):checked,
.titlebar .linked button:not(:only-child):not(:last-child):not(:backdrop):active, .titlebar > .stack-switcher.linked > button:not(:only-child):first-child:not(:backdrop):checked, .titlebar > .stack-switcher.linked > button:not(:only-child):first-child:not(:backdrop):active, .titlebar > .stack-switcher.linked > button:not(:only-child):not(:last-child):not(:backdrop):checked, .titlebar > .stack-switcher.linked > button:not(:only-child):not(:last-child):not(:backdrop):active {
  box-shadow: inset -1px 0 alpha(currentColor,0.15);
}
headerbar .linked.raised button:not(:only-child):first-child:not(:backdrop):checked + button, headerbar .linked.raised button:not(:only-child):first-child:not(:backdrop):active + button, headerbar .linked.raised button:not(:only-child):not(:last-child):not(:backdrop):checked + button, headerbar .linked.raised button:not(:only-child):not(:last-child):not(:backdrop):active + button,
headerbar .linked button:not(:only-child):first-child:not(:backdrop):checked + button,
headerbar .linked button:not(:only-child):first-child:not(:backdrop):active + button,
headerbar .linked button:not(:only-child):not(:last-child):not(:backdrop):checked + button,
headerbar .linked button:not(:only-child):not(:last-child):not(:backdrop):active + button, headerbar > .stack-switcher.linked > button:not(:only-child):first-child:not(:backdrop):checked + button, headerbar > .stack-switcher.linked > button:not(:only-child):first-child:not(:backdrop):active + button, headerbar > .stack-switcher.linked > button:not(:only-child):not(:last-child):not(:backdrop):checked + button, headerbar > .stack-switcher.linked > button:not(:only-child):not(:last-child):not(:backdrop):active + button, .titlebar .linked.raised button:not(:only-child):first-child:not(:backdrop):checked + button, .titlebar .linked.raised button:not(:only-child):first-child:not(:backdrop):active + button, .titlebar .linked.raised button:not(:only-child):not(:last-child):not(:backdrop):checked + button, .titlebar .linked.raised button:not(:only-child):not(:last-child):not(:backdrop):active + button,
.titlebar .linked button:not(:only-child):first-child:not(:backdrop):checked + button,
.titlebar .linked button:not(:only-child):first-child:not(:backdrop):active + button,
.titlebar .linked button:not(:only-child):not(:last-child):not(:backdrop):checked + button,
.titlebar .linked button:not(:only-child):not(:last-child):not(:backdrop):active + button, .titlebar > .stack-switcher.linked > button:not(:only-child):first-child:not(:backdrop):checked + button, .titlebar > .stack-switcher.linked > button:not(:only-child):first-child:not(:backdrop):active + button, .titlebar > .stack-switcher.linked > button:not(:only-child):not(:last-child):not(:backdrop):checked + button, .titlebar > .stack-switcher.linked > button:not(:only-child):not(:last-child):not(:backdrop):active + button {
  border-left-color: transparent;
}
headerbar.selection-mode button:hover:backdrop, .titlebar.selection-mode button:hover:backdrop {
  color: alpha(@accent_fg_color,0.5);
  background-color: mix(@accent_fg_color,alpha(@accent_bg_color,0.5),0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar entry, .titlebar entry {
  min-height: 32px;
  color: @headerbar_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.9);
}
headerbar entry:backdrop, .titlebar entry:backdrop {
  color: mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5);
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.9);
}
headerbar entry:disabled, .titlebar entry:disabled {
  color: alpha(mix(@headerbar_fg_color,@headerbar_bg_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_bg_color,0.5),@headerbar_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
headerbar entry:disabled:backdrop, .titlebar entry:disabled:backdrop {
  color: alpha(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
headerbar entry image.left, .titlebar entry image.left {
  margin-right: 6px;
}
headerbar entry image.right, .titlebar entry image.right {
  margin-left: 6px;
}
headerbar entry image, .titlebar entry image {
  color: mix(@window_fg_color,@view_bg_color,0.2);
}
headerbar entry image:hover, .titlebar entry image:hover {
  color: @headerbar_fg_color;
}
headerbar entry image:active, .titlebar entry image:active {
  color: @accent_bg_color;
}
headerbar entry image:backdrop, .titlebar entry image:backdrop {
  color: mix(mix(@window_fg_color,@window_bg_color,0.5),@window_bg_color,0.2);
}
headerbar entry.error, .titlebar entry.error {
  color: @error_bg_color;
  border-color: @error_bg_color;
}
headerbar entry.error:focus, .titlebar entry.error:focus {
  box-shadow: inset 0 0 0 1px @error_bg_color;
  border-color: @error_bg_color;
  transition: 300ms ease-in-out;
  transition-property: border, box-shadow;
}
headerbar entry.error selection, .titlebar entry.error selection {
  background-color: @error_bg_color;
}
headerbar entry.warning, .titlebar entry.warning {
  color: @warning_bg_color;
  border-color: @warning_bg_color;
}
headerbar entry.warning:focus, .titlebar entry.warning:focus {
  box-shadow: inset 0 0 0 1px @warning_bg_color;
  border-color: @warning_bg_color;
  transition: 300ms ease-in-out;
  transition-property: border, box-shadow;
}
headerbar entry.warning selection, .titlebar entry.warning selection {
  background-color: @warning_bg_color;
}
headerbar entry.flat:focus, headerbar entry.flat:backdrop, headerbar entry.flat:disabled, headerbar entry.flat:backdrop:disabled, headerbar entry.flat, .titlebar entry.flat:focus, .titlebar entry.flat:backdrop, .titlebar entry.flat:disabled, .titlebar entry.flat:backdrop:disabled, .titlebar entry.flat {
  min-height: 0;
  padding: 2px;
  background-color: transparent;
  border-color: transparent;
  border-radius: 0;
}
headerbar entry:drop(active):focus, headerbar entry:drop(active), .titlebar entry:drop(active):focus, .titlebar entry:drop(active) {
  border-color: @accent_bg_color;
  box-shadow: inset 0 0 0 1px @accent_bg_color;
}
headerbar entry progress, .titlebar entry progress {
  margin: 2px -6px;
  background-color: transparent;
  background-image: none;
  border-radius: 0;
  border-width: 0 0 2px;
  border-color: @accent_bg_color;
  border-style: solid;
  box-shadow: none;
}
headerbar entry progress:backdrop, .titlebar entry progress:backdrop {
  background-color: transparent;
}
headerbar .linked entry:not(:first-child), .titlebar .linked entry:not(:first-child) {
  border-left-color: alpha(currentColor,0.15);
}
headerbar .linked entry:not(:first-child):backdrop, .titlebar .linked entry:not(:first-child):backdrop {
  border-left-color: alpha(currentColor,0.15);
}
headerbar switch:backdrop, .titlebar switch:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
  border-color: mix(currentColor,@window_bg_color,0.81);
  background-color: mix(mix(currentColor,@window_bg_color,0.81),@window_bg_color,0.3);
  text-shadow: none;
  transition: 200ms ease-out;
}
headerbar switch:backdrop:checked, .titlebar switch:backdrop:checked {
  color: @window_bg_color;
  background-color: alpha(@accent_bg_color,0.5);
}
headerbar switch:backdrop:checked:disabled, .titlebar switch:backdrop:checked:disabled {
  background-color: alpha(@accent_bg_color,0.3);
}
headerbar switch:backdrop:disabled, .titlebar switch:backdrop:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
headerbar switch:backdrop slider, .titlebar switch:backdrop slider {
  transition: 200ms ease-out;
  background-color: mix(@view_bg_color,mix(white,@view_bg_color,0.2),0.7);
}
headerbar switch:backdrop:checked > slider, .titlebar switch:backdrop:checked > slider {
  background-color: mix(@view_bg_color,mix(white,@view_bg_color,0.2),0.7);
}
headerbar switch:backdrop:disabled slider, .titlebar switch:backdrop:disabled slider {
  background-color: mix(@view_bg_color,mix(white,@view_bg_color,0.2),0.4);
}
headerbar switch:disabled, .titlebar switch:disabled {
  background-color: shade(mix(mix(currentColor,@window_bg_color,0.81),@window_bg_color,0.3),0.96);
}
headerbar switch:disabled:backdrop, .titlebar switch:disabled:backdrop {
  background-color: mix(mix(currentColor,@window_bg_color,0.81),@window_bg_color,0.3);
}
headerbar:not(:backdrop), .titlebar:not(:backdrop) {
  transition: 50ms ease-in;
  transition-property: background-color, background-image, box-shadow, border-color, color, text-shadow;
}
headerbar .right, .titlebar .right {
  margin-left: -4px;
}
headerbar .right separator, .titlebar .right separator {
  min-width: 0;
}
headerbar .left, .titlebar .left {
  margin-right: -4px;
}
headerbar .left separator, .titlebar .left separator {
  min-width: 0;
}

.ssd .titlebar {
  border-width: 0;
  padding: 0px 4px;
  min-height: 36px;
  border-bottom: 1px solid @headerbar_shade_color;
}
.default-decoration .title {
  color: @headerbar_fg_color;
}
.default-decoration .title:backdrop {
  color: mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5);
}

button.titlebutton:not(.appmenu),
headerbar.default-decoration button.titlebutton, .default-decoration.titlebar:not(headerbar) button.titlebutton, .ssd .titlebar.default-decoration button.titlebutton {
  background-color: alpha(currentColor,0.1);
  min-height: 24px;
  min-width: 24px;
  margin: 0 4px;
  padding: 0;
}
button.titlebutton:hover:not(.appmenu),
headerbar.default-decoration button.titlebutton:hover, .default-decoration.titlebar:not(headerbar) button.titlebutton:hover, .ssd .titlebar.default-decoration button.titlebutton:hover {
  background-color: alpha(currentColor,0.15);
}
button.titlebutton:active:not(.appmenu),
headerbar.default-decoration button.titlebutton:active, .default-decoration.titlebar:not(headerbar) button.titlebutton:active, .ssd .titlebar.default-decoration button.titlebutton:active {
  background-color: alpha(currentColor,0.3);
}
button.titlebutton:backdrop:not(.appmenu),
headerbar.default-decoration button.titlebutton:backdrop, .default-decoration.titlebar:not(headerbar) button.titlebutton:backdrop, .ssd .titlebar.default-decoration button.titlebutton:backdrop {
  background: alpha(currentColor,0.1);
}
button.titlebutton:backdrop:hover:not(.appmenu),
headerbar.default-decoration button.titlebutton:backdrop:hover, .default-decoration.titlebar:not(headerbar) button.titlebutton:backdrop:hover, .ssd .titlebar.default-decoration button.titlebutton:backdrop:hover {
  background: alpha(currentColor,0.15);
}

headerbar button.text-button:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button,
headerbar .linked button:not(:only-child), headerbar > .stack-switcher.linked > button, .titlebar .linked.raised button,
.titlebar .linked button:not(:only-child), .titlebar > .stack-switcher.linked > button, headerbar .linked entry ~ .image-button, .titlebar .linked entry ~ .image-button {
  color: @headerbar_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.9);
}
headerbar button.text-button:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:backdrop,
headerbar .linked button:backdrop:not(:only-child), headerbar > .stack-switcher.linked > button:backdrop, .titlebar .linked.raised button:backdrop,
.titlebar .linked button:backdrop:not(:only-child), .titlebar > .stack-switcher.linked > button:backdrop, headerbar .linked entry ~ .image-button:backdrop, .titlebar .linked entry ~ .image-button:backdrop {
  color: alpha(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar button.text-button:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:hover,
headerbar .linked button:hover:not(:only-child), headerbar > .stack-switcher.linked > button:hover, .titlebar .linked.raised button:hover,
.titlebar .linked button:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:hover, headerbar .linked entry ~ .image-button:hover, .titlebar .linked entry ~ .image-button:hover {
  color: @headerbar_fg_color;
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.85);
  box-shadow: none;
}
headerbar button.text-button:hover:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:hover:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:hover:backdrop,
headerbar .linked button:hover:backdrop:not(:only-child), headerbar > .stack-switcher.linked > button:hover:backdrop, .titlebar .linked.raised button:hover:backdrop,
.titlebar .linked button:hover:backdrop:not(:only-child), .titlebar > .stack-switcher.linked > button:hover:backdrop, headerbar .linked entry ~ .image-button:hover:backdrop, .titlebar .linked entry ~ .image-button:hover:backdrop {
  color: mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.85);
  box-shadow: none;
}
headerbar button.text-button:active:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:active:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:active,
headerbar .linked button:active:not(:only-child), headerbar > .stack-switcher.linked > button:active, .titlebar .linked.raised button:active,
.titlebar .linked button:active:not(:only-child), .titlebar > .stack-switcher.linked > button:active, headerbar .linked entry ~ .image-button:active, .titlebar .linked entry ~ .image-button:active, headerbar button.text-button:checked:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:checked:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:checked,
headerbar .linked button:checked:not(:only-child), headerbar > .stack-switcher.linked > button:checked, .titlebar .linked.raised button:checked,
.titlebar .linked button:checked:not(:only-child), .titlebar > .stack-switcher.linked > button:checked, headerbar .linked entry ~ .image-button:checked, .titlebar .linked entry ~ .image-button:checked {
  color: @headerbar_fg_color;
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar button.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:active:hover,
headerbar .linked button:active:hover:not(:only-child), headerbar > .stack-switcher.linked > button:active:hover, .titlebar .linked.raised button:active:hover,
.titlebar .linked button:active:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:active:hover, headerbar .linked entry ~ .image-button:active:hover, .titlebar .linked entry ~ .image-button:active:hover, headerbar button.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:checked:hover,
headerbar .linked button:checked:hover:not(:only-child), headerbar > .stack-switcher.linked > button:checked:hover, .titlebar .linked.raised button:checked:hover,
.titlebar .linked button:checked:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:checked:hover, headerbar .linked entry ~ .image-button:checked:hover, .titlebar .linked entry ~ .image-button:checked:hover {
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.65);
}
headerbar button.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:active:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:active:hover,
headerbar .linked button:active:hover:not(:only-child), headerbar > .stack-switcher.linked > button:active:hover, .titlebar .linked.raised button:active:hover,
.titlebar .linked button:active:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:active:hover, headerbar .linked entry ~ .image-button:active:hover, .titlebar .linked entry ~ .image-button:active:hover, headerbar button.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:checked:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:checked:hover,
headerbar .linked button:checked:hover:not(:only-child), headerbar > .stack-switcher.linked > button:checked:hover, .titlebar .linked.raised button:checked:hover,
.titlebar .linked button:checked:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:checked:hover, headerbar .linked entry ~ .image-button:checked:hover, .titlebar .linked entry ~ .image-button:checked:hover {
  color: @headerbar_fg_color;
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar button.text-button:active:hover:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:active:hover:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:active:hover:hover,
headerbar .linked button:active:hover:hover:not(:only-child), headerbar > .stack-switcher.linked > button:active:hover:hover, .titlebar .linked.raised button:active:hover:hover,
.titlebar .linked button:active:hover:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:active:hover:hover, headerbar .linked entry ~ .image-button:active:hover:hover, .titlebar .linked entry ~ .image-button:active:hover:hover, headerbar button.text-button:checked:hover:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:checked:hover:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:checked:hover:hover,
headerbar .linked button:checked:hover:hover:not(:only-child), headerbar > .stack-switcher.linked > button:checked:hover:hover, .titlebar .linked.raised button:checked:hover:hover,
.titlebar .linked button:checked:hover:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:checked:hover:hover, headerbar .linked entry ~ .image-button:checked:hover:hover, .titlebar .linked entry ~ .image-button:checked:hover:hover {
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.65);
}
headerbar button.text-button:active:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:active:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:active:backdrop,
headerbar .linked button:active:backdrop:not(:only-child), headerbar > .stack-switcher.linked > button:active:backdrop, .titlebar .linked.raised button:active:backdrop,
.titlebar .linked button:active:backdrop:not(:only-child), .titlebar > .stack-switcher.linked > button:active:backdrop, headerbar .linked entry ~ .image-button:active:backdrop, .titlebar .linked entry ~ .image-button:active:backdrop, headerbar button.text-button:checked:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:checked:backdrop:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:checked:backdrop,
headerbar .linked button:checked:backdrop:not(:only-child), headerbar > .stack-switcher.linked > button:checked:backdrop, .titlebar .linked.raised button:checked:backdrop,
.titlebar .linked button:checked:backdrop:not(:only-child), .titlebar > .stack-switcher.linked > button:checked:backdrop, headerbar .linked entry ~ .image-button:checked:backdrop, .titlebar .linked entry ~ .image-button:checked:backdrop {
  color: alpha(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.7);
}
headerbar button.text-button:active:backdrop:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:active:backdrop:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:active:backdrop:hover,
headerbar .linked button:active:backdrop:hover:not(:only-child), headerbar > .stack-switcher.linked > button:active:backdrop:hover, .titlebar .linked.raised button:active:backdrop:hover,
.titlebar .linked button:active:backdrop:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:active:backdrop:hover, headerbar .linked entry ~ .image-button:active:backdrop:hover, .titlebar .linked entry ~ .image-button:active:backdrop:hover, headerbar button.text-button:checked:backdrop:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:checked:backdrop:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:checked:backdrop:hover,
headerbar .linked button:checked:backdrop:hover:not(:only-child), headerbar > .stack-switcher.linked > button:checked:backdrop:hover, .titlebar .linked.raised button:checked:backdrop:hover,
.titlebar .linked button:checked:backdrop:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:checked:backdrop:hover, headerbar .linked entry ~ .image-button:checked:backdrop:hover, .titlebar .linked entry ~ .image-button:checked:backdrop:hover {
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.6);
}
headerbar button.text-button:active:backdrop:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:active:backdrop:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:active:backdrop:hover,
headerbar .linked button:active:backdrop:hover:not(:only-child), headerbar > .stack-switcher.linked > button:active:backdrop:hover, .titlebar .linked.raised button:active:backdrop:hover,
.titlebar .linked button:active:backdrop:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:active:backdrop:hover, headerbar .linked entry ~ .image-button:active:backdrop:hover, .titlebar .linked entry ~ .image-button:active:backdrop:hover, headerbar button.text-button:checked:backdrop:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:checked:backdrop:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:checked:backdrop:hover,
headerbar .linked button:checked:backdrop:hover:not(:only-child), headerbar > .stack-switcher.linked > button:checked:backdrop:hover, .titlebar .linked.raised button:checked:backdrop:hover,
.titlebar .linked button:checked:backdrop:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:checked:backdrop:hover, headerbar .linked entry ~ .image-button:checked:backdrop:hover, .titlebar .linked entry ~ .image-button:checked:backdrop:hover {
  color: alpha(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.7);
}
headerbar button.text-button:active:backdrop:hover:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:active:backdrop:hover:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:active:backdrop:hover:hover,
headerbar .linked button:active:backdrop:hover:hover:not(:only-child), headerbar > .stack-switcher.linked > button:active:backdrop:hover:hover, .titlebar .linked.raised button:active:backdrop:hover:hover,
.titlebar .linked button:active:backdrop:hover:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:active:backdrop:hover:hover, headerbar .linked entry ~ .image-button:active:backdrop:hover:hover, .titlebar .linked entry ~ .image-button:active:backdrop:hover:hover, headerbar button.text-button:checked:backdrop:hover:hover:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:checked:backdrop:hover:hover:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:checked:backdrop:hover:hover,
headerbar .linked button:checked:backdrop:hover:hover:not(:only-child), headerbar > .stack-switcher.linked > button:checked:backdrop:hover:hover, .titlebar .linked.raised button:checked:backdrop:hover:hover,
.titlebar .linked button:checked:backdrop:hover:hover:not(:only-child), .titlebar > .stack-switcher.linked > button:checked:backdrop:hover:hover, headerbar .linked entry ~ .image-button:checked:backdrop:hover:hover, .titlebar .linked entry ~ .image-button:checked:backdrop:hover:hover {
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.6);
}
headerbar button.text-button:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:disabled,
headerbar .linked button:disabled:not(:only-child), headerbar > .stack-switcher.linked > button:disabled, .titlebar .linked.raised button:disabled,
.titlebar .linked button:disabled:not(:only-child), .titlebar > .stack-switcher.linked > button:disabled, headerbar .linked entry ~ .image-button:disabled, .titlebar .linked entry ~ .image-button:disabled {
  color: alpha(mix(@headerbar_fg_color,@headerbar_bg_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_bg_color,0.5),@headerbar_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
headerbar button.text-button:backdrop:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.text-button:backdrop:disabled:not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button:backdrop:disabled,
headerbar .linked button:backdrop:disabled:not(:only-child), headerbar > .stack-switcher.linked > button:backdrop:disabled, .titlebar .linked.raised button:backdrop:disabled,
.titlebar .linked button:backdrop:disabled:not(:only-child), .titlebar > .stack-switcher.linked > button:backdrop:disabled, headerbar .linked entry ~ .image-button:backdrop:disabled, .titlebar .linked entry ~ .image-button:backdrop:disabled {
  color: alpha(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar button.default.text-button:not(:backdrop):not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.default.text-button:not(:backdrop):not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.default:not(:backdrop):not(.suggested-action):not(.destructive-action),
headerbar .linked button.default:not(:backdrop):not(.suggested-action):not(.destructive-action):not(:only-child), headerbar > .stack-switcher.linked > button.default:not(:backdrop):not(.suggested-action):not(.destructive-action), .titlebar .linked.raised button.default:not(:backdrop):not(.suggested-action):not(.destructive-action),
.titlebar .linked button.default:not(:backdrop):not(.suggested-action):not(.destructive-action):not(:only-child), .titlebar > .stack-switcher.linked > button.default:not(:backdrop):not(.suggested-action):not(.destructive-action), headerbar .linked entry ~ .default.image-button:not(:backdrop):not(.suggested-action):not(.destructive-action), .titlebar .linked entry ~ .default.image-button:not(:backdrop):not(.suggested-action):not(.destructive-action) {
  color: @headerbar_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.9);
}
headerbar button.flat.text-button:not(:hover):not(:active):not(.suggested-action):not(.destructive-action):not(:disabled), .titlebar button.flat.text-button:not(:hover):not(:active):not(.suggested-action):not(.destructive-action):not(:disabled), headerbar .linked.raised button.flat:not(:hover):not(:active),
headerbar .linked button.flat:not(:hover):not(:active):not(:only-child), headerbar > .stack-switcher.linked > button.flat:not(:hover):not(:active), .titlebar .linked.raised button.flat:not(:hover):not(:active),
.titlebar .linked button.flat:not(:hover):not(:active):not(:only-child), .titlebar > .stack-switcher.linked > button.flat:not(:hover):not(:active), headerbar .linked entry ~ .flat.image-button:not(:hover):not(:active), .titlebar .linked entry ~ .flat.image-button:not(:hover):not(:active) {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar button:hover:not(.titlebutton), .titlebar button:hover:not(.titlebutton), headerbar viewswitcher button.radio:hover:checked, .titlebar viewswitcher button.radio:hover:checked {
  color: @headerbar_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.9);
}
headerbar button:hover:backdrop:not(.titlebutton), .titlebar button:hover:backdrop:not(.titlebutton), headerbar viewswitcher button.radio:hover:backdrop:checked, .titlebar viewswitcher button.radio:hover:backdrop:checked {
  color: alpha(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.9);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar button:active:not(.titlebutton), .titlebar button:active:not(.titlebutton), headerbar button:checked:not(.titlebutton), .titlebar button:checked:not(.titlebutton), headerbar viewswitcher button.radio:checked, .titlebar viewswitcher button.radio:checked {
  color: @headerbar_fg_color;
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar button:active:hover:not(.titlebutton), .titlebar button:active:hover:not(.titlebutton), headerbar button:checked:hover:not(.titlebutton), .titlebar button:checked:hover:not(.titlebutton), headerbar viewswitcher button.radio:checked:hover, .titlebar viewswitcher button.radio:checked:hover {
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.65);
}
headerbar button:active:hover:not(.titlebutton), .titlebar button:active:hover:not(.titlebutton), headerbar button:checked:hover:not(.titlebutton), .titlebar button:checked:hover:not(.titlebutton), headerbar viewswitcher button.radio:checked:hover, .titlebar viewswitcher button.radio:checked:hover {
  color: @headerbar_fg_color;
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
headerbar button:active:hover:hover:not(.titlebutton), .titlebar button:active:hover:hover:not(.titlebutton), headerbar button:checked:hover:hover:not(.titlebutton), .titlebar button:checked:hover:hover:not(.titlebutton), headerbar viewswitcher button.radio:checked:hover:hover, .titlebar viewswitcher button.radio:checked:hover:hover {
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.65);
}
headerbar button:active:backdrop:not(.titlebutton), .titlebar button:active:backdrop:not(.titlebutton), headerbar button:checked:backdrop:not(.titlebutton), .titlebar button:checked:backdrop:not(.titlebutton), headerbar viewswitcher button.radio:checked:backdrop, .titlebar viewswitcher button.radio:checked:backdrop {
  color: alpha(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.7);
}
headerbar button:active:backdrop:hover:not(.titlebutton), .titlebar button:active:backdrop:hover:not(.titlebutton), headerbar button:checked:backdrop:hover:not(.titlebutton), .titlebar button:checked:backdrop:hover:not(.titlebutton), headerbar viewswitcher button.radio:checked:backdrop:hover, .titlebar viewswitcher button.radio:checked:backdrop:hover {
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.6);
}
headerbar button:active:backdrop:hover:not(.titlebutton), .titlebar button:active:backdrop:hover:not(.titlebutton), headerbar button:checked:backdrop:hover:not(.titlebutton), .titlebar button:checked:backdrop:hover:not(.titlebutton), headerbar viewswitcher button.radio:checked:backdrop:hover, .titlebar viewswitcher button.radio:checked:backdrop:hover {
  color: alpha(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),0.5);
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.7);
}
headerbar button:active:backdrop:hover:hover:not(.titlebutton), .titlebar button:active:backdrop:hover:hover:not(.titlebutton), headerbar button:checked:backdrop:hover:hover:not(.titlebutton), .titlebar button:checked:backdrop:hover:hover:not(.titlebutton), headerbar viewswitcher button.radio:checked:backdrop:hover:hover, .titlebar viewswitcher button.radio:checked:backdrop:hover:hover {
  background-color: mix(mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5),@headerbar_backdrop_color,0.6);
}
headerbar button.default:not(:backdrop):not(.suggested-action):not(.destructive-action):not(.titlebutton), .titlebar button.default:not(:backdrop):not(.suggested-action):not(.destructive-action):not(.titlebutton), headerbar viewswitcher button.default.radio:not(:backdrop):not(.suggested-action):not(.destructive-action):checked, .titlebar viewswitcher button.default.radio:not(:backdrop):not(.suggested-action):not(.destructive-action):checked {
  color: @headerbar_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.9);
}
headerbar button.flat:not(:hover):not(:active):not(.titlebutton), .titlebar button.flat:not(:hover):not(:active):not(.titlebutton), headerbar viewswitcher button.flat.radio:not(:hover):not(:active):checked, .titlebar viewswitcher button.flat.radio:not(:hover):not(:active):checked {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.path-bar button.text-button, .path-bar button.image-button, .path-bar button {
  padding-left: 4px;
  padding-right: 4px;
}
.path-bar button.text-button.image-button label {
  padding-left: 0;
  padding-right: 0;
}
.path-bar button.text-button.image-button label:last-child, .path-bar button label:last-child {
  padding-right: 8px;
}
.path-bar button.text-button.image-button label:first-child, .path-bar button label:first-child {
  padding-left: 8px;
}
.path-bar button image {
  padding-left: 4px;
  padding-right: 4px;
}
.path-bar button.slider-button {
  padding-left: 0;
  padding-right: 0;
}

* {
  -GtkTreeView-horizontal-separator: 4;
  -GtkTreeView-grid-line-width: 1;
  -GtkTreeView-grid-line-pattern: "";
  -GtkTreeView-tree-line-width: 1;
  -GtkTreeView-tree-line-pattern: "";
  -GtkTreeView-expander-size: 16;
}

treeview.view {
  border-left-color: mix(@window_fg_color,@view_bg_color,0.5);
  border-top-color: @window_bg_color;
}
treeview.view:selected:focus, treeview.view:selected {
  border-radius: 0;
}
treeview.view:selected:backdrop, treeview.view:selected {
  border-left-color: mix(@accent_fg_color,@accent_bg_color,0.5);
  border-top-color: alpha(@window_fg_color,0.1);
}
treeview.view:disabled {
  color: alpha(@window_fg_color,0.5);
}
treeview.view:disabled:selected {
  color: mix(@accent_fg_color,@accent_bg_color,0.6);
}
treeview.view:disabled:selected:backdrop {
  color: mix(@window_bg_color,@accent_bg_color,0.7);
}
treeview.view.separator {
  min-height: 2px;
  color: @window_bg_color;
}
treeview.view:drop(active) {
  border-style: solid none;
  border-width: 1px;
  border-color: mix(black,@accent_bg_color,0.85);
}
treeview.view:drop(active).after {
  border-top-style: none;
}
treeview.view:drop(active).before {
  border-bottom-style: none;
}
treeview.view.expander {
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
}
treeview.view.expander:dir(rtl) {
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic-rtl");
}
treeview.view.expander {
  color: mix(@view_fg_color,@view_bg_color,0.3);
}
treeview.view.expander:hover {
  color: @view_fg_color;
}
treeview.view.expander:checked {
  -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
}
treeview.view.progressbar {
  color: @view_bg_color;
  background-color: @accent_bg_color;
  background-image: image(@accent_bg_color);
  box-shadow: none;
}
treeview.view.progressbar:selected:focus, treeview.view.progressbar:selected {
  color: @accent_bg_color;
  background-image: image(@view_bg_color);
}
treeview.view.progressbar:selected:focus:backdrop, treeview.view.progressbar:selected:backdrop {
  color: @accent_bg_color;
  background-color: @window_bg_color;
}
treeview.view.progressbar:backdrop {
  color: @window_bg_color;
  background-image: none;
  box-shadow: none;
}
treeview.view.trough {
  background-color: alpha(@window_fg_color,0.1);
}
treeview.view.trough:selected:focus, treeview.view.trough:selected {
  background-color: alpha(@accent_fg_color,0.3);
}
treeview.view header button {
  padding: 3px 6px;
  color: mix(@view_fg_color,@view_bg_color,0.6);
  background-image: none;
  background-color: @view_bg_color;
  border-bottom: 1px solid mix(currentColor,@window_bg_color,0.85);
  border-right: none;
  border-left: none;
  border-radius: 0;
  box-shadow: none;
  text-shadow: none;
  font-size: smaller;
}
treeview.view header button:hover {
  color: mix(@view_fg_color,@view_bg_color,0.3);
  transition: none;
  background-image: none;
}
treeview.view header button:active {
  color: mix(@view_fg_color,@view_bg_color,0.25);
  transition: none;
  background-color: @view_bg_color;
}
treeview.view header button:disabled {
  border-color: @window_bg_color;
  background-image: none;
}
treeview.view header button:last-child:backdrop, treeview.view header button:last-child {
  border-right-style: none;
}
treeview.view button.dnd:active, treeview.view button.dnd:selected, treeview.view button.dnd:hover, treeview.view button.dnd,
treeview.view header.button.dnd:active,
treeview.view header.button.dnd:selected,
treeview.view header.button.dnd:hover,
treeview.view header.button.dnd {
  padding: 3px 6px;
  color: @accent_fg_color;
  background-image: none;
  background-color: @accent_bg_color;
  border-style: none;
  border-radius: 0;
  box-shadow: none;
  text-shadow: none;
  transition: none;
  font-size: smaller;
}
treeview.view acceleditor > label {
  background-color: @accent_bg_color;
}

treeview.view:not(:backdrop):not(:selected):hover {
  background-color: alpha(currentColor,0.04);
}

menubar,
.menubar {
  -GtkWidget-window-dragging: true;
  padding: 0;
}
menubar:backdrop,
.menubar:backdrop {
  background-color: @window_bg_color;
}
menubar > menuitem,
.menubar > menuitem {
  border-radius: 9px;
  min-height: 16px;
  padding: 4px 8px;
}
menubar > menuitem menu:dir(rtl), menubar > menuitem menu:dir(ltr),
.menubar > menuitem menu:dir(rtl),
.menubar > menuitem menu:dir(ltr) {
  border-radius: 9px;
  padding: 6px;
}
menubar > menuitem menu:dir(rtl) menuitem, menubar > menuitem menu:dir(ltr) menuitem,
.menubar > menuitem menu:dir(rtl) menuitem,
.menubar > menuitem menu:dir(ltr) menuitem {
  border-radius: 9px;
}
menubar > menuitem:hover,
.menubar > menuitem:hover {
  background-color: alpha(currentColor,0.1);
}
menubar > menuitem:disabled,
.menubar > menuitem:disabled {
  color: alpha(@window_fg_color,0.5);
  box-shadow: none;
}
menubar .csd.popup decoration,
.menubar .csd.popup decoration {
  border-radius: 9px;
}

.background.popup {
  background-color: transparent;
}

menu,
.menu,
.context-menu {
  margin: 4px;
  padding: 6px;
  background-color: @popover_bg_color;
  border: 1px solid mix(currentColor,@window_bg_color,0.85);
}
menu separator,
.menu separator,
.context-menu separator {
  margin: 6px 0;
}
.csd menu,
.csd .menu,
.csd .context-menu {
  border: none;
  border-radius: 9px;
}
menu menuitem,
.menu menuitem,
.context-menu menuitem {
  min-height: 16px;
  min-width: 40px;
  padding: 4px 6px;
  text-shadow: none;
  font-weight: normal;
  border-radius: 9px;
}
menu menuitem:hover,
.menu menuitem:hover,
.context-menu menuitem:hover {
  color: @window_fg_color;
  background-color: alpha(currentColor,0.1);
}
menu menuitem:disabled,
.menu menuitem:disabled,
.context-menu menuitem:disabled {
  color: alpha(@window_fg_color,0.5);
}
menu menuitem arrow,
.menu menuitem arrow,
.context-menu menuitem arrow {
  min-height: 16px;
  min-width: 16px;
}
menu menuitem arrow:dir(ltr),
.menu menuitem arrow:dir(ltr),
.context-menu menuitem arrow:dir(ltr) {
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
  margin-left: 10px;
}
menu menuitem arrow:dir(rtl),
.menu menuitem arrow:dir(rtl),
.context-menu menuitem arrow:dir(rtl) {
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic-rtl");
  margin-right: 10px;
}
menu menuitem label:dir(rtl), menu menuitem label:dir(ltr),
.menu menuitem label:dir(rtl),
.menu menuitem label:dir(ltr),
.context-menu menuitem label:dir(rtl),
.context-menu menuitem label:dir(ltr) {
  color: inherit;
}
menu > arrow,
.menu > arrow,
.context-menu > arrow {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  min-height: 16px;
  min-width: 16px;
  padding: 4px;
  background-color: @popover_bg_color;
  border-radius: 0;
}
menu > arrow.top,
.menu > arrow.top,
.context-menu > arrow.top {
  margin-top: -4px;
  border-bottom: 1px solid mix(@window_fg_color,@view_bg_color,0.9);
  border-top-right-radius: 9px;
  border-top-left-radius: 9px;
  -gtk-icon-source: -gtk-icontheme("pan-up-symbolic");
}
menu > arrow.bottom,
.menu > arrow.bottom,
.context-menu > arrow.bottom {
  margin-top: 8px;
  margin-bottom: -12px;
  border-top: 1px solid mix(@window_fg_color,@view_bg_color,0.9);
  border-bottom-right-radius: 9px;
  border-bottom-left-radius: 9px;
  -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
}
menu > arrow:hover,
.menu > arrow:hover,
.context-menu > arrow:hover {
  background-color: mix(@window_fg_color,@view_bg_color,0.9);
}
menu > arrow:disabled,
.menu > arrow:disabled,
.context-menu > arrow:disabled {
  color: transparent;
  background-color: transparent;
  border-color: transparent;
}

menuitem accelerator {
  color: alpha(currentColor,0.55);
}
menuitem check,
menuitem radio {
  min-height: 16px;
  min-width: 16px;
}
menuitem check:dir(ltr),
menuitem radio:dir(ltr) {
  margin-right: 7px;
}
menuitem check:dir(rtl),
menuitem radio:dir(rtl) {
  margin-left: 7px;
}

popover.background {
  padding: 2px;
  background-color: @popover_bg_color;
  box-shadow: 0 1px 5px 1px rgba(0, 0, 0, 0.09), 0 2px 8px 3px rgba(0, 0, 0, 0.05);
}
.csd popover.background, popover.background {
  border: 1px solid mix(currentColor,@window_bg_color,0.85);
  border-radius: 15px;
}
.csd popover.background {
  background-clip: padding-box;
  border-color: rgba(0, 0, 0, 0.14);
}
popover.background > list,
popover.background > .view,
popover.background > toolbar {
  border-style: none;
  background-color: transparent;
}
.csd popover.background.touch-selection, .csd popover.background.magnifier, popover.background.touch-selection, popover.background.magnifier {
  border: 1px solid rgba(255, 255, 255, 0.1);
}
popover.background separator {
  margin: 6px 0;
}
popover.background list separator {
  margin: 0px;
}

popover.background button {
  color: @popover_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@popover_fg_color,@popover_bg_color,0.9);
}
popover.background button.flat:not(:hover):not(:active) {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
popover.background button:hover {
  color: @popover_fg_color;
  background-color: mix(@popover_fg_color,@popover_bg_color,0.85);
  box-shadow: none;
}
popover.background button:active, popover.background button:checked {
  color: @popover_fg_color;
  background-color: mix(@popover_fg_color,@popover_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
popover.background button:active:hover, popover.background button:checked:hover {
  background-color: mix(@popover_fg_color,@popover_bg_color,0.65);
}
popover.background button:disabled {
  color: alpha(mix(@popover_fg_color,@popover_bg_color,0.5),0.5);
  background-color: mix(mix(@popover_fg_color,@popover_bg_color,0.5),@popover_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
popover.background entry {
  color: @popover_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@popover_fg_color,@popover_bg_color,0.9);
}
popover.background entry:disabled {
  color: mix(@popover_fg_color,@popover_bg_color,0.5);
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(mix(@popover_fg_color,@popover_bg_color,0.5),@popover_bg_color,0.9);
}

popover.background modelbutton.flat {
  padding-left: 16px;
  padding-right: 16px;
}
popover.background modelbutton.flat,
popover.background .menuitem.button.flat {
  color: @popover_fg_color;
}
popover.background modelbutton.flat:disabled label,
popover.background .menuitem.button.flat:disabled label {
  color: mix(@popover_fg_color,@popover_bg_color,0.5);
}
popover.background modelbutton.flat:backdrop:hover,
popover.background .menuitem.button.flat:backdrop:hover {
  background-color: alpha(currentColor,0.04);
}

popover.menu > stack {
  margin: -6px;
}

notebook > header {
  padding: 1px;
  border-width: 1px;
  background-clip: padding-box;
  background-color: @window_bg_color;
  border-color: alpha(currentColor,0.15);
}
notebook > header tabs {
  margin: -1px;
}
notebook > header.top > tabs > tab:hover, notebook > header.bottom > tabs > tab:hover, notebook > header.left > tabs > tab:hover, notebook > header.right > tabs > tab:hover {
  background-color: alpha(currentColor,0.07);
}
notebook > header.top {
  border-bottom-style: solid;
}
notebook > header.top > tabs {
  margin-bottom: -2px;
}
notebook > header.top > tabs > tab:not(.reorderable-page):hover {
  box-shadow: inset 0 -4px alpha(currentColor,0.15);
}
notebook > header.top > tabs > tab:not(.reorderable-page):checked {
  box-shadow: inset 0 -4px @accent_bg_color;
}
notebook > header.bottom {
  border-top-style: solid;
}
notebook > header.bottom > tabs {
  margin-top: -2px;
}
notebook > header.bottom > tabs > tab:not(.reorderable-page):hover {
  box-shadow: inset 0 4px alpha(currentColor,0.15);
}
notebook > header.bottom > tabs > tab:not(.reorderable-page):checked {
  box-shadow: inset 0 4px @accent_bg_color;
}
notebook > header.left {
  border-right-style: solid;
}
notebook > header.left > tabs {
  margin-right: -2px;
}
notebook > header.left > tabs > tab:not(.reorderable-page):hover {
  box-shadow: inset -4px 0 alpha(currentColor,0.15);
}
notebook > header.left > tabs > tab:not(.reorderable-page):checked {
  box-shadow: inset -4px 0 @accent_bg_color;
}
notebook > header.right {
  border-left-style: solid;
}
notebook > header.right > tabs {
  margin-left: -2px;
}
notebook > header.right > tabs > tab:not(.reorderable-page):hover {
  box-shadow: inset 4px 0 alpha(currentColor,0.15);
}
notebook > header.right > tabs > tab:not(.reorderable-page):checked {
  box-shadow: inset 4px 0 @accent_bg_color;
}
notebook > header.top > tabs > arrow {
  border-top-style: none;
}
notebook > header.bottom > tabs > arrow {
  border-bottom-style: none;
}
notebook > header.top > tabs > arrow, notebook > header.bottom > tabs > arrow {
  margin-left: -5px;
  margin-right: -5px;
  padding-left: 4px;
  padding-right: 4px;
}
notebook > header.top > tabs > arrow.down, notebook > header.bottom > tabs > arrow.down {
  -gtk-icon-source: -gtk-icontheme("pan-start-symbolic");
}
notebook > header.top > tabs > arrow.up, notebook > header.bottom > tabs > arrow.up {
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
}

notebook > header.left > tabs > arrow {
  border-left-style: none;
}
notebook > header.right > tabs > arrow {
  border-right-style: none;
}
notebook > header.left > tabs > arrow, notebook > header.right > tabs > arrow {
  margin-top: -5px;
  margin-bottom: -5px;
  padding-top: 4px;
  padding-bottom: 4px;
}
notebook > header.left > tabs > arrow.down, notebook > header.right > tabs > arrow.down {
  -gtk-icon-source: -gtk-icontheme("pan-up-symbolic");
}
notebook > header.left > tabs > arrow.up, notebook > header.right > tabs > arrow.up {
  -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
}

notebook > header > tabs > arrow {
  min-height: 16px;
  min-width: 16px;
  border-radius: 0;
}
notebook > header > tabs > arrow:hover:not(:active) {
  box-shadow: none;
}
notebook > header > tabs > arrow:disabled {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
notebook > header tab {
  min-height: 30px;
  min-width: 30px;
  padding: 3px 12px;
  color: @window_fg_color;
  font-weight: normal;
  border-width: 0px;
  border-color: transparent;
}
notebook > header tab.reorderable-page {
  -gtk-outline-radius: 11px;
  margin: 4px 2px;
  border-radius: 9px;
}
notebook > header tab.reorderable-page {
  outline: 0 solid transparent;
  outline-offset: 4px;
  transition: outline-color 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), outline-width 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), outline-offset 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}
notebook > header tab.reorderable-page:focus {
  outline-color: alpha(@accent_color,0.5);
  outline-width: 2px;
  outline-offset: -2px;
}
notebook > header tab.reorderable-page:hover {
  background-color: alpha(currentColor,0.07);
  box-shadow: none;
}
notebook > header tab.reorderable-page:checked {
  background-color: alpha(currentColor,0.1);
  box-shadow: none;
}
notebook > header tab.reorderable-page:checked:hover {
  background-color: alpha(currentColor,0.12);
}
notebook > header tab button.flat {
  color: alpha(currentColor,0.3);
  padding: 0;
  margin-top: 4px;
  margin-bottom: 4px;
  min-width: 20px;
  min-height: 20px;
  border-radius: 100%;
}
notebook > header tab button.flat:hover {
  color: currentColor;
  background-color: alpha(currentColor,0.15);
}
notebook > header tab button.flat:active {
  background-color: alpha(currentColor,0.3);
}
notebook > header tab button.flat, notebook > header tab button.flat:backdrop {
  color: alpha(currentColor,0.3);
}
notebook > header tab button.flat:last-child {
  margin-left: 4px;
  margin-right: -4px;
}
notebook > header tab button.flat:first-child {
  margin-left: -4px;
  margin-right: 4px;
}
notebook > header.top tabs, notebook > header.bottom tabs {
  padding-left: 4px;
  padding-right: 4px;
}
notebook > header.top tabs:not(:only-child), notebook > header.bottom tabs:not(:only-child) {
  margin-left: 3px;
  margin-right: 3px;
}
notebook > header.top tabs:not(:only-child):first-child, notebook > header.bottom tabs:not(:only-child):first-child {
  margin-left: -1px;
}
notebook > header.top tabs:not(:only-child):last-child, notebook > header.bottom tabs:not(:only-child):last-child {
  margin-right: -1px;
}
notebook > header.top tabs tab, notebook > header.bottom tabs tab {
  margin-left: 4px;
  margin-right: 4px;
}
notebook > header.top tabs tab.reorderable-page, notebook > header.bottom tabs tab.reorderable-page {
  border-style: none;
  margin-left: 0px;
  margin-right: 0px;
}
notebook > header.left tabs, notebook > header.right tabs {
  padding-top: 4px;
  padding-bottom: 4px;
}
notebook > header.left tabs:not(:only-child), notebook > header.right tabs:not(:only-child) {
  margin-top: 3px;
  margin-bottom: 3px;
}
notebook > header.left tabs:not(:only-child):first-child, notebook > header.right tabs:not(:only-child):first-child {
  margin-top: -1px;
}
notebook > header.left tabs:not(:only-child):last-child, notebook > header.right tabs:not(:only-child):last-child {
  margin-bottom: -1px;
}
notebook > header.left tabs tab, notebook > header.right tabs tab {
  margin-top: 4px;
  margin-bottom: 4px;
}
notebook > header.left tabs tab.reorderable-page, notebook > header.right tabs tab.reorderable-page {
  border-style: none solid;
  margin-top: 0px;
  margin-bottom: 0px;
}
notebook > header.top tab {
  padding-bottom: 4px;
}
notebook > header.bottom tab {
  padding-top: 4px;
}
notebook > stack:not(:only-child) {
  background-color: @view_bg_color;
}

* {
  -GtkScrollbar-has-backward-stepper: false;
  -GtkScrollbar-has-forward-stepper: false;
}

scrollbar {
  background-color: @view_bg_color;
  transition: all 200ms linear;
}
scrollbar slider {
  color: @window_fg_color;
  min-width: 8px;
  min-height: 8px;
  margin: -1px;
  border: 4px solid transparent;
  border-radius: 10px;
  background-clip: padding-box;
  background-color: alpha(currentColor,0.2);
  transition: all 200ms linear;
}
scrollbar slider:hover {
  background-color: alpha(currentColor,0.4);
}
scrollbar slider:hover:active {
  background-color: alpha(currentColor,0.6);
}
scrollbar slider:disabled {
  background-color: transparent;
}
scrollbar.fine-tune slider, scrollbar.fine-tune slider:hover, scrollbar.fine-tune slider:active {
  background-color: alpha(@accent_color,0.6);
}
scrollbar.overlay-indicator:not(.dragging):not(.hovering) {
  border-color: transparent;
  opacity: 0.4;
  background-color: transparent;
  transition-property: background-color, min-height, min-width;
}
scrollbar.overlay-indicator:not(.dragging):not(.hovering) slider {
  margin: 0;
  min-width: 3px;
  min-height: 3px;
  background-color: @window_fg_color;
  border: 1px solid white;
}
scrollbar.overlay-indicator:not(.dragging):not(.hovering) button {
  min-width: 5px;
  min-height: 5px;
  background-color: @window_fg_color;
  background-clip: padding-box;
  border-radius: 100%;
  border: 1px solid white;
  -gtk-icon-source: none;
}
scrollbar.overlay-indicator:not(.dragging):not(.hovering).horizontal slider {
  margin: 0 2px;
  min-width: 40px;
}
scrollbar.overlay-indicator:not(.dragging):not(.hovering).horizontal button {
  margin: 1px 2px;
  min-width: 5px;
}
scrollbar.overlay-indicator:not(.dragging):not(.hovering).vertical slider {
  margin: 2px 0;
  min-height: 40px;
}
scrollbar.overlay-indicator:not(.dragging):not(.hovering).vertical button {
  margin: 2px 1px;
  min-height: 5px;
}
scrollbar.overlay-indicator.dragging, scrollbar.overlay-indicator.hovering {
  opacity: 0.8;
}
scrollbar.horizontal slider {
  min-width: 40px;
}
scrollbar.vertical slider {
  min-height: 40px;
}
scrollbar button {
  padding: 0;
  min-width: 12px;
  min-height: 12px;
  border-style: none;
  border-radius: 0;
  transition-property: min-height, min-width, color;
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: alpha(currentColor,0.2);
}
scrollbar button:hover {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: alpha(currentColor,0.4);
}
scrollbar button:active, scrollbar button:checked {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: alpha(currentColor,0.6);
}
scrollbar button:disabled {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: alpha(currentColor,0.2);
}
scrollbar.vertical button.down {
  -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
}
scrollbar.vertical button.up {
  -gtk-icon-source: -gtk-icontheme("pan-up-symbolic");
}
scrollbar.horizontal button.down {
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
}
scrollbar.horizontal button.up {
  -gtk-icon-source: -gtk-icontheme("pan-start-symbolic");
}

treeview ~ scrollbar.vertical {
  border-top: 1px solid mix(currentColor,@window_bg_color,0.85);
  margin-top: -1px;
}

switch {
  outline-offset: -4px;
  padding: 3px;
  border-radius: 14px;
  color: @window_fg_color;
  background-color: alpha(currentColor,0.15);
}
switch:hover:not(:checked) {
  background-color: alpha(currentColor,0.2);
}
switch:checked {
  color: @accent_fg_color;
  background-color: @accent_bg_color;
}
switch:checked:hover {
  background-image: image(alpha(currentColor,0.1));
}
switch:checked:hover:active {
  background-image: image(rgba(0, 0, 0, 0.2));
}
switch:checked:disabled {
  background-color: alpha(@accent_bg_color,0.5);
}
switch:disabled {
  color: alpha(@window_fg_color,0.5);
  border-color: transparent;
  background-color: mix(mix(currentColor,@window_bg_color,0.81),@window_bg_color,0.3);
  text-shadow: none;
}
switch slider {
  margin: 0px;
  min-width: 20px;
  min-height: 20px;
  background-color: mix(white,@view_bg_color,0.2);
  border: 1px solid transparent;
  border-radius: 50%;
  box-shadow: 0 2px 4px rgba(0, 0, 0, 0.2);
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  -gtk-outline-radius: 20px;
}
switch image {
  color: transparent;
}
switch:hover slider {
  background-color: white;
}
switch:checked > slider {
  background-color: white;
}
switch:disabled slider {
  background-color: mix(@view_bg_color,mix(white,@view_bg_color,0.2),0.5);
  box-shadow: none;
}
row:selected switch {
  box-shadow: none;
  border-color: mix(black,@accent_bg_color,0.8);
}
row:selected switch:backdrop {
  border-color: mix(black,@accent_bg_color,0.8);
}
row:selected switch > slider:checked, row:selected switch > slider {
  border-color: mix(black,@accent_bg_color,0.8);
}

.view.content-view.check:not(list),
.content-view:not(list) check {
  margin: 4px;
  min-width: 32px;
  min-height: 32px;
  color: transparent;
  background-color: @accent_bg_color;
  border-radius: 5px;
  background-image: none;
  transition: 200ms;
  box-shadow: none;
  border-width: 0;
  -gtk-icon-source: none;
  -gtk-icon-shadow: none;
}

.view.content-view.check:hover:not(list),
.content-view:not(list) check:hover {
  margin: 4px;
  min-width: 32px;
  min-height: 32px;
  color: transparent;
  background-color: @accent_bg_color;
  border-radius: 5px;
  background-image: none;
  transition: 200ms;
  box-shadow: none;
  border-width: 0;
  -gtk-icon-source: none;
  -gtk-icon-shadow: none;
}

.view.content-view.check:active:not(list),
.content-view:not(list) check:active {
  margin: 4px;
  min-width: 32px;
  min-height: 32px;
  color: transparent;
  background-color: @accent_bg_color;
  border-radius: 5px;
  background-image: none;
  transition: 200ms;
  box-shadow: none;
  border-width: 0;
  -gtk-icon-source: none;
  -gtk-icon-shadow: none;
}

.view.content-view.check:backdrop:not(list),
.content-view:not(list) check:backdrop {
  margin: 4px;
  min-width: 32px;
  min-height: 32px;
  color: transparent;
  background-color: alpha(@window_fg_color,0.5);
  border-radius: 5px;
  background-image: none;
  transition: 200ms;
  box-shadow: none;
  border-width: 0;
  -gtk-icon-source: none;
  -gtk-icon-shadow: none;
}

.view.content-view.check:checked:not(list),
.content-view:not(list) check:checked {
  margin: 4px;
  min-width: 32px;
  min-height: 32px;
  color: rgba(255, 255, 255, 0.9);
  background-color: @accent_bg_color;
  border-radius: 5px;
  background-image: none;
  transition: 200ms;
  box-shadow: none;
  border-width: 0;
  -gtk-icon-source: -gtk-icontheme('object-select-symbolic');
  -gtk-icon-shadow: none;
}

.view.content-view.check:checked:hover:not(list),
.content-view:not(list) check:checked:hover {
  margin: 4px;
  min-width: 32px;
  min-height: 32px;
  color: rgba(255, 255, 255, 0.9);
  background-color: @accent_bg_color;
  border-radius: 5px;
  background-image: none;
  transition: 200ms;
  box-shadow: none;
  border-width: 0;
  -gtk-icon-source: -gtk-icontheme('object-select-symbolic');
  -gtk-icon-shadow: none;
}

.view.content-view.check:checked:active:not(list),
.content-view:not(list) check:checked:active {
  margin: 4px;
  min-width: 32px;
  min-height: 32px;
  color: rgba(255, 255, 255, 0.9);
  background-color: @accent_bg_color;
  border-radius: 5px;
  background-image: none;
  transition: 200ms;
  box-shadow: none;
  border-width: 0;
  -gtk-icon-source: -gtk-icontheme('object-select-symbolic');
  -gtk-icon-shadow: none;
}

.view.content-view.check:backdrop:checked:not(list),
.content-view:not(list) check:backdrop:checked {
  margin: 4px;
  min-width: 32px;
  min-height: 32px;
  color: rgba(255, 255, 255, 0.7);
  background-color: alpha(@window_fg_color,0.5);
  border-radius: 5px;
  background-image: none;
  transition: 200ms;
  box-shadow: none;
  border-width: 0;
  -gtk-icon-source: -gtk-icontheme('object-select-symbolic');
  -gtk-icon-shadow: none;
}

checkbutton.text-button, radiobutton.text-button {
  padding: 2px 0;
  outline-offset: 0;
}
checkbutton.text-button label:not(:only-child):first-child, radiobutton.text-button label:not(:only-child):first-child {
  margin-left: 4px;
}
checkbutton.text-button label:not(:only-child):last-child, radiobutton.text-button label:not(:only-child):last-child {
  margin-right: 4px;
}

check,
radio {
  margin: 0 4px;
  padding: 1px;
  min-height: 14px;
  min-width: 14px;
  border: 2px solid;
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  -gtk-icon-source: none;
}
check:only-child,
radio:only-child {
  margin: 0;
}
popover check.left:dir(rtl),
popover radio.left:dir(rtl) {
  margin-left: 0;
  margin-right: 12px;
}
popover check.right:dir(ltr),
popover radio.right:dir(ltr) {
  margin-left: 12px;
  margin-right: 0;
}
check,
radio {
  background-clip: padding-box;
  background-image: image(transparent);
  border-color: alpha(currentColor,0.15);
  box-shadow: none;
  color: @window_fg_color;
}
check:hover:not(:checked):not(:indeterminate),
radio:hover:not(:checked):not(:indeterminate) {
  border-color: alpha(currentColor,0.2);
}
check:active,
radio:active {
  box-shadow: none;
}
check:disabled,
radio:disabled {
  box-shadow: none;
  background-image: image(transparent);
  color: alpha(@window_fg_color,0.5);
  border-color: alpha(currentColor,0.15);
}
check:checked,
radio:checked {
  background-clip: border-box;
  background-image: image(@accent_bg_color);
  border-color: @accent_bg_color;
  box-shadow: none;
  color: @accent_fg_color;
}
check:checked:hover:not(:checked):not(:indeterminate),
radio:checked:hover:not(:checked):not(:indeterminate) {
  border-color: alpha(currentColor,0.2);
}
check:checked:active,
radio:checked:active {
  box-shadow: none;
}
check:checked:disabled,
radio:checked:disabled {
  box-shadow: none;
  background-image: image(@accent_bg_color);
  color: alpha(@accent_fg_color,0.5);
  border-color: @accent_bg_color;
}
check:indeterminate,
radio:indeterminate {
  background-clip: border-box;
  background-image: image(@accent_bg_color);
  border-color: @accent_bg_color;
  box-shadow: none;
  color: @accent_fg_color;
}
check:indeterminate:hover:not(:checked):not(:indeterminate),
radio:indeterminate:hover:not(:checked):not(:indeterminate) {
  border-color: alpha(currentColor,0.2);
}
check:indeterminate:active,
radio:indeterminate:active {
  box-shadow: none;
}
check:indeterminate:disabled,
radio:indeterminate:disabled {
  box-shadow: none;
  background-image: image(@accent_bg_color);
  color: alpha(@accent_fg_color,0.5);
  border-color: @accent_bg_color;
}
row:selected check,
row:selected radio {
  border-color: mix(black,@accent_bg_color,0.8);
}
.osd check,
.osd radio {
  color: white;
  border-color: transparent;
  background-color: rgba(0, 0, 0, 0.65);
  background-clip: padding-box;
}
.osd check:hover,
.osd radio:hover {
  color: white;
  border-color: transparent;
  background-color: rgba(0, 0, 0, 0.65);
  background-clip: padding-box;
}
.osd check:active,
.osd radio:active {
  color: white;
  border-color: transparent;
  background-color: mix(black,currentColor,0.2);
  background-clip: padding-box;
}
.osd check:backdrop,
.osd radio:backdrop {
  color: rgba(255, 255, 255, 0.9);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.7));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.osd check:disabled,
.osd radio:disabled {
  color: rgba(255, 255, 255, 0.5);
  border-color: transparent;
  background-color: transparent;
  background-image: image(rgba(0, 0, 0, 0.5));
  background-clip: padding-box;
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
menu menuitem check,
menu menuitem radio {
  margin: 0;
  padding: 0;
}
menu menuitem check, menu menuitem check:hover, menu menuitem check:disabled, menu menuitem check:not(:checked), menu menuitem check:not(:checked):hover, menu menuitem check:not(:checked):disabled, menu menuitem check:checked, menu menuitem check:checked:hover, menu menuitem check:checked:disabled, menu menuitem check:indeterminate, menu menuitem check:indeterminate:hover, menu menuitem check:indeterminate:disabled,
menu menuitem radio,
menu menuitem radio:hover,
menu menuitem radio:disabled,
menu menuitem radio:not(:checked),
menu menuitem radio:not(:checked):hover,
menu menuitem radio:not(:checked):disabled,
menu menuitem radio:checked,
menu menuitem radio:checked:hover,
menu menuitem radio:checked:disabled,
menu menuitem radio:indeterminate,
menu menuitem radio:indeterminate:hover,
menu menuitem radio:indeterminate:disabled {
  min-height: 14px;
  min-width: 14px;
  padding: 1px;
  background-image: none;
  background-color: transparent;
  box-shadow: none;
  -gtk-icon-shadow: none;
  color: inherit;
  border-width: 1px;
  border-color: alpha(currentColor,0.4);
}

check {
  border-radius: 6px;
}
check:checked {
  -gtk-icon-source: image(-gtk-recolor(url("assets/check-symbolic.svg")), -gtk-recolor(url("assets/check-symbolic.symbolic.png")));
}
check:indeterminate {
  -gtk-icon-source: image(-gtk-recolor(url("assets/dash-symbolic.svg")), -gtk-recolor(url("assets/dash-symbolic.symbolic.png")));
}
menu menuitem check,
menu menuitem check:hover,
menu menuitem check:disabled,
menu menuitem check:checked,
menu menuitem check:checked:hover,
menu menuitem check:checked:disabled,
menu menuitem check:indeterminate,
menu menuitem check:indeterminate:hover,
menu menuitem check:indeterminate:disabled {
  border: none;
}

treeview.view radio:selected:focus, treeview.view radio:selected,
radio {
  border-radius: 100%;
}
treeview.view radio:checked:selected,
radio:checked {
  -gtk-icon-source: image(-gtk-recolor(url("assets/bullet-symbolic.svg")), -gtk-recolor(url("assets/bullet-symbolic.symbolic.png")));
}
treeview.view radio:indeterminate:selected,
radio:indeterminate {
  -gtk-icon-source: image(-gtk-recolor(url("assets/dash-symbolic.svg")), -gtk-recolor(url("assets/dash-symbolic.symbolic.png")));
}

menu menuitem radio:checked:not(:backdrop), menu menuitem radio:indeterminate:not(:backdrop),
menu menuitem check:checked:not(:backdrop),
menu menuitem check:indeterminate:not(:backdrop) {
  transition: none;
}

treeview.view check:selected:focus, treeview.view check:selected,
treeview.view radio:selected:focus,
treeview.view radio:selected {
  color: @accent_fg_color;
  border-color: mix(black,@accent_bg_color,0.85);
}

treeview.view check:selected:focus, treeview.view check:selected {
  border-radius: 6px;
}

progressbar trough, scale fill, scale trough {
  border-radius: 9px;
  background-color: alpha(currentColor,0.15);
}
progressbar trough:disabled, scale fill:disabled, scale trough:disabled {
  background-color: alpha(alpha(currentColor,0.15),0.5);
}
row:selected progressbar trough, progressbar row:selected trough, row:selected scale fill, scale row:selected fill, row:selected scale trough, scale row:selected trough {
  border-color: mix(black,@accent_bg_color,0.85);
}
.osd progressbar trough, progressbar .osd trough, .osd scale fill, scale .osd fill, .osd scale trough, scale .osd trough {
  border-color: rgba(0, 0, 0, 0.9);
  background-color: rgba(0, 0, 0, 0.7);
}
.osd progressbar trough:disabled, progressbar .osd trough:disabled, .osd scale fill:disabled, scale .osd fill:disabled, .osd scale trough:disabled, scale .osd trough:disabled {
  background-color: rgba(0, 0, 0, 0.5);
}

progressbar progress, scale highlight {
  border-radius: 9px;
  background-color: @accent_bg_color;
  color: @accent_fg_color;
}
progressbar progress:disabled, scale highlight:disabled {
  background-color: mix(@accent_bg_color,@view_bg_color,0.5);
  border-color: transparent;
}
row:selected progressbar progress, progressbar row:selected progress, row:selected scale highlight, scale row:selected highlight {
  border-color: mix(black,@accent_bg_color,0.85);
}
.osd progressbar progress, progressbar .osd progress, .osd scale highlight, scale .osd highlight {
  border-color: rgba(0, 0, 0, 0.9);
}
.osd progressbar progress:disabled, progressbar .osd progress:disabled, .osd scale highlight:disabled, scale .osd highlight:disabled {
  border-color: transparent;
}

scale {
  min-height: 10px;
  min-width: 10px;
  padding: 12px;
}
scale slider {
  min-height: 18px;
  min-width: 18px;
  margin: -9px;
}
scale.fine-tune.horizontal {
  padding-top: 9px;
  padding-bottom: 9px;
  min-height: 16px;
}
scale.fine-tune.vertical {
  padding-left: 9px;
  padding-right: 9px;
  min-width: 16px;
}
scale.fine-tune slider {
  margin: -6px;
}
scale.fine-tune fill,
scale.fine-tune highlight,
scale.fine-tune trough {
  border-radius: 5px;
  -gtk-outline-radius: 7px;
}
scale trough {
  outline-offset: 2px;
  -gtk-outline-radius: 5px;
}
scale.horizontal trough {
  min-height: 4px;
}
scale.vertical trough {
  min-width: 4px;
}
scale fill:backdrop, scale fill {
  background-color: mix(currentColor,@window_bg_color,0.85);
}
scale fill:disabled:backdrop, scale fill:disabled {
  border-color: transparent;
  background-color: transparent;
}
.osd scale fill {
  background-color: mix(rgba(255, 255, 255, 0.9),rgba(0, 0, 0, 0.9),0.75);
}
.osd scale fill:disabled:backdrop, .osd scale fill:disabled {
  border-color: transparent;
  background-color: transparent;
}
scale slider {
  background-color: mix(white,@view_bg_color,0.2);
  box-shadow: 0 1px 2px 1px rgba(0, 0, 0, 0.25);
  border: 1px solid transparent;
  border-radius: 100%;
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  transition-property: background, border, box-shadow;
}
scale slider:hover {
  background-color: white;
}
scale slider:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
  background-color: mix(@view_bg_color,mix(white,@view_bg_color,0.2),0.5);
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.15);
}
scale slider:backdrop {
  transition: 200ms ease-out;
  box-shadow: 0 1px 2px 1px rgba(0, 0, 0, 0.15);
}
scale slider:backdrop:disabled {
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.15);
}
row:selected scale slider:disabled, row:selected scale slider {
  border-color: mix(black,@accent_bg_color,0.85);
}
.osd scale slider {
  background-color: mix(white,@view_bg_color,0.2);
  box-shadow: 0 1px 2px 1px rgba(0, 0, 0, 0.25);
  border: 1px solid transparent;
}
.osd scale slider:hover {
  background-color: white;
}
.osd scale slider:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.2);
}
.osd scale slider:backdrop {
  transition: 200ms ease-out;
  box-shadow: 0 1px 2px 1px rgba(0, 0, 0, 0.2);
}
.osd scale slider:backdrop:disabled {
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.2);
}
scale marks,
scale value {
  color: alpha(currentColor,0.55);
  font-feature-settings: "tnum";
}
scale.horizontal marks.top {
  margin-bottom: 6px;
  margin-top: -12px;
}
scale.horizontal.fine-tune marks.top {
  margin-bottom: 6px;
  margin-top: -9px;
}
scale.horizontal marks.bottom {
  margin-top: 6px;
  margin-bottom: -12px;
}
scale.horizontal.fine-tune marks.bottom {
  margin-top: 6px;
  margin-bottom: -9px;
}
scale.vertical marks.top {
  margin-right: 6px;
  margin-left: -12px;
}
scale.vertical.fine-tune marks.top {
  margin-right: 6px;
  margin-left: -9px;
}
scale.vertical marks.bottom {
  margin-left: 6px;
  margin-right: -12px;
}
scale.vertical.fine-tune marks.bottom {
  margin-left: 6px;
  margin-right: -9px;
}
scale.horizontal indicator {
  min-height: 6px;
  min-width: 1px;
}
scale.horizontal.fine-tune indicator {
  min-height: 3px;
}
scale.vertical indicator {
  min-height: 1px;
  min-width: 6px;
}
scale.vertical.fine-tune indicator {
  min-width: 3px;
}
scale.horizontal.marks-before:not(.marks-after) slider {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above.png"), url("assets/slider-horz-scale-has-marks-above@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-top: -14px;
  background-position: top;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-top: -11px;
}
scale.horizontal.marks-before:not(.marks-after) slider:hover {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above.png"), url("assets/slider-horz-scale-has-marks-above@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-top: -14px;
  background-position: top;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-top: -11px;
}
scale.horizontal.marks-before:not(.marks-after) slider:disabled {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-disabled.png"), url("assets/slider-horz-scale-has-marks-above-disabled@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-top: -14px;
  background-position: top;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-top: -11px;
}
scale.horizontal.marks-before:not(.marks-after) slider:backdrop {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-disabled.png"), url("assets/slider-horz-scale-has-marks-above-disabled@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-top: -14px;
  background-position: top;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-top: -11px;
}
scale.horizontal.marks-before:not(.marks-after) slider:backdrop:disabled {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above-disabled.png"), url("assets/slider-horz-scale-has-marks-above-disabled@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-top: -14px;
  background-position: top;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-top: -11px;
}
scale.horizontal.marks-after:not(.marks-before) slider {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below.png"), url("assets/slider-horz-scale-has-marks-below@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-bottom: -14px;
  background-position: bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-bottom: -11px;
}
scale.horizontal.marks-after:not(.marks-before) slider:hover {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below.png"), url("assets/slider-horz-scale-has-marks-below@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-bottom: -14px;
  background-position: bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-bottom: -11px;
}
scale.horizontal.marks-after:not(.marks-before) slider:disabled {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below-disabled.png"), url("assets/slider-horz-scale-has-marks-below-disabled@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-bottom: -14px;
  background-position: bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-bottom: -11px;
}
scale.horizontal.marks-after:not(.marks-before) slider:backdrop {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below-disabled.png"), url("assets/slider-horz-scale-has-marks-below-disabled@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-bottom: -14px;
  background-position: bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-bottom: -11px;
}
scale.horizontal.marks-after:not(.marks-before) slider:backdrop:disabled {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-horz-scale-has-marks-below-disabled.png"), url("assets/slider-horz-scale-has-marks-below-disabled@2.png"));
  min-height: 28px;
  min-width: 23px;
  margin-bottom: -14px;
  background-position: bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.horizontal.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-bottom: -11px;
}
scale.vertical.marks-before:not(.marks-after) slider {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above.png"), url("assets/slider-vert-scale-has-marks-above@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-left: -14px;
  background-position: left bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-left: -11px;
}
scale.vertical.marks-before:not(.marks-after) slider:hover {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above.png"), url("assets/slider-vert-scale-has-marks-above@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-left: -14px;
  background-position: left bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-left: -11px;
}
scale.vertical.marks-before:not(.marks-after) slider:disabled {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above-disabled.png"), url("assets/slider-vert-scale-has-marks-above-disabled@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-left: -14px;
  background-position: left bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-left: -11px;
}
scale.vertical.marks-before:not(.marks-after) slider:backdrop {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above-disabled.png"), url("assets/slider-vert-scale-has-marks-above-disabled@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-left: -14px;
  background-position: left bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-left: -11px;
}
scale.vertical.marks-before:not(.marks-after) slider:backdrop:disabled {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-above-disabled.png"), url("assets/slider-vert-scale-has-marks-above-disabled@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-left: -14px;
  background-position: left bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-before:not(.marks-after).fine-tune slider {
  margin: -7px;
  margin-left: -11px;
}
scale.vertical.marks-after:not(.marks-before) slider {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below.png"), url("assets/slider-vert-scale-has-marks-below@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-right: -14px;
  background-position: right bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-right: -11px;
}
scale.vertical.marks-after:not(.marks-before) slider:hover {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below.png"), url("assets/slider-vert-scale-has-marks-below@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-right: -14px;
  background-position: right bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-right: -11px;
}
scale.vertical.marks-after:not(.marks-before) slider:disabled {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below-disabled.png"), url("assets/slider-vert-scale-has-marks-below-disabled@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-right: -14px;
  background-position: right bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-right: -11px;
}
scale.vertical.marks-after:not(.marks-before) slider:backdrop {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below-disabled.png"), url("assets/slider-vert-scale-has-marks-below-disabled@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-right: -14px;
  background-position: right bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-right: -11px;
}
scale.vertical.marks-after:not(.marks-before) slider:backdrop:disabled {
  margin: -10px;
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  background-image: -gtk-scaled(url("assets/slider-vert-scale-has-marks-below-disabled.png"), url("assets/slider-vert-scale-has-marks-below-disabled@2.png"));
  min-height: 23px;
  min-width: 28px;
  margin-right: -14px;
  background-position: right bottom;
  background-repeat: no-repeat;
  box-shadow: none;
}
scale.vertical.marks-after:not(.marks-before).fine-tune slider {
  margin: -7px;
  margin-right: -11px;
}
scale.color {
  min-height: 0;
  min-width: 0;
}
scale.color trough {
  background-image: image(mix(currentColor,@window_bg_color,0.85));
  background-repeat: no-repeat;
}
scale.color.horizontal {
  padding: 0 0 15px 0;
}
scale.color.horizontal trough {
  padding-bottom: 4px;
  background-position: 0 -3px;
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}
scale.color.horizontal slider:dir(ltr):hover, scale.color.horizontal slider:dir(ltr):backdrop, scale.color.horizontal slider:dir(ltr):disabled, scale.color.horizontal slider:dir(ltr):backdrop:disabled, scale.color.horizontal slider:dir(ltr), scale.color.horizontal slider:dir(rtl):hover, scale.color.horizontal slider:dir(rtl):backdrop, scale.color.horizontal slider:dir(rtl):disabled, scale.color.horizontal slider:dir(rtl):backdrop:disabled, scale.color.horizontal slider:dir(rtl) {
  margin-bottom: -15px;
  margin-top: 6px;
}
scale.color.vertical:dir(ltr) {
  padding: 0 0 0 15px;
}
scale.color.vertical:dir(ltr) trough {
  padding-left: 4px;
  background-position: 3px 0;
  border-bottom-right-radius: 0;
  border-top-right-radius: 0;
}
scale.color.vertical:dir(ltr) slider:hover, scale.color.vertical:dir(ltr) slider:backdrop, scale.color.vertical:dir(ltr) slider:disabled, scale.color.vertical:dir(ltr) slider:backdrop:disabled, scale.color.vertical:dir(ltr) slider {
  margin-left: -15px;
  margin-right: 6px;
}
scale.color.vertical:dir(rtl) {
  padding: 0 15px 0 0;
}
scale.color.vertical:dir(rtl) trough {
  padding-right: 4px;
  background-position: -3px 0;
  border-bottom-left-radius: 0;
  border-top-left-radius: 0;
}
scale.color.vertical:dir(rtl) slider:hover, scale.color.vertical:dir(rtl) slider:backdrop, scale.color.vertical:dir(rtl) slider:disabled, scale.color.vertical:dir(rtl) slider:backdrop:disabled, scale.color.vertical:dir(rtl) slider {
  margin-right: -15px;
  margin-left: 6px;
}
scale.color.fine-tune.horizontal:dir(ltr), scale.color.fine-tune.horizontal:dir(rtl) {
  padding: 0 0 12px 0;
}
scale.color.fine-tune.horizontal:dir(ltr) trough, scale.color.fine-tune.horizontal:dir(rtl) trough {
  padding-bottom: 7px;
  background-position: 0 -6px;
}
scale.color.fine-tune.horizontal:dir(ltr) slider, scale.color.fine-tune.horizontal:dir(rtl) slider {
  margin-bottom: -15px;
  margin-top: 6px;
}
scale.color.fine-tune.vertical:dir(ltr) {
  padding: 0 0 0 12px;
}
scale.color.fine-tune.vertical:dir(ltr) trough {
  padding-left: 7px;
  background-position: 6px 0;
}
scale.color.fine-tune.vertical:dir(ltr) slider {
  margin-left: -15px;
  margin-right: 6px;
}
scale.color.fine-tune.vertical:dir(rtl) {
  padding: 0 12px 0 0;
}
scale.color.fine-tune.vertical:dir(rtl) trough {
  padding-right: 7px;
  background-position: -6px 0;
}
scale.color.fine-tune.vertical:dir(rtl) slider {
  margin-right: -15px;
  margin-left: 6px;
}

progressbar.horizontal trough,
progressbar.horizontal progress {
  min-height: 8px;
}
progressbar.vertical trough,
progressbar.vertical progress {
  min-width: 8px;
}
progressbar.horizontal progress {
  margin: 0 -1px;
}
progressbar.vertical progress {
  margin: -1px 0;
}
progressbar {
  font-size: smaller;
  color: alpha(@window_fg_color,0.4);
  font-feature-settings: "tnum";
}
progressbar:backdrop {
  box-shadow: none;
  transition: 200ms ease-out;
}
progressbar progress {
  border-radius: 8px;
}
progressbar progress.left {
  border-top-left-radius: 8px;
  border-bottom-left-radius: 8px;
}
progressbar progress.right {
  border-top-right-radius: 8px;
  border-bottom-right-radius: 8px;
}
progressbar progress.top {
  border-top-right-radius: 8px;
  border-top-left-radius: 8px;
}
progressbar progress.bottom {
  border-bottom-right-radius: 8px;
  border-bottom-left-radius: 8px;
}
progressbar.osd {
  min-width: 3px;
  min-height: 3px;
  background-color: transparent;
}
progressbar.osd trough {
  border-style: none;
  border-radius: 0;
  background-color: transparent;
  box-shadow: none;
}
progressbar.osd progress {
  border-style: none;
  border-radius: 0;
}
progressbar trough.empty progress {
  all: unset;
}

levelbar.horizontal block {
  min-height: 8px;
}
levelbar.horizontal.continuous block:first-child, levelbar.horizontal.continuous block:last-child {
  border-radius: 8px;
}
levelbar.horizontal.discrete block {
  margin: 0 1px;
  min-width: 32px;
}
levelbar.horizontal.discrete block:first-child {
  border-radius: 8px 0 0 8px;
}
levelbar.horizontal.discrete block:last-child {
  border-radius: 0 8px 8px 0;
}
levelbar.vertical block {
  min-width: 8px;
}
levelbar.vertical.continuous block:first-child, levelbar.vertical.continuous block:last-child {
  border-radius: 8px;
}
levelbar.vertical.discrete block {
  margin: 1px 0;
  min-height: 32px;
}
levelbar.vertical.discrete block:first-child {
  border-radius: 8px 8px 0 0;
}
levelbar.vertical.discrete block:last-child {
  border-radius: 0 0 8px 8px;
}
levelbar:backdrop {
  transition: 200ms ease-out;
}
levelbar trough {
  border: 0px solid;
  padding: 0;
  border-radius: 8px;
  background-color: alpha(currentColor,0.15);
}
levelbar trough:backdrop {
  background-color: mix(mix(currentColor,@window_bg_color,0.81),@window_bg_color,0.3);
}
levelbar block {
  border: 0px solid;
  border-radius: 0;
}
levelbar block.low {
  border-color: shade(@warning_bg_color,0.8);
  background-color: @warning_bg_color;
}
levelbar block.low:backdrop {
  border-color: @warning_bg_color;
}
levelbar block.high, levelbar block:not(.empty) {
  border-color: shade(@accent_bg_color,0.8);
  background-color: @accent_bg_color;
}
levelbar block.high:backdrop, levelbar block:not(.empty):backdrop {
  border-color: @accent_bg_color;
}
levelbar block.full {
  border-color: shade(@success_bg_color,0.8);
  background-color: @success_bg_color;
}
levelbar block.full:backdrop {
  border-color: @success_bg_color;
}
levelbar block.empty {
  background-color: transparent;
  border-color: alpha(@window_fg_color,0.2);
}
levelbar block.empty:backdrop {
  border-color: alpha(mix(@window_fg_color,@window_bg_color,0.5),0.15);
}

printdialog paper {
  color: @window_fg_color;
  border: 1px solid mix(currentColor,@window_bg_color,0.85);
  background: white;
  padding: 0;
  border-radius: 0;
  box-shadow: none;
}
printdialog paper:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
  border-color: mix(currentColor,@window_bg_color,0.81);
}
printdialog .dialog-action-box {
  margin: 12px;
}

frame > border,
.frame {
  box-shadow: none;
  margin: 0;
  padding: 0;
  border: 1px solid mix(currentColor,@window_bg_color,0.85);
}
frame > border.flat,
.frame.flat {
  border-style: none;
}

actionbar > revealer > box {
  padding: 6px;
  border-top: 1px solid mix(currentColor,@window_bg_color,0.85);
}

scrolledwindow viewport.frame {
  border-style: none;
}
scrolledwindow overshoot.top {
  background-image: -gtk-gradient(radial, center top, 0, center top, 0.5, to(@window_fg_color), to(alpha(@window_fg_color,0))), -gtk-gradient(radial, center top, 0, center top, 0.6, from(alpha(@window_fg_color,0.07)), to(alpha(@window_fg_color,0)));
  background-size: 100% 5%, 100% 100%;
  background-repeat: no-repeat;
  background-position: center top;
  background-color: transparent;
  border: none;
  box-shadow: none;
}
scrolledwindow overshoot.top:backdrop {
  background-image: -gtk-gradient(radial, center top, 0, center top, 0.5, to(@window_fg_color), to(alpha(@window_fg_color,0)));
  background-size: 100% 5%;
  background-repeat: no-repeat;
  background-position: center top;
  background-color: transparent;
  border: none;
  box-shadow: none;
}
scrolledwindow overshoot.bottom {
  background-image: -gtk-gradient(radial, center bottom, 0, center bottom, 0.5, to(@window_fg_color), to(alpha(@window_fg_color,0))), -gtk-gradient(radial, center bottom, 0, center bottom, 0.6, from(alpha(@window_fg_color,0.07)), to(alpha(@window_fg_color,0)));
  background-size: 100% 5%, 100% 100%;
  background-repeat: no-repeat;
  background-position: center bottom;
  background-color: transparent;
  border: none;
  box-shadow: none;
}
scrolledwindow overshoot.bottom:backdrop {
  background-image: -gtk-gradient(radial, center bottom, 0, center bottom, 0.5, to(@window_fg_color), to(alpha(@window_fg_color,0)));
  background-size: 100% 5%;
  background-repeat: no-repeat;
  background-position: center bottom;
  background-color: transparent;
  border: none;
  box-shadow: none;
}
scrolledwindow overshoot.left {
  background-image: -gtk-gradient(radial, left center, 0, left center, 0.5, to(@window_fg_color), to(alpha(@window_fg_color,0))), -gtk-gradient(radial, left center, 0, left center, 0.6, from(alpha(@window_fg_color,0.07)), to(alpha(@window_fg_color,0)));
  background-size: 5% 100%, 100% 100%;
  background-repeat: no-repeat;
  background-position: left center;
  background-color: transparent;
  border: none;
  box-shadow: none;
}
scrolledwindow overshoot.left:backdrop {
  background-image: -gtk-gradient(radial, left center, 0, left center, 0.5, to(@window_fg_color), to(alpha(@window_fg_color,0)));
  background-size: 5% 100%;
  background-repeat: no-repeat;
  background-position: left center;
  background-color: transparent;
  border: none;
  box-shadow: none;
}
scrolledwindow overshoot.right {
  background-image: -gtk-gradient(radial, right center, 0, right center, 0.5, to(@window_fg_color), to(alpha(@window_fg_color,0))), -gtk-gradient(radial, right center, 0, right center, 0.6, from(alpha(@window_fg_color,0.07)), to(alpha(@window_fg_color,0)));
  background-size: 5% 100%, 100% 100%;
  background-repeat: no-repeat;
  background-position: right center;
  background-color: transparent;
  border: none;
  box-shadow: none;
}
scrolledwindow overshoot.right:backdrop {
  background-image: -gtk-gradient(radial, right center, 0, right center, 0.5, to(@window_fg_color), to(alpha(@window_fg_color,0)));
  background-size: 5% 100%;
  background-repeat: no-repeat;
  background-position: right center;
  background-color: transparent;
  border: none;
  box-shadow: none;
}
scrolledwindow junction {
  border-color: transparent;
  border-image: linear-gradient(to bottom, mix(currentColor,@window_bg_color,0.85) 1px, transparent 1px) 0 0 0 1/0 1px stretch;
  background-color: @view_bg_color;
}
scrolledwindow junction:dir(rtl) {
  border-image-slice: 0 1 0 0;
}

separator {
  background: mix(currentColor,@window_bg_color,0.85);
  min-width: 1px;
  min-height: 1px;
}

list {
  color: @view_fg_color;
  background-color: @view_bg_color;
  border-color: @card_shade_color;
}
list row {
  padding: 2px;
  outline-color: alpha(@accent_color,0.5);
  outline-style: solid;
  outline-offset: -3px;
  outline-width: 2px;
  -gtk-outline-radius: 9px;
}

row {
  transition: all 150ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}
row:hover {
  transition: none;
}
row:backdrop {
  transition: 200ms ease-out;
}
row.activatable.has-open-popup, row.activatable:hover {
  background-color: alpha(currentColor,0.07);
}
row.activatable:active {
  background-color: alpha(currentColor,0.16);
}
row.activatable:backdrop:hover {
  background-color: transparent;
}
row.activatable:selected {
  color: @window_fg_color;
}
row.activatable:selected:active {
  background-color: alpha(currentColor,0.19);
}
row.activatable:selected.has-open-popup, row.activatable:selected:hover {
  background-color: alpha(currentColor,0.12);
  color: @window_fg_color;
}
row.activatable:selected:backdrop {
  background-color: alpha(currentColor,0.07);
  color: @window_fg_color;
}
row:selected {
  background-color: alpha(currentColor,0.1);
}

.content list button, .content list > row:not(:selected):hover button,
list.content button,
list.content > row:not(:selected):hover button,
list.view.frame button,
list.view.frame > row:not(:selected):hover button {
  color: @card_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@card_fg_color,@card_bg_color,0.9);
}
.content list button.flat:not(:hover):not(:active), .content list > row:not(:selected):hover button.flat:not(:hover):not(:active),
list.content button.flat:not(:hover):not(:active),
list.content > row:not(:selected):hover button.flat:not(:hover):not(:active),
list.view.frame button.flat:not(:hover):not(:active),
list.view.frame > row:not(:selected):hover button.flat:not(:hover):not(:active) {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.content list button:hover, .content list > row:not(:selected):hover button:hover,
list.content button:hover,
list.content > row:not(:selected):hover button:hover,
list.view.frame button:hover,
list.view.frame > row:not(:selected):hover button:hover {
  color: @card_fg_color;
  background-color: mix(@card_fg_color,@card_bg_color,0.85);
  box-shadow: none;
}
.content list button:active, .content list button:checked, .content list > row:not(:selected):hover button:active, .content list > row:not(:selected):hover button:checked,
list.content button:active,
list.content button:checked,
list.content > row:not(:selected):hover button:active,
list.content > row:not(:selected):hover button:checked,
list.view.frame button:active,
list.view.frame button:checked,
list.view.frame > row:not(:selected):hover button:active,
list.view.frame > row:not(:selected):hover button:checked {
  color: @card_fg_color;
  background-color: mix(@card_fg_color,@card_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.content list button:active:hover, .content list button:checked:hover, .content list > row:not(:selected):hover button:active:hover, .content list > row:not(:selected):hover button:checked:hover,
list.content button:active:hover,
list.content button:checked:hover,
list.content > row:not(:selected):hover button:active:hover,
list.content > row:not(:selected):hover button:checked:hover,
list.view.frame button:active:hover,
list.view.frame button:checked:hover,
list.view.frame > row:not(:selected):hover button:active:hover,
list.view.frame > row:not(:selected):hover button:checked:hover {
  background-color: mix(@card_fg_color,@card_bg_color,0.65);
}
.content list button:disabled, .content list > row:not(:selected):hover button:disabled,
list.content button:disabled,
list.content > row:not(:selected):hover button:disabled,
list.view.frame button:disabled,
list.view.frame > row:not(:selected):hover button:disabled {
  color: alpha(@card_fg_color,0.5);
  background-color: mix(@card_fg_color,@card_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}
.content list entry,
list.content entry,
list.view.frame entry {
  color: @card_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@card_fg_color,@card_bg_color,0.9);
}
.content list entry:disabled,
list.content entry:disabled,
list.view.frame entry:disabled {
  color: alpha(@card_fg_color,0.5);
  background-color: mix(@card_fg_color,@card_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
}

list.content .horizontal spinbutton entry {
  border-top-left-radius: 9px;
  border-bottom-left-radius: 9px;
}

list.view.frame {
  border-radius: 12px;
}
list.view.frame, list.view.frame:backdrop {
  background-color: @card_bg_color;
}

.app-notification,
.app-notification.frame {
  padding: 10px;
  border-radius: 0 0 15px 15px;
  background-color: rgba(0, 0, 0, 0.7);
  background-image: linear-gradient(to bottom, rgba(0, 0, 0, 0.2), transparent 2px);
  background-clip: padding-box;
  border: none;
}
.app-notification:backdrop,
.app-notification.frame:backdrop {
  background-image: none;
  transition: 200ms ease-out;
}
.app-notification border,
.app-notification.frame border {
  border: none;
}

expander title > arrow {
  min-width: 16px;
  min-height: 16px;
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
}
expander title > arrow:dir(rtl) {
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic-rtl");
}
expander title > arrow:hover {
  color: mix(white,@window_fg_color,0.7);
}
expander title > arrow:disabled {
  color: alpha(@window_fg_color,0.5);
}
expander title > arrow:disabled:backdrop {
  color: mix(mix(@window_fg_color,@window_bg_color,0.5),@window_bg_color,0.5);
}
expander title > arrow:checked {
  -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
}

calendar {
  color: @view_fg_color;
  border: 1px solid alpha(currentColor,0.15);
}
calendar:selected, calendar:backdrop:selected {
  background-color: @accent_bg_color;
  color: @accent_fg_color;
  border-radius: 9px;
}
calendar.header {
  border-bottom-color: alpha(currentColor,0.15);
}
calendar.header:backdrop {
  border-bottom-color: alpha(currentColor,0.15);
}
calendar.button {
  color: @view_fg_color;
}
calendar.button:hover {
  background-color: alpha(currentColor,0.07);
}
calendar.button:active {
  background-color: alpha(currentColor,0.16);
}
calendar.button:checked {
  background-color: alpha(currentColor,0.1);
}
calendar.button:checked:hover {
  background-color: alpha(currentColor,0.12);
}
calendar.button:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
}
calendar.button:disabled {
  color: alpha(@window_fg_color,0.5);
}
calendar.highlight {
  color: alpha(@window_fg_color,0.5);
}
calendar.highlight:backdrop {
  color: mix(mix(@window_fg_color,@window_bg_color,0.5),@window_bg_color,0.5);
}
calendar:backdrop {
  color: mix(@view_fg_color,@window_bg_color,0.5);
  border-color: mix(currentColor,@window_bg_color,0.81);
}
calendar:indeterminate {
  color: alpha(currentColor,0.1);
}

messagedialog.csd.background button {
  outline-color: alpha(@accent_color,0.5);
  color: rgba(0, 0, 6, 0.8);
  background-color: alpha(currentColor,0.1);
}
messagedialog.csd.background button:hover {
  outline-color: alpha(@accent_color,0.5);
  color: rgba(0, 0, 6, 0.8);
  background-color: alpha(currentColor,0.15);
}
messagedialog.csd.background button:checked, messagedialog.csd.background button:active {
  outline-color: alpha(@accent_color,0.5);
  color: rgba(0, 0, 6, 0.8);
  background-color: alpha(currentColor,0.3);
}
messagedialog.csd.background button:backdrop {
  outline-color: transparent;
  color: alpha(rgba(0, 0, 6, 0.8),0.5);
  background-color: alpha(currentColor,0.1);
}
messagedialog.csd.background button:backdrop:hover {
  background-color: alpha(currentColor,0.15);
}
messagedialog.csd.background button:backdrop:checked {
  background-color: alpha(currentColor,0.3);
}
messagedialog.csd.background button:backdrop:active {
  background-color: alpha(currentColor,0.3);
}
messagedialog.csd.background button:backdrop:checked:hover {
  background-color: alpha(currentColor,0.35);
}
messagedialog.csd.background button:backdrop:checked:active {
  background-color: alpha(currentColor,0.4);
}
messagedialog.csd.background button:disabled {
  outline-color: transparent;
  color: alpha(rgba(0, 0, 6, 0.8),0.5);
  background-color: alpha(currentColor,0.1);
}
messagedialog .titlebar, messagedialog .titlebar:backdrop {
  min-height: 20px;
  background-image: none;
  background: @dialog_bg_color;
  color: @dialog_fg_color;
  border-style: none;
  border-top-left-radius: 15px;
  border-top-right-radius: 15px;
}
messagedialog.csd.background {
  border-bottom-left-radius: 15px;
  border-bottom-right-radius: 15px;
  background-color: @dialog_bg_color;
  color: @dialog_fg_color;
}
messagedialog.csd .dialog-action-area {
  margin: 0;
  padding: 21px 24px 24px 24px;
}
messagedialog.csd .dialog-action-area button {
  padding: 10px 20px;
  margin: 0 10px;
  border-radius: 12px;
  -gtk-outline-radius: 10px;
  border-left-style: none;
  border-right-style: none;
  border-color: transparent;
}

filechooser .dialog-action-box {
  border-top: 1px solid mix(currentColor,@window_bg_color,0.85);
}
filechooser #pathbarbox {
  border-bottom: 1px solid @window_bg_color;
}

filechooserbutton:drop(active) {
  box-shadow: none;
  border-color: transparent;
}

.sidebar {
  border-style: none;
  background-color: @sidebar_bg_color;
  color: @sidebar_fg_color;
}
.sidebar:backdrop {
  background-color: @sidebar_backdrop_color;
}
stacksidebar.sidebar:dir(ltr) list, stacksidebar.sidebar.left list, stacksidebar.sidebar.left:dir(rtl) list, .sidebar:not(separator):dir(ltr), .sidebar:not(separator).left, .sidebar:not(separator).left:dir(rtl) {
  border-right: 1px solid @sidebar_border_color;
  border-left-style: none;
}

stacksidebar.sidebar:dir(rtl) list, stacksidebar.sidebar.right list, .sidebar:not(separator):dir(rtl), .sidebar:not(separator).right {
  border-left: 1px solid @sidebar_border_color;
  border-right-style: none;
}

.sidebar list {
  background-color: transparent;
}
paned .sidebar.left, paned .sidebar.right, paned .sidebar.left:dir(rtl), paned .sidebar:dir(rtl), paned .sidebar:dir(ltr), paned .sidebar {
  border-style: none;
}
.sidebar row {
  border-radius: 9px;
}
.sidebar row:backdrop {
  color: @sidebar_fg_color;
}
.sidebar row, .sidebar separator {
  margin: 0 4px;
}

stacksidebar row {
  padding: 10px 4px;
}
stacksidebar row > label {
  padding-left: 6px;
  padding-right: 6px;
}
stacksidebar row.needs-attention > label {
  background-size: 6px 6px, 0 0;
}

separator.sidebar {
  background-color: @sidebar_border_color;
}
separator.sidebar.selection-mode, .selection-mode separator.sidebar {
  background-color: shade(@accent_bg_color,0.8);
}

row image.sidebar-icon {
  opacity: 0.7;
}

placessidebar > viewport.frame {
  border-style: none;
}
placessidebar row {
  min-height: 36px;
  padding: 0px;
}
placessidebar row > revealer {
  padding: 0 14px;
}
placessidebar row:selected {
  color: @accent_fg_color;
}
placessidebar row:disabled {
  color: alpha(@window_fg_color,0.5);
}
placessidebar row:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
}
placessidebar row:backdrop:selected {
  color: @window_bg_color;
}
placessidebar row:backdrop:disabled {
  color: mix(mix(@window_fg_color,@window_bg_color,0.5),@window_bg_color,0.5);
}
placessidebar row image.sidebar-icon:dir(ltr) {
  padding-right: 8px;
}
placessidebar row image.sidebar-icon:dir(rtl) {
  padding-left: 8px;
}
placessidebar row label.sidebar-label:dir(ltr) {
  padding-right: 2px;
}
placessidebar row label.sidebar-label:dir(rtl) {
  padding-left: 2px;
}
button.sidebar-button {
  min-height: 26px;
  min-width: 26px;
  margin-top: 3px;
  margin-bottom: 3px;
  padding: 0;
  border-radius: 100%;
  -gtk-outline-radius: 100%;
}
button.sidebar-button:not(:hover):not(:active) > image, button.sidebar-button:backdrop > image {
  opacity: 0.7;
}

placessidebar row:selected:active {
  box-shadow: none;
}
placessidebar row.sidebar-placeholder-row {
  padding: 0 8px;
  min-height: 2px;
  background-image: image(@accent_bg_color);
  background-clip: content-box;
}
placessidebar row.sidebar-new-bookmark-row {
  color: @accent_bg_color;
}
placessidebar row:drop(active):not(:disabled) {
  color: @accent_bg_color;
  box-shadow: inset 0 1px @accent_bg_color, inset 0 -1px @accent_bg_color;
}
placessidebar row:drop(active):not(:disabled):selected {
  color: @accent_fg_color;
  background-color: @accent_bg_color;
}

placesview .server-list-button > image {
  transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  -gtk-icon-transform: rotate(0turn);
}
placesview .server-list-button:checked > image {
  transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  -gtk-icon-transform: rotate(-0.5turn);
}
placesview row.activatable:hover {
  background-color: transparent;
}
placesview > actionbar > revealer > box > label {
  padding-left: 8px;
  padding-right: 8px;
}

paned > separator {
  min-width: 1px;
  min-height: 1px;
  -gtk-icon-source: none;
  border-style: none;
  background-color: transparent;
  background-image: image(mix(currentColor,@window_bg_color,0.85));
  background-size: 1px 1px;
}
paned > separator:selected {
  background-image: image(@accent_bg_color);
}
paned > separator:backdrop {
  background-image: image(mix(currentColor,@window_bg_color,0.81));
}
paned > separator.wide {
  min-width: 5px;
  min-height: 5px;
  background-color: @window_bg_color;
  background-image: image(mix(currentColor,@window_bg_color,0.85)), image(mix(currentColor,@window_bg_color,0.85));
  background-size: 1px 1px, 1px 1px;
}
paned > separator.wide:backdrop {
  background-color: @window_bg_color;
  background-image: image(mix(currentColor,@window_bg_color,0.81)), image(mix(currentColor,@window_bg_color,0.81));
}
paned.horizontal > separator {
  background-repeat: repeat-y;
}
paned.horizontal > separator:dir(ltr) {
  margin: 0 -8px 0 0;
  padding: 0 8px 0 0;
  background-position: left;
}
paned.horizontal > separator:dir(rtl) {
  margin: 0 0 0 -8px;
  padding: 0 0 0 8px;
  background-position: right;
}
paned.horizontal > separator.wide {
  margin: 0;
  padding: 0;
  background-repeat: repeat-y, repeat-y;
  background-position: left, right;
}
paned.vertical > separator {
  margin: 0 0 -8px 0;
  padding: 0 0 8px 0;
  background-repeat: repeat-x;
  background-position: top;
}
paned.vertical > separator.wide {
  margin: 0;
  padding: 0;
  background-repeat: repeat-x, repeat-x;
  background-position: bottom, top;
}

infobar {
  border-style: none;
}
infobar.info > revealer > box {
  background-color: mix(@accent_bg_color,@window_bg_color,0.7);
  color: @window_fg_color;
  text-shadow: none;
}
infobar.info > revealer > box:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
}
infobar.info button {
  background-color: mix(@accent_bg_color,@window_bg_color,0.5);
}
infobar.info button:hover {
  background-color: mix(@accent_bg_color,@window_bg_color,0.35);
}
infobar.info button:checked, infobar.info button:active {
  background-color: mix(@accent_bg_color,@window_bg_color,0.2);
}
infobar.info button:backdrop:not(:disabled) label, infobar.info button:backdrop:not(:disabled), infobar.info button:not(:disabled) label, infobar.info button:not(:disabled) {
  color: @window_fg_color;
}
infobar.info button:disabled {
  background-color: mix(@accent_bg_color,@window_bg_color,0.6);
}
infobar.info button:disabled label, infobar.info button:disabled {
  color: alpha(mix(@window_fg_color,@accent_bg_color,0.3),0.6);
}
infobar.action.info:hover > revealer > box {
  background-color: mix(@accent_bg_color,@window_bg_color,0.6);
  box-shadow: inset 0 -1px mix(@accent_bg_color,@window_bg_color,0.3);
}
infobar.question > revealer > box {
  background-color: mix(@accent_bg_color,@window_bg_color,0.7);
  color: @window_fg_color;
  text-shadow: none;
}
infobar.question > revealer > box:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
}
infobar.question button {
  background-color: mix(@accent_bg_color,@window_bg_color,0.5);
}
infobar.question button:hover {
  background-color: mix(@accent_bg_color,@window_bg_color,0.35);
}
infobar.question button:checked, infobar.question button:active {
  background-color: mix(@accent_bg_color,@window_bg_color,0.2);
}
infobar.question button:backdrop:not(:disabled) label, infobar.question button:backdrop:not(:disabled), infobar.question button:not(:disabled) label, infobar.question button:not(:disabled) {
  color: @window_fg_color;
}
infobar.question button:disabled {
  background-color: mix(@accent_bg_color,@window_bg_color,0.6);
}
infobar.question button:disabled label, infobar.question button:disabled {
  color: alpha(mix(@window_fg_color,@accent_bg_color,0.3),0.6);
}
infobar.action.question:hover > revealer > box {
  background-color: mix(@accent_bg_color,@window_bg_color,0.6);
  box-shadow: inset 0 -1px mix(@accent_bg_color,@window_bg_color,0.3);
}
infobar.warning > revealer > box {
  background-color: mix(@warning_bg_color,@window_bg_color,0.7);
  color: @window_fg_color;
  text-shadow: none;
}
infobar.warning > revealer > box:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
}
infobar.warning button {
  background-color: mix(@warning_bg_color,@window_bg_color,0.5);
}
infobar.warning button:hover {
  background-color: mix(@warning_bg_color,@window_bg_color,0.35);
}
infobar.warning button:checked, infobar.warning button:active {
  background-color: mix(@warning_bg_color,@window_bg_color,0.2);
}
infobar.warning button:backdrop:not(:disabled) label, infobar.warning button:backdrop:not(:disabled), infobar.warning button:not(:disabled) label, infobar.warning button:not(:disabled) {
  color: @window_fg_color;
}
infobar.warning button:disabled {
  background-color: mix(@warning_bg_color,@window_bg_color,0.6);
}
infobar.warning button:disabled label, infobar.warning button:disabled {
  color: alpha(mix(@window_fg_color,@warning_bg_color,0.3),0.6);
}
infobar.action.warning:hover > revealer > box {
  background-color: mix(@warning_bg_color,@window_bg_color,0.6);
  box-shadow: inset 0 -1px mix(@warning_bg_color,@window_bg_color,0.3);
}
infobar.error > revealer > box {
  background-color: mix(@error_bg_color,@window_bg_color,0.7);
  color: @window_fg_color;
  text-shadow: none;
}
infobar.error > revealer > box:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
}
infobar.error button {
  background-color: mix(@error_bg_color,@window_bg_color,0.5);
}
infobar.error button:hover {
  background-color: mix(@error_bg_color,@window_bg_color,0.35);
}
infobar.error button:checked, infobar.error button:active {
  background-color: mix(@error_bg_color,@window_bg_color,0.2);
}
infobar.error button:backdrop:not(:disabled) label, infobar.error button:backdrop:not(:disabled), infobar.error button:not(:disabled) label, infobar.error button:not(:disabled) {
  color: @window_fg_color;
}
infobar.error button:disabled {
  background-color: mix(@error_bg_color,@window_bg_color,0.6);
}
infobar.error button:disabled label, infobar.error button:disabled {
  color: alpha(mix(@window_fg_color,@error_bg_color,0.3),0.6);
}
infobar.action.error:hover > revealer > box {
  background-color: mix(@error_bg_color,@window_bg_color,0.6);
  box-shadow: inset 0 -1px mix(@error_bg_color,@window_bg_color,0.3);
}
infobar .close {
  min-width: 18px;
  min-height: 18px;
  padding: 4px;
  border-radius: 50%;
}
infobar selection {
  background-color: shade(@window_bg_color,0.9);
}
infobar *:link {
  color: @accent_color;
}

tooltip {
  padding: 6px 10px;
  border-radius: 9px;
  box-shadow: none;
  text-shadow: 0 1px black;
}
tooltip.background {
  background-color: rgba(0, 0, 0, 0.8);
  background-clip: padding-box;
  border: 1px solid rgba(255, 255, 255, 0.1);
}
tooltip decoration {
  background-color: transparent;
}
tooltip * {
  padding: 4px;
  background-color: transparent;
  color: white;
}

colorswatch:drop(active), colorswatch {
  border-style: none;
}
colorswatch.top {
  border-top-left-radius: 9.5px;
  border-top-right-radius: 9.5px;
}
colorswatch.top overlay {
  border-top-left-radius: 9px;
  border-top-right-radius: 9px;
}
colorswatch.bottom {
  border-bottom-left-radius: 9.5px;
  border-bottom-right-radius: 9.5px;
}
colorswatch.bottom overlay {
  border-bottom-left-radius: 9px;
  border-bottom-right-radius: 9px;
}
colorswatch.left, colorswatch:first-child:not(.top) {
  border-top-left-radius: 9.5px;
  border-bottom-left-radius: 9.5px;
}
colorswatch.left overlay, colorswatch:first-child:not(.top) overlay {
  border-top-left-radius: 9px;
  border-bottom-left-radius: 9px;
}
colorswatch.right, colorswatch:last-child:not(.bottom) {
  border-top-right-radius: 9.5px;
  border-bottom-right-radius: 9.5px;
}
colorswatch.right overlay, colorswatch:last-child:not(.bottom) overlay {
  border-top-right-radius: 9px;
  border-bottom-right-radius: 9px;
}
colorswatch.dark {
  outline-color: rgba(255, 255, 255, 0.6);
}
colorswatch.dark overlay {
  color: white;
}
colorswatch.dark overlay:hover {
  border-color: rgba(0, 0, 0, 0.8);
}
colorswatch.light {
  outline-color: rgba(0, 0, 0, 0.6);
}
colorswatch.light overlay {
  color: black;
}
colorswatch.light overlay:hover {
  border-color: rgba(0, 0, 0, 0.5);
}
colorswatch:drop(active) {
  box-shadow: none;
}
colorswatch:drop(active).light overlay {
  border-color: @accent_bg_color;
  box-shadow: inset 0 0 0 2px shade(@accent_bg_color,0.93), inset 0 0 0 1px @accent_bg_color;
}
colorswatch:drop(active).dark overlay {
  border-color: @accent_bg_color;
  box-shadow: inset 0 0 0 2px rgba(0, 0, 0, 0.3), inset 0 0 0 1px @accent_bg_color;
}
colorswatch overlay {
  border: 1px solid rgba(0, 0, 0, 0.3);
}
colorswatch overlay:hover {
  box-shadow: inset 0 1px rgba(255, 255, 255, 0.4), inset 0 -1px rgba(0, 0, 0, 0.2);
}
colorswatch#add-color-button {
  border-radius: 9px 9px 0 0;
}
colorswatch#add-color-button:only-child {
  border-radius: 9px;
}
colorswatch#add-color-button overlay {
  color: @window_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.9);
}
colorswatch#add-color-button overlay:hover {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.85);
  box-shadow: none;
}
colorswatch:disabled {
  opacity: 0.5;
}
colorswatch:disabled overlay {
  border-color: rgba(0, 0, 0, 0.6);
  box-shadow: none;
}
row:selected colorswatch {
  box-shadow: 0 0 0 2px @accent_fg_color;
}
colorswatch#editor-color-sample {
  border-radius: 4px;
}
colorswatch#editor-color-sample overlay {
  border-radius: 4.5px;
}

colorchooser .popover.osd {
  border-radius: 5px;
}

.checkerboard {
  background-position: 0px 0px, 10px 10px;
  background-size: 20px 20px;
  background-image: linear-gradient(45deg, mix(currentColor,@window_bg_color,0.85) 25%, transparent 25%, transparent 75%, mix(currentColor,@window_bg_color,0.85) 75%, mix(currentColor,@window_bg_color,0.85) 100%), linear-gradient(45deg, mix(currentColor,@window_bg_color,0.85) 25%, @view_bg_color 25%, @view_bg_color 75%, mix(currentColor,@window_bg_color,0.85) 75%, mix(currentColor,@window_bg_color,0.85) 100%);
}

.content-view {
  background-color: shade(@window_bg_color,0.93);
}
.content-view:hover {
  -gtk-icon-effect: highlight;
}
.osd .scale-popup button.flat {
  border-style: none;
  border-radius: 5px;
}
.scale-popup button:hover {
  background-color: alpha(@window_fg_color,0.1);
  border-radius: 5px;
}

cursor-handle {
  background-color: transparent;
  background-image: none;
  box-shadow: none;
  border-style: none;
}
cursor-handle.top:dir(ltr), cursor-handle.bottom:dir(rtl) {
  -gtk-icon-source: -gtk-scaled(url("assets/text-select-start.png"), url("assets/text-select-start@2.png"));
  padding-left: 10px;
}
cursor-handle.bottom:dir(ltr), cursor-handle.top:dir(rtl) {
  -gtk-icon-source: -gtk-scaled(url("assets/text-select-end.png"), url("assets/text-select-end@2.png"));
  padding-right: 10px;
}
cursor-handle.insertion-cursor:dir(ltr), cursor-handle.insertion-cursor:dir(rtl) {
  -gtk-icon-source: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above.png"), url("assets/slider-horz-scale-has-marks-above@2.png"));
}
cursor-handle.top:dir(ltr), cursor-handle.bottom:dir(rtl) {
  -gtk-icon-source: -gtk-scaled(url("assets/text-select-start.png"), url("assets/text-select-start@2.png"));
  padding-left: 10px;
}
cursor-handle.bottom:dir(ltr), cursor-handle.top:dir(rtl) {
  -gtk-icon-source: -gtk-scaled(url("assets/text-select-end.png"), url("assets/text-select-end@2.png"));
  padding-right: 10px;
}
cursor-handle.insertion-cursor:dir(ltr), cursor-handle.insertion-cursor:dir(rtl) {
  -gtk-icon-source: -gtk-scaled(url("assets/slider-horz-scale-has-marks-above.png"), url("assets/slider-horz-scale-has-marks-above@2.png"));
}

.context-menu {
  font: initial;
}

.keycap {
  min-width: 20px;
  min-height: 25px;
  margin-top: 2px;
  padding-bottom: 3px;
  padding-left: 6px;
  padding-right: 6px;
  color: @window_fg_color;
  background-color: @view_bg_color;
  border: 1px solid;
  border-color: mix(mix(currentColor,@window_bg_color,0.85),@window_bg_color,0.5);
  border-radius: 5px;
  box-shadow: inset 0 -3px mix(@view_bg_color,@window_bg_color,0.8);
  font-size: smaller;
}
.keycap:backdrop {
  background-color: @window_bg_color;
  color: mix(@window_fg_color,@window_bg_color,0.5);
  transition: 200ms ease-out;
}

:not(decoration):not(window):drop(active):focus,
:not(decoration):not(window):drop(active) {
  border-color: @accent_bg_color;
  box-shadow: inset 0 0 0 1px @accent_bg_color;
  caret-color: @accent_bg_color;
}

stackswitcher button.text-button {
  min-width: 100px;
}

stackswitcher button.circular,
stackswitcher button.text-button.circular {
  min-width: 32px;
  min-height: 32px;
  padding: 0;
}

.lowres-icon {
  -gtk-icon-shadow: 0 -1px rgba(0, 0, 0, 0.05), 1px 0 rgba(0, 0, 0, 0.1), 0 1px rgba(0, 0, 0, 0.3), -1px 0 rgba(0, 0, 0, 0.1);
}

.icon-dropshadow {
  -gtk-icon-shadow: 0 1px 12px rgba(0, 0, 0, 0.05), 0 -1px rgba(0, 0, 0, 0.05), 1px 0 rgba(0, 0, 0, 0.1), 0 1px rgba(0, 0, 0, 0.3), -1px 0 rgba(0, 0, 0, 0.1);
}

popover.emoji-picker {
  padding-left: 0;
  padding-right: 0;
}
popover.emoji-picker entry.search {
  margin: 3px 5px 5px 5px;
}

button.emoji-section {
  border-color: transparent;
  border-width: 3px;
  border-style: none none solid;
  border-radius: 0;
  margin: 2px 4px 2px 4px;
  padding: 3px 0 0;
  min-width: 32px;
  min-height: 28px;
  background: none;
  box-shadow: none;
  text-shadow: none;
  outline-offset: -5px;
}
button.emoji-section:first-child {
  margin-left: 7px;
}
button.emoji-section:last-child {
  margin-right: 7px;
}
button.emoji-section:backdrop:not(:checked) {
  border-color: transparent;
}
button.emoji-section:hover {
  border-color: mix(currentColor,@window_bg_color,0.85);
}
button.emoji-section:checked {
  border-color: alpha(currentColor,0.1);
}
button.emoji-section label {
  padding: 0;
  opacity: 0.55;
}
button.emoji-section:hover label {
  opacity: 0.775;
}
button.emoji-section:checked label {
  opacity: 1;
}

popover.emoji-picker .emoji {
  font-size: x-large;
  padding: 6px;
  border-radius: 9px;
}
popover.emoji-picker .emoji :focus,
popover.emoji-picker .emoji :hover {
  background: alpha(currentColor,0.07);
}
popover.emoji-picker .emoji :active {
  background: alpha(currentColor,0.16);
}

popover.emoji-completion arrow {
  border: none;
  background: none;
}

popover.emoji-completion contents row box {
  padding: 2px 10px;
}

popover.emoji-completion .emoji:hover {
  background: alpha(currentColor,0.04);
}

.flat.popup:not(.title) {
  font-weight: normal;
}

statusbar {
  font-size: small;
}

scrolledwindow list:not(.content) {
  padding: 6px 0;
}
scrolledwindow list row {
  margin: 0 6px;
  border-radius: 9px;
}
scrolledwindow list separator.horizontal {
  margin: 6px;
}

decoration {
  border-radius: 15px 15px 0 0;
  border-width: 0px;
  box-shadow: 0 3px 8px 1px rgba(0, 0, 0, 0.3), 0 0 0 1px rgba(0, 0, 0, 0.05);
  margin: 10px;
}
decoration:backdrop {
  box-shadow: 0 3px 8px 1px transparent, 0 2px 6px 2px rgba(0, 0, 0, 0.14), 0 0 0 1px rgba(0, 0, 0, 0.02);
  transition: 200ms ease-out;
}
.maximized decoration, .fullscreen decoration {
  border-radius: 0;
  box-shadow: none;
}
.tiled decoration, .tiled-top decoration, .tiled-right decoration, .tiled-bottom decoration, .tiled-left decoration {
  border-radius: 0;
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.05), 0 0 0 20px transparent;
}
.tiled decoration:backdrop, .tiled-top decoration:backdrop, .tiled-right decoration:backdrop, .tiled-bottom decoration:backdrop, .tiled-left decoration:backdrop {
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.02), 0 0 0 20px transparent;
}
.popup decoration {
  box-shadow: none;
}
.ssd decoration {
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.3);
}
.ssd decoration:backdrop {
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.2);
}
.ssd.maximized decoration, .ssd.maximized decoration:backdrop {
  box-shadow: none;
}
.csd.popup decoration {
  border-radius: 9px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.2), 0 0 0 1px alpha(rgba(0, 0, 0, 0.05),0.9);
}
tooltip.csd decoration {
  border-radius: 5px;
  box-shadow: none;
}
messagedialog.csd decoration {
  border-radius: 15px;
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.2), 0 0 0 1px alpha(rgba(0, 0, 0, 0.05),0.9);
}
.solid-csd decoration {
  margin: 0;
  padding: 4px;
  background-color: mix(currentColor,@window_bg_color,0.85);
  border: solid 1px mix(currentColor,@window_bg_color,0.85);
  border-radius: 0;
  box-shadow: inset 0 0 0 5px mix(currentColor,@window_bg_color,0.85), inset 0 0 0 4px @headerbar_bg_color, inset 0 0 0 1px mix(currentColor,@window_bg_color,0.85);
}
.solid-csd decoration:backdrop {
  box-shadow: inset 0 0 0 3px @window_bg_color;
}

button.titlebutton {
  padding: 2px;
  margin: 0;
  box-shadow: none;
  border: none;
  background: none;
  text-shadow: none;
}
button.titlebutton:not(.appmenu) {
  border-radius: 9999px;
}
button.titlebutton:backdrop {
  -gtk-icon-shadow: none;
}

.selection-mode headerbar button.titlebutton:backdrop,
.selection-mode .titlebar button.titlebutton:backdrop,
headerbar.selection-mode button.titlebutton:backdrop,
.titlebar.selection-mode button.titlebutton:backdrop {
  -gtk-icon-shadow: none;
}

.monospace {
  font-family: monospace;
}

window.csd.unified:not(.solid-csd):not(.fullscreen):not(.tiled):not(.tiled-top):not(.tiled-bottom):not(.tiled-left):not(.tiled-right):not(.maximized),
window.csd.unified:not(.solid-csd):not(.fullscreen):not(.tiled):not(.tiled-top):not(.tiled-bottom):not(.tiled-left):not(.tiled-right):not(.maximized) > decoration,
window.csd.unified:not(.solid-csd):not(.fullscreen):not(.tiled):not(.tiled-top):not(.tiled-bottom):not(.tiled-left):not(.tiled-right):not(.maximized) > decoration-overlay {
  border-radius: 15px;
}

window.csd.unified scrollbar {
  background-color: @view_bg_color;
  color: @window_fg_color;
}
window.csd.unified scrollbar trough {
  margin: 7px;
  transition: all 200ms linear;
  border-radius: 10px;
  background-color: alpha(currentColor,0.15);
}
window.csd.unified scrollbar trough > slider {
  min-width: 8px;
  min-height: 8px;
  margin: -6px;
  border: 6px solid transparent;
  border-radius: 10px;
  background-clip: padding-box;
  transition: all 200ms linear;
}
window.csd.unified scrollbar.horizontal trough {
  margin-top: 6px;
  margin-bottom: 6px;
}
window.csd.unified scrollbar.vertical trough {
  margin-left: 6px;
  margin-right: 6px;
}
window.csd.unified scrollbar.overlay-indicator {
  background: none;
  color: inherit;
  box-shadow: none;
  padding: 0;
}
window.csd.unified scrollbar.overlay-indicator:not(.hovering) trough {
  background-color: transparent;
}
window.csd.unified scrollbar.overlay-indicator.horizontal trough > slider {
  min-width: 40px;
}
window.csd.unified scrollbar.overlay-indicator.horizontal.hovering trough > slider {
  min-height: 8px;
}
window.csd.unified scrollbar.overlay-indicator.horizontal:not(.hovering) trough {
  margin-top: 3px;
  margin-bottom: 3px;
}
window.csd.unified scrollbar.overlay-indicator.vertical trough > slider {
  min-height: 40px;
}
window.csd.unified scrollbar.overlay-indicator.vertical.hovering trough > slider {
  min-width: 8px;
}
window.csd.unified scrollbar.overlay-indicator.vertical:not(.hovering) trough {
  margin-left: 3px;
  margin-right: 3px;
}
window.csd.unified scrollbar.horizontal trough > slider {
  min-width: 40px;
}
window.csd.unified scrollbar.vertical trough > slider {
  min-height: 40px;
}

popover.combo {
  padding: 0px;
}
popover.combo list {
  border-style: none;
  background-color: transparent;
  min-width: 200px;
  margin-top: 6px;
  margin-bottom: 6px;
}
popover.combo list > row {
  padding: 0px 8px 0px 8px;
  min-height: 50px;
}
popover.combo list > row:not(:last-child) {
  border-bottom: 1px solid alpha(mix(currentColor,@window_bg_color,0.85),0.5);
}
popover.combo list > row:first-child {
  border-top-left-radius: 8px;
  -gtk-outline-top-left-radius: 7px;
  border-top-right-radius: 8px;
  -gtk-outline-top-right-radius: 7px;
}
popover.combo list > row:last-child {
  border-bottom-left-radius: 8px;
  -gtk-outline-bottom-left-radius: 7px;
  border-bottom-right-radius: 8px;
  -gtk-outline-bottom-right-radius: 7px;
}
popover.combo overshoot.top {
  border-top-left-radius: 8px;
  -gtk-outline-top-left-radius: 7px;
  border-top-right-radius: 8px;
  -gtk-outline-top-right-radius: 7px;
}
popover.combo overshoot.bottom {
  border-bottom-left-radius: 8px;
  -gtk-outline-bottom-left-radius: 7px;
  border-bottom-right-radius: 8px;
  -gtk-outline-bottom-right-radius: 7px;
}
popover.combo scrollbar.vertical {
  padding-top: 2px;
  padding-bottom: 2px;
}
popover.combo scrollbar.vertical:dir(ltr) {
  border-top-right-radius: 8px;
  -gtk-outline-top-right-radius: 7px;
  border-bottom-right-radius: 8px;
  -gtk-outline-bottom-right-radius: 7px;
}
popover.combo scrollbar.vertical:dir(rtl) {
  border-top-left-radius: 8px;
  -gtk-outline-top-left-radius: 7px;
  border-bottom-left-radius: 8px;
  -gtk-outline-bottom-left-radius: 7px;
}

row.expander {
  padding: 0px;
}
row.expander image.expander-row-arrow:dir(ltr) {
  margin-left: 6px;
}
row.expander image.expander-row-arrow:dir(rtl) {
  margin-right: 6px;
}

row.expander {
  background-color: transparent;
}
row.expander list.nested > row {
  background-color: transparent;
  border-radius: 0;
  border: none;
}
row.expander list.nested > row:not(:selected).activatable:hover {
  background-color: alpha(currentColor,0.03);
}
row.expander list.nested > row:not(:selected).activatable:active {
  background-color: alpha(currentColor,0.08);
}
row.expander list.nested {
  background-color: alpha(@card_shade_color,0.5);
  color: inherit;
  border-radius: 0;
  border-top: 1px solid alpha(currentColor,0.15);
}
row.expander image.expander-row-arrow {
  transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}
row.expander:checked image.expander-row-arrow {
  -gtk-icon-transform: rotate(0turn);
}
row.expander:not(:checked) image.expander-row-arrow {
  opacity: 0.55;
  text-shadow: none;
}
row.expander:not(:checked) image.expander-row-arrow:dir(ltr) {
  -gtk-icon-transform: rotate(-0.25turn);
}
row.expander:not(:checked) image.expander-row-arrow:dir(rtl) {
  -gtk-icon-transform: rotate(0.25turn);
}
row.expander:checked image.expander-row-arrow:not(:disabled) {
  color: @accent_bg_color;
}
row.expander image.expander-row-arrow:disabled {
  color: alpha(@window_fg_color,0.5);
}

list.content list {
  background-color: transparent;
  border-radius: 12px;
}
list.content {
  background-color: transparent;
  color: @card_fg_color;
  border-radius: 12px;
}
list.content > row {
  background-color: @card_bg_color;
  color: @card_fg_color;
  margin: 0;
  transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}
list.content > row:not(:last-child) {
  box-shadow: inset 0 -1px @card_shade_color;
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}
list.content > row:not(:first-child):not(:last-child) {
  border-radius: 0;
  border-left: 1px solid @card_shade_color;
  border-right: 1px solid @card_shade_color;
  -gtk-outline-radius: 0;
}
list.content > row:first-child {
  border-top-left-radius: 12px;
  border-top-right-radius: 12px;
  border-top: 1px solid @card_shade_color;
  border-left: 1px solid @card_shade_color;
  border-right: 1px solid @card_shade_color;
  -gtk-outline-radius: 12px 12px 0 0;
}
list.content > row:last-child {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
  border-bottom-left-radius: 12px;
  border-bottom-right-radius: 12px;
  border-bottom: 1px solid @card_shade_color;
  border-left: 1px solid @card_shade_color;
  border-right: 1px solid @card_shade_color;
  -gtk-outline-radius: 0 0 12px 12px;
}
list.content > row:only-child {
  border: 1px solid @card_shade_color;
  border-radius: 12px;
  -gtk-outline-radius: 12px;
}
list.content > row:not(.expander):not(:active):hover.activatable:not(:selected), list.content > row.expander row.header:not(:active):hover.activatable:not(:selected) {
  color: @card_fg_color;
  background-color: mix(currentColor,@card_bg_color,0.96);
}
list.content + .inline-toolbar {
  border-color: transparent;
}

button.list-button:not(:active):not(:checked):not(:hover) {
  color: @card_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@card_fg_color,@card_bg_color,0.9);
}

keypad .digit {
  font-size: 200%;
  font-weight: bold;
}
keypad .letters {
  font-size: 70%;
}
keypad .symbol {
  font-size: 160%;
}

headerbar viewswitcher button {
  margin: 0 4px;
}
headerbar viewswitcher button.radio:checked, headerbar viewswitcher button.radio:hover:not(:backdrop) {
  background: image(@headerbar_bg_color);
}
headerbar viewswitcher button:backdrop, headerbar viewswitcher button:hover:backdrop, headerbar viewswitcher button.radio:backdrop, headerbar viewswitcher button.radio:checked:backdrop, headerbar viewswitcher button.radio:hover:backdrop, headerbar viewswitcher button.radio:checked:hover:backdrop {
  background: image(@headerbar_backdrop_color);
  transition: none;
}
headerbar viewswitcher button.radio:checked:hover:not(:backdrop) {
  background: image(@headerbar_bg_color);
}
headerbar viewswitcher button, headerbar viewswitcher button:hover:not(:backdrop), headerbar viewswitcher button:checked, headerbar viewswitcher button:active, headerbar viewswitcher button:checked:hover, headerbar viewswitcher button:active:hover {
  background: image(@headerbar_bg_color);
  transition: none;
}
headerbar viewswitcher button > stack > box {
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  border-radius: 9px;
}
headerbar viewswitcher button:hover > stack > box {
  background-color: alpha(currentColor,0.07);
}
headerbar viewswitcher button:checked > stack > box, headerbar viewswitcher button:active > stack > box {
  background-color: alpha(currentColor,0.1);
}
headerbar viewswitcher button:checked:hover > stack > box, headerbar viewswitcher button:backdrop:checked:hover > stack > box {
  background-color: alpha(currentColor,0.19);
}

viewswitcher, viewswitcher button {
  margin: 0;
  padding: 0;
}
viewswitcher button {
  border-radius: 0;
  border-top: 0;
  border-bottom: 0;
  box-shadow: none;
  font-size: 1rem;
  border-width: 0;
}
viewswitcher button:not(:checked):not(:hover) {
  background: transparent;
}
viewswitcher button:not(:only-child):not(:last-child) {
  border-right-width: 0px;
}
viewswitcher button:not(only-child):first-child:not(:checked):not(:hover), viewswitcher button:not(:checked):not(:hover) + button:not(:checked):not(:hover) {
  border-left-color: transparent;
}
viewswitcher button:not(only-child):last-child:not(:checked):not(:hover) {
  border-right-color: transparent;
}
viewswitcher button:not(:checked):hover:not(:backdrop) {
  background-image: none;
}
viewswitcher button:not(only-child):first-child:not(:checked):hover, viewswitcher button:not(:checked):hover + button:not(:checked):not(:hover), viewswitcher button:not(:checked):not(:hover) + button:not(:checked):hover {
  border-left-color: alpha(currentColor,0.15);
}
viewswitcher button:not(only-child):last-child:not(:checked):hover {
  border-right-color: alpha(currentColor,0.15);
}
headerbar viewswitcher button:not(:checked) {
  background: none;
}
headerbar viewswitcher button:not(:checked):hover:not(:backdrop) {
  background: none;
}
headerbar viewswitcher button:not(:checked):not(only-child):first-child:hover, headerbar viewswitcher button:not(:checked):hover + button:not(:checked):not(:hover), headerbar viewswitcher button:not(:checked):not(:hover) + button:not(:checked):hover {
  border-left-color: alpha(currentColor,0.15);
}
headerbar viewswitcher button:not(:checked):not(only-child):last-child:hover {
  border-right-color: alpha(currentColor,0.15);
}
headerbar viewswitcher button:not(:checked):hover:backdrop, headerbar viewswitcher button:not(:checked):active:backdrop, headerbar viewswitcher button:not(:checked):active:hover:backdrop {
  background: image(@headerbar_backdrop_color);
}
headerbar viewswitcher button:not(:checked):active, headerbar viewswitcher button:not(:checked):active:hover {
  background: image(@headerbar_bg_color);
}
viewswitcher button > stack > box.narrow {
  font-size: 0.75rem;
  padding-top: 7px;
  padding-bottom: 5px;
}
viewswitcher button > stack > box.narrow image,
viewswitcher button > stack > box.narrow label {
  padding-left: 8px;
  padding-right: 8px;
}
viewswitcher button > stack > box.wide {
  padding: 8px 12px;
}
viewswitcher button > stack > box.wide image:dir(ltr) {
  padding-left: 7px;
}
viewswitcher button > stack > box.wide image:dir(rtl) {
  padding-right: 7px;
}
viewswitcher button > stack > box.wide label:dir(ltr) {
  padding-right: 7px;
}
viewswitcher button > stack > box.wide label:dir(rtl) {
  padding-left: 7px;
}
viewswitcher button > stack > box label.active {
  font-weight: bold;
}
viewswitcher button.needs-attention:active > stack > box label, viewswitcher button.needs-attention:checked > stack > box label {
  animation: none;
  background-image: none;
}
viewswitcher button.needs-attention > stack > box label {
  animation: needs_attention 150ms ease-in;
  background-image: -gtk-gradient(radial, center center, 0, center center, 0.5, to(@accent_bg_color), to(transparent));
  background-size: 6px 6px, 6px 6px;
  background-repeat: no-repeat;
  background-position: right 0px, right 1px;
}
viewswitcher button.needs-attention > stack > box label:backdrop {
  background-size: 6px 6px, 0 0;
}
viewswitcher button.needs-attention > stack > box label:dir(rtl) {
  background-position: left 0px, left 1px;
}

viewswitcherbar actionbar > revealer > box {
  padding: 0;
}

viewswitchertitle viewswitcher {
  margin-left: 12px;
  margin-right: 12px;
}

statuspage > scrolledwindow > viewport > box > clamp > box > .icon {
  color: alpha(@window_fg_color,0.5);
}

statuspage > scrolledwindow > viewport > box {
  margin: 36px 12px;
}
statuspage > scrolledwindow > viewport > box > clamp:not(:last-child) > box {
  margin-bottom: 36px;
}
statuspage > scrolledwindow > viewport > box > clamp > box > .icon:not(:last-child) {
  margin-bottom: 36px;
}
statuspage > scrolledwindow > viewport > box > clamp > box > .title:not(:last-child) {
  margin-bottom: 12px;
}

row label.subtitle {
  font-size: smaller;
  opacity: 0.55;
  text-shadow: none;
}
row > box.header {
  margin-left: 12px;
  margin-right: 12px;
  min-height: 50px;
}
row > box.header > box.title {
  margin-top: 8px;
  margin-bottom: 8px;
}

tabbar .box {
  min-height: 38px;
  background: @headerbar_bg_color;
  border-bottom: 1px solid @headerbar_shade_color;
}
tabbar .box:backdrop {
  color: mix(@headerbar_fg_color,@headerbar_backdrop_color,0.5);
  background-color: @headerbar_backdrop_color;
}
tabbar scrolledwindow.pinned undershoot {
  border: 0 solid mix(black,@window_bg_color,0.76);
}
tabbar scrolledwindow.pinned:dir(rtl) undershoot.left {
  border-left-width: 1px;
}
tabbar scrolledwindow.pinned:dir(ltr) undershoot.right {
  border-right-width: 1px;
}
tabbar scrolledwindow.pinned:backdrop undershoot {
  border-color: mix(currentColor,@window_bg_color,0.81);
}
tabbar scrolledwindow.pinned tabbox:dir(ltr) {
  padding-right: 1px;
  box-shadow: inset -1px 0 alpha(currentColor,0.15);
}
tabbar scrolledwindow.pinned tabbox:dir(rtl) {
  padding-left: 1px;
  box-shadow: inset 1px 0 alpha(currentColor,0.15);
}
tabbar undershoot {
  transition: none;
}
tabbar undershoot.left {
  background: linear-gradient(to right, rgba(0, 0, 0, 0.07), rgba(0, 0, 0, 0) 20px);
}
tabbar undershoot.right {
  background: linear-gradient(to left, rgba(0, 0, 0, 0.07), rgba(0, 0, 0, 0) 20px);
}
tabbar .needs-attention-left undershoot.left {
  background: linear-gradient(to right, alpha(@accent_bg_color,0.7), alpha(@accent_bg_color,0.5) 1px, alpha(@accent_bg_color,0) 20px);
}
tabbar .needs-attention-right undershoot.right {
  background: linear-gradient(to left, alpha(@accent_bg_color,0.7), alpha(@accent_bg_color,0.5) 1px, alpha(@accent_bg_color,0) 20px);
}
tabbar tab {
  margin: 4px 3px;
  border-style: none;
  border-radius: 9px;
  transition: background-color 150ms ease-in-out;
  background-position: left;
  background-repeat: no-repeat;
}
tabbar tab:hover {
  background-color: alpha(currentColor,0.07);
}
tabbar tab:checked {
  background-color: alpha(currentColor,0.1);
}
tabbar tab:checked:hover {
  background-color: alpha(currentColor,0.12);
}
tabbar .start-action,
tabbar .end-action {
  background-color: @headerbar_bg_color;
  border-color: transparent;
  border-style: solid;
  transition: background-color 150ms ease-in-out;
}
tabbar .start-action:backdrop,
tabbar .end-action:backdrop {
  border-color: transparent;
  background-color: @headerbar_backdrop_color;
}
tabbar .start-action button,
tabbar .end-action button {
  border: none;
  border-radius: 0;
}
tabbar .start-action:dir(ltr),
tabbar .end-action:dir(rtl) {
  border-right-width: 1px;
}
tabbar .start-action:dir(ltr) > *,
tabbar .end-action:dir(rtl) > * {
  margin-right: 1px;
}
tabbar .start-action:dir(rtl),
tabbar .end-action:dir(ltr) {
  border-left-width: 1px;
}
tabbar .start-action:dir(rtl) > *,
tabbar .end-action:dir(ltr) > * {
  margin-left: 1px;
}

.tab-drag-icon tab {
  min-height: 26px;
  background-color: alpha(currentColor,0.16);
  box-shadow: 0 3px 9px 1px rgba(0, 0, 0, 0.25), 0 0 0 1px rgba(0, 0, 0, 0.23), inset 0 1px alpha(white,0.5);
  margin: 25px;
}

tabbar tab,
.tab-drag-icon tab {
  padding: 6px;
}
tabbar tab.needs-attention,
.tab-drag-icon tab.needs-attention {
  background-image: radial-gradient(ellipse at bottom, rgba(255, 255, 255, 0.8), alpha(@accent_bg_color,0.2) 15%, alpha(@accent_bg_color,0) 15%);
}
tabbar tab .tab-close-button,
tabbar tab .tab-indicator,
.tab-drag-icon tab .tab-close-button,
.tab-drag-icon tab .tab-indicator {
  padding: 0;
  margin: 0;
  min-width: 24px;
  min-height: 24px;
  border-radius: 99px;
  border: none;
  box-shadow: none;
  -gtk-icon-shadow: none;
  text-shadow: none;
  background: none;
}
tabbar tab .tab-close-button:hover,
tabbar tab .tab-indicator.clickable:hover,
.tab-drag-icon tab .tab-close-button:hover,
.tab-drag-icon tab .tab-indicator.clickable:hover {
  background: alpha(@window_fg_color,0.15);
}
tabbar tab .tab-close-button:active,
tabbar tab .tab-indicator.clickable:active,
.tab-drag-icon tab .tab-close-button:active,
.tab-drag-icon tab .tab-indicator.clickable:active {
  background: alpha(black,0.2);
}

flap > dimming,
deck > dimming,
leaflet > dimming {
  background: rgba(0, 0, 0, 0.12);
}
flap > border,
deck > border,
leaflet > border {
  min-width: 1px;
  min-height: 1px;
  background: rgba(0, 0, 0, 0.05);
}
flap > shadow,
deck > shadow,
leaflet > shadow {
  min-width: 56px;
  min-height: 56px;
}
flap > shadow.left,
deck > shadow.left,
leaflet > shadow.left {
  background-image: linear-gradient(to right, rgba(0, 0, 0, 0.05), rgba(0, 0, 0, 0.01) 40px, rgba(0, 0, 0, 0) 56px), linear-gradient(to right, rgba(0, 0, 0, 0.03), rgba(0, 0, 0, 0.01) 7px, rgba(0, 0, 0, 0) 24px);
}
flap > shadow.right,
deck > shadow.right,
leaflet > shadow.right {
  background-image: linear-gradient(to left, rgba(0, 0, 0, 0.05), rgba(0, 0, 0, 0.01) 40px, rgba(0, 0, 0, 0) 56px), linear-gradient(to left, rgba(0, 0, 0, 0.03), rgba(0, 0, 0, 0.01) 7px, rgba(0, 0, 0, 0) 24px);
}
flap > shadow.up,
deck > shadow.up,
leaflet > shadow.up {
  background-image: linear-gradient(to bottom, rgba(0, 0, 0, 0.05), rgba(0, 0, 0, 0.01) 40px, rgba(0, 0, 0, 0) 56px), linear-gradient(to bottom, rgba(0, 0, 0, 0.03), rgba(0, 0, 0, 0.01) 7px, rgba(0, 0, 0, 0) 24px);
}
flap > shadow.down,
deck > shadow.down,
leaflet > shadow.down {
  background-image: linear-gradient(to top, rgba(0, 0, 0, 0.05), rgba(0, 0, 0, 0.01) 40px, rgba(0, 0, 0, 0) 56px), linear-gradient(to top, rgba(0, 0, 0, 0.03), rgba(0, 0, 0, 0.01) 7px, rgba(0, 0, 0, 0) 24px);
}
flap > outline,
deck > outline,
leaflet > outline {
  min-width: 1px;
  min-height: 1px;
  background: rgba(255, 255, 255, 0.2);
}

avatar {
  border-radius: 9999px;
  -gtk-outline-radius: 9999px;
  font-weight: bold;
}
avatar.color1 {
  background-image: linear-gradient(#83b6ec, #337fdc);
  color: #cfe1f5;
}
avatar.color2 {
  background-image: linear-gradient(#7ad9f1, #0f9ac8);
  color: #caeaf2;
}
avatar.color3 {
  background-image: linear-gradient(#8de6b1, #29ae74);
  color: #cef8d8;
}
avatar.color4 {
  background-image: linear-gradient(#b5e98a, #6ab85b);
  color: #e6f9d7;
}
avatar.color5 {
  background-image: linear-gradient(#f8e359, #d29d09);
  color: #f9f4e1;
}
avatar.color6 {
  background-image: linear-gradient(#ffcb62, #d68400);
  color: #ffead1;
}
avatar.color7 {
  background-image: linear-gradient(#ffa95a, #ed5b00);
  color: #ffe5c5;
}
avatar.color8 {
  background-image: linear-gradient(#f78773, #e62d42);
  color: #f8d2ce;
}
avatar.color9 {
  background-image: linear-gradient(#e973ab, #e33b6a);
  color: #fac7de;
}
avatar.color10 {
  background-image: linear-gradient(#cb78d4, #9945b5);
  color: #e7c2e8;
}
avatar.color11 {
  background-image: linear-gradient(#9e91e8, #7a59ca);
  color: #d5d2f5;
}
avatar.color12 {
  background-image: linear-gradient(#e3cf9c, #b08952);
  color: #f2eade;
}
avatar.color13 {
  background-image: linear-gradient(#be916d, #785336);
  color: #e5d6ca;
}
avatar.color14 {
  background-image: linear-gradient(#c0bfbc, #6e6d71);
  color: #d8d7d3;
}
avatar.contrasted {
  color: #fff;
}
avatar.image {
  background: none;
}

window.preferences > deck > deck > box > stack > stack > scrolledwindow > viewport > clamp,
preferencespage > scrolledwindow > viewport > clamp {
  margin: 0 12px;
}
window.preferences > deck > deck > box > stack > stack > scrolledwindow > viewport > clamp > list,
window.preferences > deck > deck > box > stack > stack > scrolledwindow > viewport > clamp > box > preferencesgroup,
preferencespage > scrolledwindow > viewport > clamp > list,
preferencespage > scrolledwindow > viewport > clamp > box > preferencesgroup {
  margin-top: 24px;
}

preferencesgroup > box > label:not(:first-child) {
  margin-top: 6px;
}
preferencesgroup > box > box:not(:first-child) {
  margin-top: 12px;
}

tabbar .tab-indicator:not(.clickable) {
  background: none;
  box-shadow: none;
  border-color: transparent;
}

.windowhandle, .windowhandle * {
  -GtkWidget-window-dragging: true;
}

.heading.h4 {
  color: @window_fg_color;
}

window.csd.unified list.content spinbutton {
  border: none;
}
window.csd.unified list.content spinbutton:focus {
  box-shadow: inset 0 0 0 2px alpha(@accent_color,0.5);
}

window.csd.unified .titlebar {
  border-bottom: 0;
  box-shadow: none;
}

window.csd.unified .sidebar separator {
  min-height: 0;
}
window.csd.unified .sidebar list row:backdrop:hover {
  background-color: alpha(currentColor,0.07);
}

window.rounded,
window.rounded actionbar {
  border-radius: 0 0 15px 15px;
}
window.rounded decoration,
window.rounded actionbar decoration {
  border-radius: 15px;
}

window.flat headerbar {
  box-shadow: none;
}

.accent {
  color: @accent_bg_color;
}

.h1 {
  font-size: 20pt;
  font-weight: 300;
}

.h2 {
  font-size: 16pt;
  font-weight: 200;
}

.h3 {
  font-size: 11pt;
}

.h4,
.category-label {
  color: shade(@window_fg_color,1.2);
  font-weight: 700;
}

.h4 {
  padding-top: 0.5em;
  padding-bottom: 0.5em;
}

list .h4 {
  padding-left: 0.5em;
}

.storage-bar .trough {
  padding: 8px 6px;
  border: none;
  border-radius: 9px 9px 0 0;
  background-image: none;
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.03), 0 1px 3px 1px rgba(0, 0, 0, 0.07), 0 2px 6px 2px rgba(0, 0, 0, 0.03);
}
.storage-bar .fill-block {
  padding: 8px 6px;
  border: 1px solid rgba(0, 0, 0, 0.35);
  border-top-width: 0;
  border-right-width: 0;
  border-bottom-width: 1px;
  border-left-width: 0;
  border-radius: 0;
  border-bottom-color: shade(mix(currentColor,@window_bg_color,0.85),0.75);
  background-color: shade(@view_bg_color,0.85);
}
.storage-bar .fill-block:first-child {
  border-radius: 9px 0 0;
}
.storage-bar .fill-block:last-child {
  border-radius: 0 9px 0 0;
}
.storage-bar .fill-block:only-child {
  border-radius: 9px 9px 0 0;
}
.storage-bar .fill-block image {
  -gtk-icon-style: symbolic;
}
.storage-bar .empty-block {
  border-bottom-width: 1px;
  border-bottom-color: shade(mix(currentColor,@window_bg_color,0.85),0.6);
  background-color: shade(@view_bg_color,0.95);
}
.storage-bar .empty-block image {
  color: black;
}
.storage-bar .app {
  border-bottom-width: 1px;
  border-bottom-color: shade(#dc8add,0.7);
  background-color: #dc8add;
}
.storage-bar .app image {
  color: white;
}
.storage-bar .audio {
  border-bottom-width: 1px;
  border-bottom-color: shade(#ffa348,0.8);
  background-color: #ffa348;
}
.storage-bar .audio image {
  color: black;
}
.storage-bar .files {
  border-bottom-width: 1px;
  border-bottom-color: shade(#99c1f1,0.8);
  background-color: #99c1f1;
}
.storage-bar .files image {
  color: black;
}
.storage-bar .photo {
  border-bottom-width: 1px;
  border-bottom-color: shade(#57e389,0.7);
  background-color: #57e389;
}
.storage-bar .photo image {
  color: black;
}
.storage-bar .video {
  border-bottom-width: 1px;
  border-bottom-color: shade(#f66151,0.8);
  background-color: #f66151;
}
.storage-bar .video image {
  color: white;
}
.storage-bar .legend {
  padding: 8px;
  border-radius: 50%;
}
.storage-bar .legend image {
  color: black;
}
.storage-bar .disk-bar {
  padding: 0;
  border-radius: 9px;
  background-color: #cdab8f;
}
.storage-bar .ext2, .storage-bar .ext3, .storage-bar .ext4, .storage-bar .fat16, .storage-bar .fat32, .storage-bar .btrfs,
.storage-bar .xfs, .storage-bar .ntfs, .storage-bar .luks, .storage-bar .lvm, .storage-bar .none, .storage-bar .swap, .storage-bar .unused {
  border: none;
  box-shadow: inset 0 -2px rgba(94, 92, 100, 0.5), inset 1px 0 rgba(94, 92, 100, 0.5), inset -1px 0 rgba(94, 92, 100, 0.5);
}
.storage-bar .swap {
  background-color: #a51d2d;
}
.storage-bar .swap image {
  color: white;
}
.storage-bar .ext4 {
  background-color: #57e389;
}
.storage-bar .ext4 image {
  color: black;
}
.storage-bar .ext3 {
  background-color: #26a269;
}
.storage-bar .ext3 image {
  color: white;
}
.storage-bar .ext2 {
  background-color: #8ff0a4;
}
.storage-bar .ext2 image {
  color: black;
}
.storage-bar .fat16,
.storage-bar .fat32 {
  background-color: #f8e45c;
}
.storage-bar .fat16 image,
.storage-bar .fat32 image {
  color: black;
}
.storage-bar .btrfs {
  background-color: #1a5fb4;
}
.storage-bar .btrfs image {
  color: white;
}
.storage-bar .xfs {
  background-color: #99c1f1;
}
.storage-bar .xfs image {
  color: black;
}
.storage-bar .ntfs {
  background-color: #ffa348;
}
.storage-bar .ntfs image {
  color: black;
}
.storage-bar .luks {
  background-color: #c061cb;
}
.storage-bar .luks image {
  color: black;
}
.storage-bar .lvm {
  background-color: #dc8add;
}
.storage-bar .lvm image {
  color: black;
}
.storage-bar .none {
  background-color: #99c1f1;
}
.storage-bar .none image {
  color: black;
}
.storage-bar .unused {
  background-color: #cdab8f;
}
.storage-bar .unused image {
  color: black;
}
.storage-bar .legend {
  box-shadow: none;
}

GraniteWidgetsPopOver {
  -GraniteWidgetsPopOver-arrow-width: 21;
  -GraniteWidgetsPopOver-arrow-height: 10;
  -GraniteWidgetsPopOver-border-radius: 15px;
  -GraniteWidgetsPopOver-border-width: 0;
  -GraniteWidgetsPopOver-shadow-size: 12;
  border: 1px solid @view_bg_color;
  background: @view_bg_color;
  color: @window_fg_color;
}
GraniteWidgetsPopOver .button {
  background-image: none;
  background: none;
  border: none;
}
GraniteWidgetsPopOver .button:active, GraniteWidgetsPopOver .button:active:hover {
  color: @accent_bg_color;
}
GraniteWidgetsPopOver > .frame {
  border: none;
}
GraniteWidgetsPopOver .sidebar.view {
  border: none;
  background: none;
}

GraniteWidgetsStaticNotebook .frame {
  border: none;
}

.popover_bg {
  background-color: @view_bg_color;
  background-image: none;
  border: 1px solid @view_bg_color;
  color: @window_fg_color;
}

.deck {
  background-color: @view_bg_color;
}

paper,
.card {
  transition: all 300ms cubic-bezier(0.25, 0.8, 0.25, 1);
  border: none;
  background-color: @view_bg_color;
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.03), 0 1px 3px 1px rgba(0, 0, 0, 0.07), 0 2px 6px 2px rgba(0, 0, 0, 0.03);
}
paper, paper.rounded,
.card,
.card.rounded {
  border-radius: 12px;
}
paper.collapsed,
.card.collapsed {
  background-color: @window_bg_color;
}

.source-list {
  -GtkTreeView-horizontal-separator: 1px;
  -GtkTreeView-vertical-separator: 6px;
  background-color: @window_bg_color;
  border: solid mix(currentColor,@window_bg_color,0.85);
  color: @window_fg_color;
  border-right-width: 1px;
}
.source-list .category-expander {
  color: transparent;
}
.source-list .badge {
  background-image: none;
  background-color: rgba(0, 0, 0, 0.4);
  color: @window_bg_color;
  border-radius: 10px;
  padding: 0 6px;
  margin: 0 3px;
  border-width: 0;
}
.source-list .badge:selected:backdrop, .source-list .badge:selected:hover:backdrop {
  background-color: rgba(0, 0, 0, 0.2);
  color: shade(@window_bg_color,0.95);
}
.source-list row,
.source-list .list-row {
  border: none;
  padding: 0;
}
.source-list row > GtkLabel,
.source-list row > label,
.source-list .list-row > GtkLabel,
.source-list .list-row > label {
  padding-left: 6px;
  padding-right: 6px;
}

.avatar {
  border-radius: 999px;
  box-shadow: none;
}

.overlay-bar {
  padding: 4px;
}

.dynamic-notebook tab.reorderable-page {
  padding: 8px;
}

scale.temperature trough {
  background-image: linear-gradient(to right, rgba(153, 193, 241, 0.4), #c0bfbc, #f9f06b);
}
scale.temperature:dir(rtl) trough {
  background-image: linear-gradient(to left, rgba(153, 193, 241, 0.4), #c0bfbc, #f9f06b);
}
scale.warmth trough {
  background-image: linear-gradient(to right, rgba(249, 240, 107, 0.4), rgba(229, 165, 10, 0.6));
}
scale.warmth:dir(rtl) trough {
  background-image: linear-gradient(to left, rgba(249, 240, 107, 0.4), rgba(229, 165, 10, 0.6));
}

.terminal,
.terminal text {
  background-color: #3d3846;
  color: white;
  font-family: monospace;
}
.terminal selection,
.terminal text selection {
  background-color: @accent_bg_color;
  color: white;
}
.terminal:backdrop,
.terminal text:backdrop {
  background-color: #5e5c64;
  color: white;
}

label.terminal {
  padding: 1em;
}

.welcome {
  font-size: 10pt;
  text-shadow: none;
}
.welcome .h1, .welcome .h3 {
  color: shade(@window_fg_color,1.2);
}

button.back-button,
button.back-button.text-button {
  padding-right: 8px;
  padding-left: 36px;
  transition: background 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94), box-shadow 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  background-image: -gtk-icontheme("go-previous-symbolic");
  background-repeat: no-repeat no-repeat;
  background-position: 5px 50%;
  background-size: 21px;
}
.titlebar button.back-button,
.titlebar button.back-button.text-button {
  background-color: @headerbar_bg_color;
  background-image: -gtk-icontheme("go-previous-symbolic");
  background-repeat: no-repeat no-repeat;
  background-position: 5px 50%;
  background-size: 21px;
  padding-left: 36px;
  color: @headerbar_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.9);
}
.titlebar button.back-button:hover,
.titlebar button.back-button.text-button:hover {
  background-image: -gtk-icontheme("go-previous-symbolic");
  background-repeat: no-repeat no-repeat;
  background-position: 5px 50%;
  background-size: 21px;
  color: @headerbar_fg_color;
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.85);
  box-shadow: none;
}
.titlebar button.back-button:active,
.titlebar button.back-button.text-button:active {
  background-image: -gtk-icontheme("go-previous-symbolic");
  background-repeat: no-repeat no-repeat;
  background-position: 5px 50%;
  background-size: 21px;
  color: @headerbar_fg_color;
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.titlebar button.back-button:active:hover,
.titlebar button.back-button.text-button:active:hover {
  background-color: mix(@headerbar_fg_color,@headerbar_bg_color,0.65);
}
button.back-button:dir(rtl),
button.back-button.text-button:dir(rtl) {
  padding: 0.5em 0.6em;
  padding-right: 36px;
  padding-left: 8px;
  background-image: -gtk-icontheme("go-next-symbolic");
  background-repeat: no-repeat no-repeat;
  background-position: 90% 50%;
}

.checkerboard {
  box-shadow: 0px 1px 2px 0px alpha(mix(currentColor,@window_bg_color,0.85),0.5), 0px 0px 0px 1px alpha(mix(currentColor,@window_bg_color,0.85),0.5), 0px 2px 0px 0px alpha(mix(currentColor,@window_bg_color,0.85),0.5);
  border-radius: 2px;
}

.org-gnome-gedit notebook > header, .gedit-side-panel-paned notebook > header {
  background-color: @headerbar_bg_color;
}
.org-gnome-gedit notebook > header:backdrop, .gedit-side-panel-paned notebook > header:backdrop {
  background-color: @headerbar_backdrop_color;
}
.org-gnome-gedit notebook > header, .gedit-side-panel-paned notebook > header {
  box-shadow: inset 0 -1px @headerbar_shade_color;
}

.gedit-search-slider {
  background: @window_bg_color;
  border: 1px solid mix(currentColor,@window_bg_color,0.85);
  border-top-style: none;
  padding: 4px 8px;
  border-radius: 0 0 9px 9px;
}
.gedit-search-slider button {
  padding-top: 7px;
  padding-bottom: 7px;
}

.gedit-document-panel {
  background-color: @window_bg_color;
}
.gedit-document-panel row.activatable {
  padding: 6px;
}
.gedit-document-panel row button {
  min-width: 22px;
  min-height: 22px;
  padding: 0;
  margin: 0;
  border: none;
}
.gedit-document-panel row:hover button:hover {
  background-color: alpha(currentColor,0.15);
}
.gedit-document-panel row:hover button:active {
  background-color: alpha(currentColor,0.3);
}
.gedit-document-panel row:hover:selected button:hover {
  color: @window_fg_color;
}

.XfceHeading {
  background-color: @view_bg_color;
  margin: 0;
  padding: 0;
  border-width: 0;
}

XfdesktopIconView.view {
  background: transparent;
  color: rgba(255, 255, 255, 0.9);
  border-radius: 9px;
}
XfdesktopIconView.view:active {
  background: alpha(mix(black,@accent_bg_color,0.85),0.5);
  text-shadow: 0 1px 1px black;
}
XfdesktopIconView.view .label {
  text-shadow: 1px 1px 2px black;
}
XfdesktopIconView.view .rubberband {
  border-radius: 0;
}

.xfce4-panel.background {
  border: none;
  background-color: @panel_bg_color;
  color: @panel_fg_color;
  transition-duration: 250ms;
}
.xfce4-panel.background .tasklist button,
.xfce4-panel.background #actions-button,
.xfce4-panel.background #clock-button {
  margin: 0 1px;
}
.xfce4-panel.background.horizontal .tasklist button {
  margin: 0 1px;
}
.xfce4-panel.background.horizontal .tasklist button label {
  padding-right: 3px;
}
.xfce4-panel.background.vertical .tasklist button {
  margin: 1px 0;
}
.xfce4-panel.background.vertical .tasklist button label {
  padding-bottom: 3px;
}
.xfce4-panel.background .tasklist image {
  margin-left: 2px;
  margin-right: 2px;
}
.xfce4-panel.background button label {
  padding-left: 6px;
  padding-right: 6px;
}
.xfce4-panel.background button {
  padding: 0 6px;
  min-height: 12px;
  min-width: 12px;
  border-radius: 24px;
  color: @panel_fg_color;
  transition-duration: 250ms;
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
}
.xfce4-panel.background button:disabled {
  background-color: transparent;
  color: alpha(@panel_fg_color,0.2);
}
.xfce4-panel.background button:hover:not(:active):not(:checked) {
  color: @panel_fg_color;
  background-color: alpha(@panel_fg_color,0.25);
  background-image: none;
}
.xfce4-panel.background button:checked {
  background-color: alpha(@panel_fg_color,0.2);
  color: @panel_fg_color;
}
.xfce4-panel.background button:checked:hover {
  background-color: alpha(@panel_fg_color,0.35);
}
.xfce4-panel.background button:checked:disabled {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: inset 0 1px rgba(255, 255, 255, 0);
  text-shadow: none;
  -gtk-icon-shadow: none;
  color: alpha(@panel_fg_color,0.2);
}
.xfce4-panel.background button:drop(active) {
  color: @accent_bg_color;
  border-color: @accent_bg_color;
  box-shadow: inset 0 0 0 1px @accent_bg_color;
}
.xfce4-panel.background button entry {
  caret-color: @view_fg_color;
}
.xfce4-panel.background button menu {
  -gtk-icon-effect: none;
  text-shadow: none;
}
.xfce4-panel.background button menu .linked button {
  color: @view_fg_color;
  outline-color: alpha(@accent_color,0.5);
  background-color: mix(@view_fg_color,@popover_bg_color,0.9);
}
.xfce4-panel.background button menu .linked button:disabled {
  color: alpha(@window_fg_color,0.5);
  background-color: mix(@window_fg_color,@window_bg_color,0.95);
  text-shadow: none;
  -gtk-icon-shadow: none;
  box-shadow: none;
  border-color: alpha(mix(currentColor,@window_bg_color,0.85),0.5);
}
.xfce4-panel.background progressbar {
  margin: 0;
}
.xfce4-panel.background progressbar.horizontal trough {
  min-height: 6px;
  padding: 0;
}
.xfce4-panel.background progressbar.horizontal progress {
  min-height: 4px;
}
.xfce4-panel.background progressbar.vertical trough {
  min-width: 6px;
  padding: 0;
}
.xfce4-panel.background progressbar.vertical progress {
  min-width: 4px;
}
.xfce4-panel.background progressbar progress {
  border-radius: 0px;
}
.xfce4-panel.background progressbar trough {
  background: shade(@panel_bg_color,0.7);
  border-radius: 0px;
  border-color: shade(@panel_bg_color,0.4);
}

wnck-pager {
  background-color: alpha(@panel_fg_color,0.1);
}
wnck-pager:selected {
  background-color: shade(@accent_bg_color,0.88);
}
wnck-pager:hover {
  background-color: @accent_bg_color;
}

#xfwm-tabwin {
  padding: 12px;
  border-radius: 15px;
  -XfwmTabwinWidget-icon-size: 64px;
  -XfwmTabwinWidget-preview-size: 128px;
}
#xfwm-tabwin .tabwin-app-grid button {
  min-width: 96px;
  min-height: 96px;
}

window#whiskermenu-window button {
  transition: none;
}

#whiskermenu-button {
  transition: none;
}

#XfceNotifyWindow {
  border-radius: 9px;
  background-color: @view_bg_color;
}

.caja-desktop.caja-canvas-item,
.nemo-desktop.nemo-canvas-item {
  color: @accent_fg_color;
  text-shadow: 1px 1px 2px rgba(0, 0, 0, 0.8);
}

.nemo-properties-dialog .dialog-action-box .dialog-action-area {
  margin: 2px 6px 4px;
}
.nemo-properties-dialog toolbar stackswitcher.linked button {
  margin-right: 0;
}
.nemo-properties-dialog toolbar stackswitcher.linked button:backdrop {
  color: mix(@window_fg_color,@window_bg_color,0.5);
  background-color: @window_bg_color;
}

.nemo-window .sidebar scrolledwindow.frame.nemo-places-sidebar {
  border: none;
}
.nemo-window .sidebar viewport.frame box.vertical treeview.view.places-treeview {
  background-image: image(@window_bg_color);
  border-radius: 0;
}
.nemo-window .sidebar viewport.frame box.vertical treeview.view.places-treeview:selected {
  background: @accent_bg_color;
}
.nemo-window .sidebar viewport.frame box.vertical treeview.view.places-treeview:not(:selected):not(:backdrop):hover {
  background-image: image(alpha(@window_fg_color,0.05));
}
.nemo-window .nemo-inactive-pane .view,
.nemo-window .nemo-inactive-pane iconview {
  background-color: @window_bg_color;
}
.nemo-window .nemo-window-pane widget.entry {
  border: 1px solid;
  border-radius: 3px;
  color: @window_fg_color;
  border-color: @accent_bg_color;
  background-color: @window_bg_color;
}
.nemo-window .nemo-window-pane widget.entry:selected {
  border: 1px solid;
  border-radius: 3px;
  color: @accent_fg_color;
  border-color: @accent_bg_color;
  background-color: @accent_bg_color;
}
.nemo-window toolbar.horizontal.primary-toolbar {
  border-bottom: 1px solid mix(currentColor,@window_bg_color,0.85);
}
.nemo-window toolbar.horizontal.primary-toolbar widget.linked.raised button:first-child widget {
  -gtk-icon-source: -gtk-icontheme("go-previous-symbolic");
}
.nemo-window toolbar.horizontal.primary-toolbar widget.linked.raised button:last-child widget {
  -gtk-icon-source: -gtk-icontheme("go-next-symbolic");
}
.nemo-window toolbar.horizontal.primary-toolbar toolitem box widget * {
  min-height: 0;
  min-width: 0;
}
.nemo-window toolbar.horizontal.primary-toolbar .linked button {
  margin-right: 0;
}

#MozillaGtkWidget.background selection:focus, #MozillaGtkWidget.background selection {
  background-color: @accent_bg_color;
  color: @accent_fg_color;
}

terminal-window notebook > header.top tabs:not(:only-child):first-child, terminal-window notebook > header.bottom tabs:not(:only-child):first-child {
  margin-left: 0px;
}
terminal-window notebook > header.top tabs:not(:only-child):last-child, terminal-window notebook > header.bottom tabs:not(:only-child):last-child {
  margin-right: 0px;
}

terminal-window notebook scrollbar slider {
  margin: 0;
  border-width: 3px;
}

window.background.caja-navigation-window #Toolbar {
  border-bottom: 1px solid mix(currentColor,@window_bg_color,0.85);
}
window.background.caja-navigation-window #Toolbar:backdrop {
  background-color: @window_bg_color;
  border-color: mix(currentColor,@window_bg_color,0.81);
}

.caja-pathbar button {
  min-width: 32px;
  border-radius: 0;
  border-right-width: 0;
  margin-right: -3px;
}
.caja-pathbar button.slider-button:nth-child(odd) {
  border-radius: 9px 0px 0px 9px;
}
.caja-pathbar button.slider-button:nth-child(even) {
  border-radius: 0 9px 9px 0;
  border-right-width: 1px;
}

.caja-side-pane .frame {
  border-style: solid none none none;
}

.caja-notebook .view, .caja-notebook .view.frame, .caja-notebook .frame {
  border-left: 0;
  border-right: 0;
}

.caja-side-pane + separator + box:backdrop {
  background: @window_bg_color;
}

.caja-navigation-window widget.view widget.entry,
.caja-desktop-window widget.view widget.entry {
  background: mix(currentColor,@window_bg_color,0.95);
  color: @window_fg_color;
  border-color: mix(currentColor,@window_bg_color,0.85);
}
.caja-navigation-window widget.view widget.entry:selected,
.caja-desktop-window widget.view widget.entry:selected {
  background: @accent_bg_color;
  color: @accent_fg_color;
}

#gnc-id-main-window treeview.view:not(:selected):hover {
  background-color: mix(currentColor,@view_bg_color,0.93);
}

gnc-id-sheet-list {
  background-color: @window_bg_color;
}

geary-conversation-viewer#conversation_viewer list.background.conversation-listbox.content {
  background-color: @window_bg_color;
  border: none;
  border-radius: 0;
}
geary-conversation-viewer#conversation_viewer list.background.conversation-listbox.content > row.activatable {
  border: 1px solid mix(currentColor,@window_bg_color,0.85);
  border-bottom-width: 0;
}
geary-conversation-viewer#conversation_viewer list.background.conversation-listbox.content > row.activatable:first-child {
  border-top-left-radius: 12px;
  border-top-right-radius: 12px;
}
geary-conversation-viewer#conversation_viewer list.background.conversation-listbox.content .geary-attachment-pane {
  border-radius: 0 0 12px 12px;
}
geary-conversation-viewer#conversation_viewer list.background.conversation-listbox.content .geary-attachment-pane actionbar.background {
  background-color: transparent;
}
geary-conversation-viewer#conversation_viewer list.background.conversation-listbox.content .geary-attachment-pane actionbar.background > revealer > box {
  border-radius: 0 0 12px 12px;
}
geary-conversation-viewer#conversation_viewer list.background.conversation-listbox.content geary-composer-editor > box.background {
  background-color: transparent;
}
geary-conversation-viewer#conversation_viewer list.background.conversation-listbox.content geary-composer-editor > box.background > actionbar > revealer > box {
  border-radius: 0 0 12px 12px;
}
geary-conversation-viewer#conversation_viewer searchbar > revealer > box {
  background-color: @view_bg_color;
  border-color: mix(currentColor,@window_bg_color,0.85);
}

.geary-folder-popover-list-row {
  border-radius: 0;
  margin: 0;
}

.geary-main-window leaflet > leaflet > box:first-child {
  background-color: @sidebar_bg_color;
}
.geary-main-window leaflet > leaflet > box:first-child:backdrop {
  background-color: @sidebar_backdrop_color;
}

.geary-main-window leaflet > box:first-child > .titlebar {
  background-color: @sidebar_bg_color;
  background-image: none;
  border-bottom: transparent;
  box-shadow: none;
}
.geary-main-window leaflet > box:first-child > .titlebar:backdrop {
  background-color: @sidebar_backdrop_color;
}
.geary-main-window leaflet > box:first-child > .titlebar button {
  background-color: transparent;
}
.geary-main-window leaflet > box:first-child > .titlebar button:hover {
  background-color: alpha(currentColor,0.15);
}
.geary-main-window leaflet > box:first-child > .titlebar button:hover:active, .geary-main-window leaflet > box:first-child > .titlebar button:active, .geary-main-window leaflet > box:first-child > .titlebar button:hover:checked, .geary-main-window leaflet > box:first-child > .titlebar button:checked {
  background-color: alpha(currentColor,0.3);
}
.geary-main-window leaflet > box:first-child > .titlebar button:backdrop {
  background: transparent;
}
.geary-main-window leaflet > box:first-child > .titlebar button:backdrop:hover {
  background: alpha(currentColor,0.15);
}
.geary-main-window leaflet > box:not(:first-child) > .titlebar {
  border-bottom: transparent;
  box-shadow: none;
}
.geary-main-window leaflet > box:not(:first-child) > .titlebar:backdrop {
  background-image: none;
  background-color: @headerbar_bg_color;
}

.geary-main-window separator.sidebar {
  background-color: mix(currentColor,@window_bg_color,0.85);
}

.geary-main-window .titlebar .image-button {
  padding: 0 6px;
}

.geary-folder .sidebar.view:selected {
  background-color: alpha(currentColor,0.1);
}
.geary-folder .sidebar.view:selected:hover {
  background-color: alpha(currentColor,0.12);
}
.geary-folder .sidebar.view:hover {
  background-color: alpha(currentColor,0.07);
}

#DialogNotebook treeview.view:hover {
  background-color: alpha(@accent_bg_color,0.5);
}

.budgie-panel button {
  min-height: 16px;
  min-width: 16px;
  padding: 0;
  border-radius: 0;
}
.budgie-panel button.flat.launcher {
  padding: 0;
}
.budgie-panel.horizontal button,
.budgie-panel #tasklist-button {
  padding: 0 4px;
}
.budgie-panel.vertical button {
  padding: 4px 0;
}
.budgie-panel.vertical #tasklist-button {
  min-height: 32px;
}

toolbar.toolbar box:not(.linked) > button {
  font-weight: normal;
}

window.background:not(.solid-csd):not(.csd) > notebook > header {
  border: 0;
}
window.background:not(.solid-csd):not(.csd) > notebook > header tab {
  margin: 0 1px;
  border-radius: 9px;
  box-shadow: none;
}
window.background:not(.solid-csd):not(.csd) > notebook > header tab:hover {
  background-color: alpha(currentColor,0.07);
}
window.background:not(.solid-csd):not(.csd) > notebook > header tab:active, window.background:not(.solid-csd):not(.csd) > notebook > header tab:checked {
  background-color: alpha(currentColor,0.1);
}
window.background:not(.solid-csd):not(.csd) > notebook > header tab:checked:hover {
  background-color: alpha(currentColor,0.12);
}

window.background:not(.solid-csd):not(.csd) > notebook > stack:not(:only-child) {
  background-color: @window_bg_color;
}

#settingsDialog .dialog-vbox.vertical tab {
  box-shadow: none;
  margin-right: 8px;
  border-radius: 9px;
}
#settingsDialog .dialog-vbox.vertical tab:hover {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.85);
  box-shadow: none;
}
#settingsDialog .dialog-vbox.vertical tab:checked {
  color: @window_fg_color;
  background-color: mix(@window_fg_color,@window_bg_color,0.7);
  box-shadow: none;
  text-shadow: none;
  -gtk-icon-shadow: none;
}
#settingsDialog .dialog-vbox.vertical tab:checked:hover {
  background-color: mix(@window_fg_color,@window_bg_color,0.65);
}

/* Matugen Colors */

@define-color accent_color {{colors.primary_fixed_dim.default.hex}};
@define-color accent_fg_color {{colors.on_primary_fixed.default.hex}};
@define-color accent_bg_color {{colors.primary_fixed_dim.default.hex}};
@define-color window_bg_color {{colors.background.default.hex}};
@define-color window_fg_color {{colors.on_surface.default.hex}};
@define-color headerbar_bg_color {{colors.background.default.hex}};
@define-color headerbar_fg_color {{colors.on_surface.default.hex}};
@define-color popover_bg_color {{colors.background.default.hex}};
@define-color popover_fg_color {{colors.on_surface.default.hex}};
@define-color view_bg_color {{colors.background.default.hex}};
@define-color view_fg_color {{colors.on_surface.default.hex}};
@define-color card_bg_color {{colors.background.default.hex}};
@define-color card_fg_color {{colors.on_surface.default.hex}};
@define-color sidebar_bg_color @window_bg_color;
@define-color sidebar_fg_color @window_fg_color;
@define-color sidebar_border_color @window_bg_color;
@define-color sidebar_backdrop_color @window_bg_color;
